audio = {
	loadedsongs = {},
	loadedsfx = {},
	activesong = nil,
	loopingsfx = {},
	activesongoneshot = false,
	musicvolume = 1,
	sfxvolume = 1,
	lowpass = {
		type = 'lowpass',
		volume = 0.5,
		highgain = 0.025
	},
	--oncesfx = {},
	playedsfx = {},
	currentpitch = 1,
	currentvolume = 1, --multiplier on top of sfxvolume
}

function audio.loadsong(filepath)
	local name = nil
	if audio.loadedsongs[name] then
		print("audio.loadsong: already loaded " .. name)
		return
	end
	local src = love.audio.newSource(filepath, "stream")
	
end

function audio.stopsong()
	if audio.activesong ~= nil then
		song = audio.loadedsongs[audio.activesong]
		if audio.activesongoneshot then
			song:stop()
		else
			song.loop_playing = false
			if song.intro ~= nil then song.intro:stop() end
			song.loop:stop()
		end
	end
	audio.activesong = nil
	audio.activesongoneshot = false
end

function audio.pausesong()
	if audio.activesong ~= nil then
		song = audio.loadedsongs[audio.activesong]
		if audio.activesongoneshot then
			song:pause()
		else
			--preserve song.loop_playing
			if song.intro ~= nil then song.intro:pause() end
			song.loop:pause()
		end
	end
end

function audio.flushpauses()
	for _,song in pairs(audio.loadedsongs) do
		if song.intro or song.loop then
			if song.intro then song.intro:seek(0) end
			song.loop:seek(0)
			song.loop_playing = false
		else song:seek(0) end
	end
end

--gather sfx. will later also be used for music
do
	print("loading sfx...")
	local audiofiles = love.filesystem.getDirectoryItems("audial/sfx")
	for i,v in ipairs(audiofiles) do
		local last_dot_index = string.find(v, "%.[^%.]*$")
		if last_dot_index then --todo: deal with directories that have . in them. this is only going to be a problem down the line once user input is dealt with
			local name = string.sub(v, 1, last_dot_index - 1)
			if audio.loadedsfx[name] then
				error("name " .. tostring(name) .. " used twice")
			end
			audio.loadedsfx[name] = love.audio.newSource("audial/sfx/" .. v, "static")
		end
	end
	print("loading sfx done")
end

function audio.playsong(songname, oneshot, pauseold, dotmod)
	if (songname) and (songname ~= "") and (songname ~= " ") then
		--is specifying it's a oneshot necessary? should this just be able to determine whether it's a oneshot on its own
		if pauseold then
			audio.pausesong()
		else
			audio.stopsong()
		end
		if audio.loadedsongs[songname] == nil then
			if dotmod then
				--everything about audio is dreadful to look at rn and badly needs a rewrite
				if not oneshot then
					audio.loadedsongs[songname] = {
						loop_playing = false,
						loop = love.audio.newSource("audial/music/" .. songname .. ".mod", "stream") --kludge ass shit
					}
				else
					audio.loadedsongs[songname] = love.audio.newSource("audial/music/" .. songname .. ".mod", "stream")
				end
			else
				if not oneshot then
					audio.loadedsongs[songname] = {
						loop_playing = false, --initially this was for a dumb kludge to play the loop again when it wasn't already playing because i had no idea :setLooping() was a thing. i have no idea why i kept this. helps with pause() a bit i guess
						loop = love.audio.newSource("audial/music/" .. songname .. " loop.ogg", "stream")
					}
					if love.filesystem.getInfo("audial/music/" .. songname .. " intro.ogg") then
						audio.loadedsongs[songname].intro = love.audio.newSource("audial/music/" .. songname .. " intro.ogg", "stream")
					end
				else
					audio.loadedsongs[songname] = love.audio.newSource("audial/music/" .. songname .. ".ogg", "stream")
				end
			end
		end
		song = audio.loadedsongs[songname]
		if oneshot then
			song:setVolume(audio.musicvolume)
			love.audio.play(song)
			audio.activesongoneshot = true
		else
			if song.intro ~= nil then
				song.intro:setVolume(audio.musicvolume)
			end
			song.loop:setVolume(audio.musicvolume)
			if song.intro ~= nil and not song.loop_playing then
				love.audio.play(song.intro)
			else
				song.loop:setLooping(true)
				love.audio.play(song.loop)
				song.loop_playing = true
			end
			audio.activesongoneshot = false
		end
		audio.activesong = songname
		print("now playing: " .. audio.activesong)
	end
end

function audio.loadsfx(soundname)
	--todos
end

function audio.setpitch(pitch)
	audio.currentpitch = pitch
end

function audio.timesplayed(soundname)
	--times played this frame
	return audio.playedsfx[soundname] or 0
end

function audio.playsfx(soundname, looping)
	--Source:clone() is way faster than love.audio.newSource() so potentially the source could be cloned and put in some weak table if it needs to be played twice without just restarting.
	if not audio.playedsfx[soundname] then
		local sfx = audio.loadedsfx[soundname]
		sfx:setVolume(audio.sfxvolume * audio.currentvolume)
		sfx:setPitch(audio.currentpitch)
		if looping then
			sfx:setLooping(true)
			table.insert(audio.loopingsfx, sfx)
		end
		sfx:seek(0)
		sfx:play()
		audio.playedsfx[soundname] = (audio.playedsfx[soundname] or 0) + 1
	end
	audio.setpitch(1) --set pitch back automatically when a sfx tries to play since most pitch-shifts are just for one sfx
end

function audio.playsfxonce(soundname) --deprecated. playing sfx not once could be brought back if e.g. i or a user wants to play the same sound multiple times on the same frame at different pitches - but that would require letting sounds play over themselves which i took out for now lol.
	audio.playsfx(soundname)
	--if not audio.oncesfx[soundname] then
	--audio.playedsfx[soundname] = (audio.playedsfx[soundname] or 0) + 1
	--audio.oncesfx[soundname] = true
--	if not audio.playedsfx[soundname] then
--		audio.playsfx(soundname)
--	end
end

function audio.stoploopingsfx(soundname)
	if audio.loopingsfx[soundname] then audio.loopingsfx[soundname]:stop() end
	audio.loopingsfx[soundname] = nil
end

function audio.stoploopingsfxall()
	for _,sound in pairs(audio.loopingsfx) do
		sound:stop()
	end
	audio.loopingsfx = {}
end

function audio.changemusicvolume(vol)
	audio.musicvolume = vol
	if audio.activesong ~= nil and audio.loadedsongs[audio.activesong] ~= nil then
		song = audio.loadedsongs[audio.activesong]
		if audio.activesongoneshot then
			song:setVolume(vol)
		else
			if song.intro ~= nil then song.intro:setVolume(vol) end
			song.loop:setVolume(vol)
		end
	end
end

function audio.changesfxvolume(vol)
	audio.sfxvolume = vol
end

function audio.update()
	if (not audio.activesongoneshot) and (audio.activesong ~= nil) then
		song = audio.loadedsongs[audio.activesong]
		if (not song.loop_playing) and (not song.loop:isPlaying()) and (song.intro ~= nil and not song.intro:isPlaying()) then
			song.loop:setLooping(true)
			love.audio.play(song.loop)
			song.loop_playing = true
		end
	end
	if audio.activesongoneshot and not audio.loadedsongs[audio.activesong]:isPlaying() then
		print("stopping " .. audio.activesong)
		audio.stopsong()
	end
	--[[
	local oncesfxnonempty = false
	for k,v in pairs(audio.oncesfx) do
		oncesfxnonempty = true
		if v then
			local sfx = love.audio.newSource("audial/sfx/" .. k .. ".ogg", "static")
			sfx:setVolume(audio.sfxvolume)
			love.audio.play(sfx)
		end
	end
	if oncesfxnonempty then audio.oncesfx = {} end
	]]
	if next(audio.playedsfx) then audio.playedsfx = {} end
	if audio.currentpitch ~= 1 then
		error("bleeding pitch (an object set sfx pitch to a non-1 value but it never got reset to 1)")
		--"bleeding pitch" is some surreal horror shit
	end
end