audio = {
	loadedsongs = {},
	songnames = {},
	customsongnames = {},
	loadedsfx = {},
	activesong = nil,
	loopingsfx = {},
	activesongoneshot = false,
	musicvolume = 1,
	sfxvolume = 1,
	lowpass = {
		type = 'lowpass',
		volume = 0.5,
		highgain = 0.025
	},
	--oncesfx = {},
	playedsfx = {},
	currentpitch = 1,
	currentvolume = 1, --multiplier on top of sfxvolume
}

function audio.loadsong(filepath)
	local name = nil
	if audio.loadedsongs[name] then
		print("audio.loadsong: already loaded " .. name)
		return
	end
	local src = love.audio.newSource(filepath, "stream")
	
end

function audio.stopsong()
	if audio.activesong ~= nil then
		song = audio.loadedsongs[audio.activesong]
		song:stop()
	end
	audio.activesong = nil
	audio.activesongoneshot = false
end

function audio.pausesong()
	if audio.activesong ~= nil then
		song = audio.loadedsongs[audio.activesong]
		song:pause()
	end
end

function audio.flushpauses()
	for _,song in pairs(audio.loadedsongs) do
		song:seek(0)
	end
end

--this should become a function at some point, with "audial" as an argument, so music/sfx from custom levels can be loaded in
do
	--gather sfx
	print("loading sfx...")
	local soundfiles = love.filesystem.getDirectoryItems("audial/sfx")
	for i,v in ipairs(soundfiles) do
		local last_dot_index = string.find(v, "%.[^%.]*$") or 0 --play sfx with no filetype, sure, why not. not like the world can get any crazier anyway!!!
		if love.filesystem.getInfo("audial/sfx/" .. v, "file") then
			local name = string.sub(v, 1, last_dot_index - 1) --if there was no dot and we set last_dot_index to 0, then last_dot_index - 1 will be -1, and 1,-1 as arguments to string.sub means "the entire string"
			if audio.loadedsfx[name] then
				error("sfx name " .. tostring(name) .. " used twice")
			end
			audio.loadedsfx[name] = love.audio.newSource("audial/sfx/" .. v, "static")
		end
	end
	print("loading sfx done")
	--gather music
	print("loading music filenames...")
	local musicfiles = love.filesystem.getDirectoryItems("audial/music")
	for i,v in ipairs(musicfiles) do
		if love.filesystem.getInfo("audial/music/" .. v, "file") then
			local last_dot_index = string.find(v, "%.[^%.]*$") or 0
			local name = string.sub(v, 1, last_dot_index - 1)
			if audio.songnames[name] then
				error("song name " .. tostring(name) .. " used twice")
			end
			audio.songnames[name] = v
		end
	end
	print("loading music filenames done")
end
function audio.fetchcustommusic(path)
	audio.customsongnames = {}
	if not path then return end
	if not love.filesystem.getInfo(path, "directory") then return end
	local musicfiles = love.filesystem.getDirectoryItems(path)
	for i,v in ipairs(musicfiles) do
		if love.filesystem.getInfo((path .. "/") .. v, "file") then
			local last_dot_index = string.find(v, "%.[^%.]*$") or 0
			local name = string.sub(v, 1, last_dot_index - 1)
			if audio.customsongnames[name] then
				error("custom song name " .. tostring(name) .. " used twice")
			end
			audio.customsongnames[name] = v
		end
	end
end

function audio.playsong(songname, oneshot, pauseold)
	if (songname) and (songname ~= "") and (songname ~= " ") then
		fullsongname = audio.customsongnames[songname] or audio.songnames[songname]
		if not fullsongname then print("invalid song requested for playsong: " .. songname); return end
		if pauseold then
			audio.pausesong()
		else
			audio.stopsong()
		end
		if audio.loadedsongs[songname] == nil then
			audio.loadedsongs[songname] = love.audio.newSource("audial/music/" .. fullsongname, "stream")
		end
		local song = audio.loadedsongs[songname]
		song:setVolume(audio.musicvolume)
		love.audio.play(song)
		if oneshot then
			audio.activesongoneshot = true
		else
			song:setLooping(true)
			audio.activesongoneshot = false
		end
		audio.activesong = songname
		print("now playing: " .. audio.activesong)
	end
end

function audio.setpitch(pitch)
	audio.currentpitch = pitch
end

function audio.timesplayed(soundname)
	--times played this frame
	return audio.playedsfx[soundname] or 0
end

function audio.playsfx(soundname, looping)
	--Source:clone() is way faster than love.audio.newSource() so potentially the source could be cloned and put in some weak table if it needs to be played twice without just restarting.
	if not audio.playedsfx[soundname] then
		local sfx = audio.loadedsfx[soundname]
		sfx:setVolume(audio.sfxvolume * audio.currentvolume)
		sfx:setPitch(audio.currentpitch)
		if looping then
			sfx:setLooping(true)
			table.insert(audio.loopingsfx, sfx)
		end
		sfx:seek(0)
		sfx:play()
		audio.playedsfx[soundname] = (audio.playedsfx[soundname] or 0) + 1
	end
	audio.setpitch(1) --set pitch back automatically when a sfx tries to play since most pitch-shifts are just for one sfx
end

function audio.playsfxonce(soundname) --deprecated. playing sfx not once could be brought back if e.g. i or a user wants to play the same sound multiple times on the same frame at different pitches - but that would require letting sounds play over themselves which i took out for now lol.
	audio.playsfx(soundname)
	--if not audio.oncesfx[soundname] then
	--audio.playedsfx[soundname] = (audio.playedsfx[soundname] or 0) + 1
	--audio.oncesfx[soundname] = true
--	if not audio.playedsfx[soundname] then
--		audio.playsfx(soundname)
--	end
end

function audio.stoploopingsfx(soundname)
	if audio.loopingsfx[soundname] then audio.loopingsfx[soundname]:stop() end
	audio.loopingsfx[soundname] = nil
end

function audio.stoploopingsfxall()
	for _,sound in pairs(audio.loopingsfx) do
		sound:stop()
	end
	audio.loopingsfx = {}
end

function audio.changemusicvolume(vol)
	audio.musicvolume = vol
	if audio.activesong ~= nil and audio.loadedsongs[audio.activesong] ~= nil then
		song = audio.loadedsongs[audio.activesong]
		song:setVolume(vol)
	end
end

function audio.changesfxvolume(vol)
	audio.sfxvolume = vol
end

function audio.update()
	if audio.activesongoneshot and not audio.loadedsongs[audio.activesong]:isPlaying() then
		print("stopping " .. audio.activesong)
		audio.stopsong()
	end
	--[[
	local oncesfxnonempty = false
	for k,v in pairs(audio.oncesfx) do
		oncesfxnonempty = true
		if v then
			local sfx = love.audio.newSource("audial/sfx/" .. k .. ".ogg", "static")
			sfx:setVolume(audio.sfxvolume)
			love.audio.play(sfx)
		end
	end
	if oncesfxnonempty then audio.oncesfx = {} end
	]]
	if next(audio.playedsfx) then audio.playedsfx = {} end
	if audio.currentpitch ~= 1 then
		error("bleeding pitch (an object set sfx pitch to a non-1 value but it never got reset to 1)")
		--"bleeding pitch" is some surreal horror shit
	end
end