tags = {}
local tags_img = graphics.load("ui/tags")
tags_img:setFilter("nearest")

for i=0, 9 do
	tags["spawn" .. i] = {
		name = "Spawn " .. i,
		tooltip = "apply to objects or tiles. this object/tile will only be present if the player entered from entrance #" .. i,
		kind = "any",
		quad = quad(i * 8, 0, 8, 8, tags_img)
	}
	if i == 0 then tags["spawn" .. i].rotations = {"spawn9", "spawn1"}
	elseif i == 9 then tags["spawn" .. i].rotations = {"spawn8", "spawn0"}
	else tags["spawn" .. i].rotations = {"spawn" .. (i - 1), "spawn" .. (i + 1)} end
end

for i=0, 9 do
	tags["variant" .. i] = {
		name = "Variant " .. i,
		tooltip = "apply to objects. use variant #" .. i .. " of this object. meaning varies per object, no effect on most.",
		kind = "object",
		quad = quad(i * 8, 8, 8, 8, tags_img)
	}
	if i == 0 then tags["variant" .. i].rotations = {"variant9", "variant1"}
	elseif i == 9 then tags["variant" .. i].rotations = {"variant8", "variant0"}
	else tags["variant" .. i].rotations = {"variant" .. (i - 1), "variant" .. (i + 1)} end
end

tags["nomoveuntilcollide"] = {
	name = "No Moving Until Collision",
	tooltip = "apply to an ogmo. it won't be able to move at the start of the level until it hits a solid tile",
	kind = "ogmo",
	quad = quad(0, 16, 8, 8, tags_img)
}

tags["preservecareen"] = {
	name = "Preserve Careening",
	tooltip = "apply to an ogmo. if last level was exited while careening, this ogmo will careen in the same direction. (use w/ preserve momentum!)",
	kind = "ogmo",
	quad = quad(8, 16, 8, 8, tags_img)
}

tags["preservejumpstate"] = {
	name = "Preserve Jump State",
	tooltip = "apply to an ogmo. if last level was exited with double-jump depleted, this ogmo starts with its double-jump depleted.",
	kind = "ogmo",
	quad = quad(0, 24, 8, 8, tags_img)
}

local dirs = {"up", "right", "down", "left"}
local dirs_titlecase = {"Up", "Right", "Down", "Left"}

for i,v in ipairs(dirs) do
	tags["edge" .. v] = {
		name = "Edge " .. dirs_titlecase[i],
		tooltip = "apply to objects. this object will act as though it is on the " .. v .. " border of the level.",
		kind = "object",
		quad = quad(16 + ((i - 1) * 8), 16, 8, 8, tags_img)
	}
	tags["face" .. v] = {
		name = "Face " .. dirs_titlecase[i] .. "ward",
		tooltip = "apply to objects. makes this object face " .. v .. "ward, if applicable.",
		kind = "object",
		quad = quad(16 + ((i - 1) * 8), 24, 8, 8, tags_img)
	}
end

tags["edgenone"] = {
	name = "Edge None",
	tooltip = "apply to objects. this object will act as though it is not on any border of the level, even if it is.",
	kind = "object",
	quad = quad(48, 16, 8, 8, tags_img)
}

tags["facerandom"] = {
	name = "Face Random",
	tooltip = "apply to objects. makes this object face in a random direction, if applicable.",
	kind = "object",
	quad = quad(48, 24, 8, 8, tags_img)
}

tags["edgeup"].rotations = {"edgenone", "edgeright"}
tags["edgeright"].rotations = {"edgeup", "edgedown"}
tags["edgedown"].rotations = {"edgeright", "edgeleft"}
tags["edgeleft"].rotations = {"edgedown", "edgenone"}
tags["edgenone"].rotations = {"edgeleft", "edgeup"}

tags["faceup"].rotations = {"facerandom", "faceright"}
tags["faceright"].rotations = {"faceup", "facedown"}
tags["facedown"].rotations = {"faceright", "faceleft"}
tags["faceleft"].rotations = {"facedown", "facerandom"}
tags["facerandom"].rotations = {"faceleft", "faceup"}

tags["invisible"] = {
	name = "Invisible",
	tooltip = "apply to objects or tiles. makes this object/tile invisible",
	kind = "any",
	quad = quad(56, 16, 8, 8, tags_img),
	rotations = {"fake", "silent"}
}

tags["fake"] = {
	name = "Fake",
	tooltip = "apply to objects or tiles. makes this object/tile unsolid and have no effect on anything else",
	kind = "any",
	quad = quad(64, 16, 8, 8, tags_img),
	rotations = {"silent", "invisible"}
}

tags["camerafocus"] = {
	name = "Camera Focus",
	tooltip = "apply to an ogmo. this ogmo will be prioritized for camera focus (if it spawns).",
	kind = "ogmo",
	quad = quad(72, 16, 8, 8, tags_img)
}

tags["nocutscenemovement"] = {
	name = "No Cutscene Movement",
	tooltip = "apply to an ogmo. no involuntary movement happens on level start. (on the bottom edge, ogmo falls immediately!)",
	kind = "ogmo",
	quad = quad(8, 24, 8, 8, tags_img)
}

tags["momentum"] = {
	name = "Preserve Momentum",
	tooltip = "apply to an ogmo. preserve momentum of exiting ogmo from previous level.",
	kind = "ogmo",
	quad = quad(56, 24, 8, 8, tags_img)
}

tags["winoffset"] = {
	name = "Preserve Win Offset",
	tooltip = "apply to an ogmo. ogmo will be offset depending on where exiting ogmo exited the previously level.",
	kind = "ogmo",
	quad = quad(64, 24, 8, 8, tags_img)
}

tags["fluid"] = {
	name = "Fluid Movement",
	tooltip = "apply to an ogmo. combination of several tags that providez manual flipscreen transitions.",
	--camerafocus, winoffset, momentum, nocutscenemovement, preservejumpstate, preservecareen
	kind = "ogmo",
	quad = quad(72, 24, 8, 8, tags_img)
}

rotationchain(tags, {"fluid", "camerafocus", "winoffset", "momentum", "nocutscenemovement", "preservejumpstate", "preservecareen"})

tags["nosave"] = {
	name = "No Save",
	tooltip = "apply to a win or a door. game won't save when using this exit.",
	kind = "exit",
	quad = quad(0, 32, 8, 8, tags_img)
}

tags["ignoreplayeramt"] = {
	name = "Ignore Player Amount",
	tooltip = "will not increment amount of player deaths needed to reset the level. always applies to ghost blocks",
	kind = "ogmo",
	quad = quad(8, 32, 8, 8, tags_img)
}

for i,v in ipairs(dirs) do
	tags["spawn" .. v] = {
		name = "Spawn " .. dirs_titlecase[i],
		tooltip = "apply to objects or tiles. this object/tile will only be present if the player went " .. v .. " to enter this level",
		kind = "any",
		quad = quad(16 + ((i - 1) * 8), 32, 8, 8, tags_img)
	}
end

tags["spawnup"].rotations = {"spawnleft", "spawnright"}
tags["spawnright"].rotations = {"spawnup", "spawndown"}
tags["spawndown"].rotations = {"spawnright", "spawnleft"}
tags["spawnleft"].rotations = {"spawndown", "spawnup"}

--[[
tags["noedgespawnbehavior"] = {
	name = "No Edge Spawn Behavior",
	tooltip = "by default, if ogmo is on the edge of a level, it only spawns if player entered the level from the opposite direction. this overrides that",
	kind = "ogmo",
	quad = quad(48, 32, 8, 8, tags_img)
}]]

tags["preservestars"] = {
	name = "Preserve Stars",
	tooltip = "apply to an ogmo. this ogmo keeps the star debuffs from the ogmo exiting the previous level.",
	kind = "ogmo",
	quad = quad(48, 32, 8, 8, tags_img)
}

tags["boomlink"] = { --Unimplemented.
	name = "Boom Link",
	tooltip = "apply to a tile. when a tile with this tag is blown up, adjacent tiles with the tag will also blow up. todo",
	kind = "tile",
	quad = quad(64, 32, 8, 8, tags_img)
}


tags["silhouette"] = {
	name = "Silhouette",
	tooltip = "apply to a tile or object. will appear completely dark.",
	kind = "any",
	rotations = {"lowalpha", "lowalpha"},
	quad = quad(56, 32, 8, 8, tags_img)
}

tags["lowalpha"] = {
	name = "Translucent",
	tooltip = "Apply to a tile or object. Will appear translucent.",
	kind = "any",
	rotations = {"silhouette", "silhouette"},
	quad = quad(72, 32, 8, 8, tags_img)
}

tags["r"] = {
	name = "Red Only",
	tooltip = "Apply to a tile or object. Non-red color will be filtered out.",
	kind = "any",
	quad = quad(0, 40, 8, 8, tags_img)
}

tags["g"] = {
	name = "Green Only",
	tooltip = "Apply to a tile or object. Non-green color will be filtered out.",
	kind = "any",
	quad = quad(8, 40, 8, 8, tags_img)
}

tags["b"] = {
	name = "Blue Only",
	tooltip = "Apply to a tile or object. Non-blue color will be filtered out.",
	kind = "any",
	quad = quad(16, 40, 8, 8, tags_img)
}

tags.r.rotations = {"b","g"}
tags.g.rotations = {"r","b"}
tags.b.rotations = {"g","r"}

tags["silent"] = {
	name = "Silent",
	tooltip = "Apply to an object. Object will not make any sounds.",
	kind = "object",
	rotations = {"invisible","fake"},
	quad = quad(24, 40, 8, 8, tags_img)
}

tags["additive"] = {
	name = "Additive Blending",
	tooltip = "Apply to a tile or object. Enables additive blending. If combined with Silhouette, gives mostly identical behavior to Invisible.",
	kind = "any",
	quad = quad(32, 40, 8, 8, tags_img)
}

tags["spawnblue"] = {
	name = "Spawn Blue",
	tooltip = "apply to objects or tiles. this object/tile will only be present if a blue gem was collected in this room.",
	kind = "any",
	rotations = {"spawnred","spawnred"},
	quad = quad(56, 40, 8, 8, tags_img)
}
tags["spawnred"] = {
	name = "Spawn Red",
	tooltip = "apply to objects or tiles. this object/tile will only be present if a red gem was collected in this room.",
	kind = "any",
	rotations = {"spawnblue","spawnblue"},
	quad = quad(64, 40, 8, 8, tags_img)
}