dingbats = {
	img = graphics.load("ui/dingbats"),
	quads = {},
	map_top = {},
	map_left = {},
	map_right = {},
	map_bottom = {},
	chance = 0.5,
	decaytimer_default = 30,
	--decaytimer_default = 5,
	decaytimer = 30, --minimum amount of frames that must pass before another decay can occur
	--decaychance = (4/60), --approximately 4 per second
	
	--decaychance = (0.125/60), --1 in 8 chance per second
	decaychance = (4/60), --averaging 4 decays per second
	--decaychance = 1,
	
	--decaymaxamt = 4, --maximum amount of cells that can be decayed at once
	
	decaytimer_default_fast = 1,
	decaychance_fast = 1,
	
	decaysdone = 0,
	
	--decay_removenext = false,
	--fader = false,
	faders = {},
	fadercells = {},
	--faderx = 0,
	--fadery = 0,
	--faderval = 0,
	fadervalmax = 60,
	--fadervalmax_fast = 15,
	fadervalmax_fast = 60,
	lastviewportw = 0,
	lastviewporth = 0,
	map_top_end = 0,
	map_left_end = 0,
	map_right_end = 0,
	map_bottom_end = 0, --total amount of cells
	
	--these all need to be set due to screenwidhts/screenheights not necessarily being a multiple of 16.
	map_right_x = 0,
	map_middle_y = 0,
	map_bottom_y = 0,
}

for y=0, dingbats.img:getHeight() - 16, 16 do
	for x=0, dingbats.img:getWidth() - 16, 16 do
		table.insert(dingbats.quads, quad(x, y, 16, 16, dingbats.img))
	end
end

local function newfader(x, y, q, cell)
	dingbats.fadercells[cell] = true
	local val = dingbats.fadervalmax
	if cheat.isactive("fastdingbats") then val = dingbats.fadervalmax_fast end
	table.insert(dingbats.faders, {["x"] = x, ["y"] = y, ["q"] = q, ["val"] = val, ["cell"] = cell})
	--table.insert for a table with holes is sort of undefined behavior but we're iterating with pairs anyway so the important part is that it just finds *somewhere* to shove it in (which the lua 5.1 manual specifies it does - "If the array has 'holes' (that is, nil values between other non-nil values), then #t can be any of the indices that directly precedes a nil value (that is, it may consider any such nil value as the end of the array).")
	--we assume the axiom of choice
end

function dingbats.draw()
	local function drawmap(map, xoff, yoff)
		for y=1, #map do
			for x=1, #map[y] do
				local dingbat = map[y][x]
				if dingbat then
					love.graphics.draw(dingbats.img, dingbats.quads[dingbat], ((x - 1) * 16) + xoff, ((y - 1) * 16) + yoff)
				end
			end
		end
	end
	local vp = game.viewport
	drawmap(dingbats.map_top, 0, 0)
	--drawmap(dingbats.map_left, 0, vp.y)
	drawmap(dingbats.map_left, 0, dingbats.map_middle_y)
	--drawmap(dingbats.map_right, vp.x + vp.width, vp.y)
	drawmap(dingbats.map_right, dingbats.map_right_x, dingbats.map_middle_y)
	--drawmap(dingbats.map_bottom, 0, vp.y + vp.height)
	drawmap(dingbats.map_bottom, 0, dingbats.map_bottom_y)
	local r,g,b,a = love.graphics.getColor()
	for k,fader in pairs(dingbats.faders) do
		
		--test
		--love.graphics.setColor(1, 0, 1, 1)
		--love.graphics.rectangle("line", fader.x, fader.y, 16, 16)
		
		if fader.q == 0 then
			love.graphics.setColor(0.5, 0.5, 0.5, fader.val / dingbats.fadervalmax)
			love.graphics.rectangle("fill", fader.x, fader.y, 16, 16)
		else
			love.graphics.setColor(r, g, b, fader.val / dingbats.fadervalmax)
			love.graphics.draw(dingbats.img, dingbats.quads[fader.q], fader.x, fader.y)
		end
	end
	love.graphics.setColor(r,g,b,a)
end

function dingbats.update()
	for k,fader in pairs(dingbats.faders) do
		fader.val = fader.val - 1
		if fader.val == 0 then
			dingbats.fadercells[fader.cell] = nil
			dingbats.faders[k] = nil
		end
	end
	if dingbats.decaytimer <= 0 then
		local decaychance = dingbats.decaychance
		if dingbats.decaytimer < 0 then decaychance = 1 end --decaytimer will be below 0 if a cell was prevented from decaying due to already being faded and its horizontal neighbors also already being faded
		if cheat.isactive("fastdingbats") then decaychance = dingbats.decaychance_fast end
		if love.math.random() < decaychance then
			--for i=1, love.math.random(1, dingbats.decaymaxamt) do
			do
				dingbats.decay()
			end
		end
	else
		dingbats.decaytimer = dingbats.decaytimer - 1
	end
end

function dingbats.decay()
	local cellno = love.math.random(1, dingbats.map_bottom_end)
	--local cellno = love.math.random(dingbats.map_left_end, dingbats.map_right_end)
	if dingbats.fadercells[cellno] then --already a cell fading out here!
		local goleft = (love.math.random() < 0.5)
		if goleft then
			if cellno == 1 then goleft = false end
			if dingbats.fadercells[cellno - 1] then goleft = false end
		end
		if goleft then
			cellno = cellno - 1
		else
			local goright = true
			if cellno == dingbats.map_bottom_end then goright = false end
			if dingbats.fadercells[cellno + 1] then goright = false end
			if goright then
				cellno = cellno + 1
			else
				dingbats.decaytimer = dingbats.decaytimer - 1 --negative decaytimer means the decay is trying again in a different cell
				return --give up
			end
		end
	end
	dingbats.decaysdone = dingbats.decaysdone + 1
	local effectivecell = cellno
	local map = dingbats.map_top
	local chosenmap = "top"
	if cellno > dingbats.map_top_end then
		effectivecell = cellno - dingbats.map_top_end
		map = dingbats.map_left
		chosenmap = "left"
		if cellno > dingbats.map_left_end then
			effectivecell = cellno - dingbats.map_left_end
			map = dingbats.map_right
			chosenmap = "right"
			if cellno > dingbats.map_right_end then
				effectivecell = cellno - dingbats.map_right_end
				map = dingbats.map_bottom
				chosenmap = "bottom"
			end
		end
	end
	local y = 1
	local x = 1
	if not map[1] then
		print("landed on length-zero dingbat map while trying to decay a dingbat?? (dingbats shouldn't be drawn if the viewport is trivial)")
		dingbats.decaytimer = 600000
		return
	end
	
	--[[
	.....
	.....
	.....
	.....
	.....
	5x5 map (25 cells total)
	get the 14th cell
	x = ((14 - 1) % 5) + 1
	y = math.ceil(14 / 5)
	
	x = 4
	y = 3
	
	checks out
	
	get the 15th cell
	x = ((15 - 1) % 5 + 1
	y = math.ceil(15 / 5)
	
	x = 5
	y = 3
	]]
	
	x = ((effectivecell - 1) % #map[1]) + 1 --map[1] will always exist; empty maps don't get selected because their end is always the same as another end
	y = math.ceil(effectivecell / #map[1])
	
	local screenx = (x - 1) * 16
	local screeny = (y - 1) * 16
	if chosenmap == "left" then
		--screeny = screeny + game.viewport.y
		screeny = screeny + dingbats.map_middle_y
	elseif chosenmap == "right" then
		--screenx = screenx + game.viewport.x + game.viewport.width
		screenx = screenx + dingbats.map_right_x
		--screeny = screeny + game.viewport.y
		screeny = screeny + dingbats.map_middle_y
	elseif chosenmap == "bottom" then
		--screeny = screeny + game.viewport.y + game.viewport.height
		screeny = screeny + dingbats.map_bottom_y
	end
	
	if map[y][x] then
		newfader(screenx, screeny, map[y][x], cellno)
		map[y][x] = false
	else
		newfader(screenx, screeny, 0, cellno)
		map[y][x] = love.math.random(1, #dingbats.quads)
	end
	--dingbats.decay_removenext = not dingbats.decay_removenext
	dingbats.decaytimer = dingbats.decaytimer_default
	if cheat.isactive("fastdingbats") then
		--dingbats.faderval = 0
		--dingbats.fader = false
		dingbats.decaytimer = dingbats.decaytimer_default_fast
	end
end

function dingbats.place()
	dingbats.decaysdone = 0
	dingbats.faders = {}
	dingbats.fadercells = {}
	print("placing dingbats")
	local dingbatsplaced = 0
	local maps = {dingbats.map_top, dingbats.map_left, dingbats.map_right, dingbats.map_bottom}
	for _,map in ipairs(maps) do
		for y=1, #map do
			for x=1, #map[y] do
				dingbatsplaced = dingbatsplaced + 1
				if love.math.random() < dingbats.chance then
					dingbatsplaced = dingbatsplaced + 1
					map[y][x] = love.math.random(1, #dingbats.quads)
				else
					map[y][x] = false
				end
			end
		end
	end
	print("placed " .. tostring(dingbatsplaced) .. " dingbats")
	dingbats.decaytimer = dingbats.decaytimer_default
	if cheat.isactive("fastdingbats") then dingbats.decaytimer = dingbats.decaytimer_default_fast end
end

function dingbats.initMaps()
	local vp = game.viewport
	if vp.width == dingbats.lastviewportwidth and vp.height == dingbats.lastviewportheight and game.width == dingbats.lastgamewidth and game.height == dingbats.lastgameheight then
		print("no need to reinit dingbat maps")
		return
	end
	dingbats.lastviewportwith = vp.width
	dingbats.lastviewportheight = vp.height
	dingbats.lastgamewidth = game.width
	dingbats.lastgameheight = game.height
	print("initializing dingbats maps")
	dingbats.map_top = {}
	dingbats.map_left = {}
	dingbats.map_right = {}
	dingbats.map_bottom = {}
	--dingbats.map_right_x = vp.x + vp.width
	--dingbats.map_bottom_x = vp.y + vp.height
	local hmaps_rows = math.ceil(((game.height - vp.height) / 2 ) / 16)
	if hmaps_rows < 0 then hmaps_rows = 0 end
	local hmaps_columns = math.ceil(game.width / 16)
	local vmaps_rows = math.ceil(game.height / 16) - (2 * hmaps_rows)
	local vmaps_columns = math.ceil(((game.width - vp.width) / 2 ) / 16)
	if vmaps_columns < 0 then vmaps_columns = 0 end
	
	dingbats.map_middle_y = (hmaps_rows * 16)
	dingbats.map_right_x = (game.width - (vmaps_columns * 16))
	dingbats.map_bottom_y = (game.height - (hmaps_rows * 16))
	
	dingbats.map_top_end = (hmaps_rows * hmaps_columns)
	dingbats.map_left_end = (vmaps_rows * vmaps_columns) + dingbats.map_top_end
	dingbats.map_right_end = (vmaps_rows * vmaps_columns) + dingbats.map_left_end
	dingbats.map_bottom_end = (hmaps_rows * hmaps_columns) + dingbats.map_right_end
	--originally when calculating the map ends, the previous map ends were added as well
	--e.g. dingbats.map_bottom_end = (hmaps_rows * hmaps_columns) + dingbats.map_right_end + dingbats.map_left_end + dingbats.map_top_end
	--cumulative effect caused map ends to be reported as way larger than in actuality and caused some crashes with trying to decay a nonexistent cell
	if hmaps_columns > 0 then
		for y=1, hmaps_rows do
			dingbats.map_top[y] = {}
			dingbats.map_bottom[y] = {}
			for x=1, hmaps_columns do
				dingbats.map_top[y][x] = false
				dingbats.map_bottom[y][x] = false
			end
		end
	end
	if vmaps_columns > 0 then
		for y=1, vmaps_rows do
			dingbats.map_left[y] = {}
			dingbats.map_right[y] = {}
			for x=1, vmaps_columns do
				dingbats.map_left[y][x] = false
				dingbats.map_right[y][x] = false
			end
		end
	end
end

--dingbats.initMaps()