--game is basically a mini state machine to access levels

local font = love.graphics.getFont()
game = {
	width = 640,
	height = 480,
	levelsetpath = "",
	--[[
	playeramt = 0,
	liveplayeramt = 0,
	loadedobjects = {},
	leveloptions = {},
	mapname = "",
	exits = {},
	currentsong = "",
	map = {},
	tilemap = {}, --multiple tiles actually can go on each coordinate so each square here is represented by a table
	levelwidth = 512,
	levelheight = 512,
	hints = {},
	]]
	godmode = false,
	editormode = false,
	ogmoskin = "ogmo",
	ogmosnapto = 1,
	ogmofakelag = false,
	ogmohmombug = false,
	paused = false,
	pausedformenu = false,
	pausedfortextbox = false,
	textboxtext = love.graphics.newText(font, ""),
	textboxwraplimit = 350,
	textboxwraplimit_default = 350,
	textboxwidth = nil,
	textboxheight = nil,
	textboxpadding = 25,
	textboxedgeofscreen = 25,
	textboxX = nil,
	textboxY = nil,
	
	shownamebox = false,
	nameboxtext = love.graphics.newText(font, ""),
	nameboxwidth = nil,
	nameboxheight = nil,
	nameboxpadding_h = 10,
	nameboxpadding_v = 5,
	nameboxX = nil,
	nameboxXoffset = 15,
	nameboxY = nil,
	nameboxcolor = false,
	
	textboxqueue = {},
	current_textbox = 1,
	
	templatelevels = {},
	activelevels = {},
	
	levelsets = {},
	int_levelsets = {},
	ext_levelsets = {},
	currentlevelset = nil,
	
	whitescreen = false, --used for testing pause screen gradients
	
	alreadydone_temp = {},
	
	solid_colors_default = {electricity = true, voltageinv = false, red = true, blue = false, cyan = true, yellow = false}, --included inverted voltage in the list of colors when it doesn't even use color mechanics like electricity does is probably eyebrow-raising. i really just did it because colors have all the checkpoint/saving stuff built in. i should probably rename colors to "switch states" or something
	solid_colors_start = {electricity = true, voltageinv = false, red = true, blue = false, cyan = true, yellow = false},
	solid_colors = {electricity = true, voltageinv = false, red = true, blue = false, cyan = true, yellow = false},
	
	switchpadstate = true,
	switchpadstate_start = true,
	switchpad2state = true,
	switchpad2state_start = true,
	
	corpses = {},
	
	save_entrance = 0,
	start_in_different_level = false,
	
	saveslot = nil,
	
	viewport = {
		width = 640,
		height = 480,
		trivial = true,
		x = 0,
		y = 0
	},
	
	screenborder_img = graphics.load("ui/screenborders"),
	screenborderquads = {},
	
	currentsong = "none",
	
	collectedgems = {
		["blue"] = {},
		["red"] = {},
	},
	unsavedgems = {
		["blue"] = {},
		["red"] = {},
	},
	collectedgems_bluecount = 0,
	collectedgems_redcount = 0,
	unsavedgems_bluecount = 0,
	unsavedgems_redcount = 0,
	new_gems_this_session = false,
	
	keymap = {},
	removefromkeymap = {},
	anykeypressed = false,
	
	flipscreen = false,
	
	emptyclique = false, --for playtesting; adding an ogmo when none exist
	
	interactprompt = love.graphics.newText(font, controls["P1INTERACT"]),
	
	backtomenu = false,
	youwin = false,
}

game.screenborderquads = {
	["d"] = quad(16, 0, 16, 16, game.screenborder_img),
	["dl"] = quad(32, 0, 16, 16, game.screenborder_img),
	["l"] = quad(32, 16, 16, 16, game.screenborder_img),
	["ul"] = quad(32, 32, 16, 16, game.screenborder_img),
	["u"] = quad(16, 32, 16, 16, game.screenborder_img),
	["ur"] = quad(0, 32, 16, 16, game.screenborder_img),
	["r"] = quad(0, 16, 16, 16, game.screenborder_img),
	["dr"] = quad(0, 0, 16, 16, game.screenborder_img),
}

function game.parseLevelset(foldername, ext)
	local levelset = {info = {}}
	levelset.info.songs = {}
	local key = nil --|
	local index = nil -->
	local pref = "levelsets/"
	if ext then pref = "ext_levelsets/" end
	levelset["foldername"] = foldername
	levelset["filepath"] = pref .. foldername
	if love.filesystem.getInfo(pref .. foldername .. "/LEVELINFO.txt") then
		for row in love.filesystem.lines(pref .. foldername .. "/LEVELINFO.txt") do
			if string.sub(row, 1, 3) == "===" and string.sub(row, -3, -1) == "===" then
				index = nil
				key = string.lower(string.sub(row, 4, -4))
				if string.sub(key, 1, 1) == "_" then
					error("in LEVELINFO for levelset \"" .. foldername .. "\", key \"" .. key .. "\" cannot start with _")
				end
			elseif key then
				local firstchar = string.sub(row, 1, 1)
				if firstchar == "|" then
					row = string.sub(row, 2, -1)
					if index then
						levelset.info[key][index] = row
					else
						levelset.info[key] = row
					end
				elseif firstchar == ">" then
					if not index then
						levelset.info[key] = {}
					end
					index = tonumber(string.sub(row, 2, -1))
					if not index then
						error("while parsing LEVELINFO of " .. foldername .. ", encountered > for non-number index (not supported rn lol)")
					end
				end
				--[[if key == "FLIPSCREEN" then
					levelset.info[key] = (row ~= "false")
				end]] --weird, unnecessary. why do it here?
				--...why *not* do it here. i guess i wanted levelinfo to only ever care about storing strings
			end
		end
	else
		levelset.info.name = foldername
		levelset.invalid = true
	end
	return levelset
end

local int_levelsets = love.filesystem.getDirectoryItems("levelsets")
local ext_levelsets = love.filesystem.getDirectoryItems("ext_levelsets")

for i,v in ipairs(int_levelsets) do
	local levelset = game.parseLevelset(v, false)
	table.insert(game.levelsets, levelset)
	table.insert(game.int_levelsets, levelset)
end
for i,v in ipairs(ext_levelsets) do
	local levelset = game.parseLevelset(v, true)
	table.insert(game.levelsets, levelset)
	table.insert(game.ext_levelsets, levelset)
end

function game.begin()
	
end

function game.resetLevel()
	game.do_cutscene_movement_after_respawn = true
	local samelevelid = game.templatelevels[1].filepath == game.activelevels[1].filepath
	if not samelevelid then game.corpses = {} end
	for k,v in pairs(game.solid_colors) do
		game.solid_colors[k] = game.solid_colors_start[k]
	end
	game.switchpadstate = game.switchpadstate_start
	game.switchpad2state = game.switchpad2state_start
	--for k,v in pairs(game.unsavedgems["blue"]) do game.unsavedgems["blue"][k] = nil end
	--for k,v in pairs(game.unsavedgems["red"]) do game.unsavedgems["red"][k] = nil end
	game.unsavedgems["blue"] = {}
	game.unsavedgems["red"] = {}
	game.unsavedgems_bluecount = 0
	game.unsavedgems_redcount = 0
	if game.templatelevels[1] then
		game.activelevels = {}
		game.activelevels[1] = game.templatelevels[1]:gamify(game.templatelevels[1].entrance)
		--[[if game.solid_colors.voltageinv then
			print("inverting voltage in respawned level")
			game.activelevels[1]:invertvoltage()
		end]]
		for k,v in pairs(game.activelevels[1].counts) do
			v.amt = 0
		end
	end
	for i,corpse in ipairs(game.corpses) do
		corpse.level = game.activelevels[1]
	end
	if game.start_in_different_level then
		local baselevel = game.activelevels[1]
		if not (game.editormode and not editor.playingamemusic) and (baselevel.music ~= audio.activesong or baselevel.options.playmusicevenifsame) then
			audio.playsong(baselevel.music, false, baselevel.options.pauseoldmusic)
			game.currentsong = baselevel.music
		end
	end
	game.start_in_different_level = false
end

function game.getpregamifiedlevel()
	--unused i guess. i think this had to do with checkpoints
	--Now meaningless anyway, since gamify is no longer called with discard_templates being true anymore.
end

function game.savegems()
	if game.unsavedgems_bluecount > 0 or game.unsavedgems_redcount > 0 then
		game.new_gems_this_session = true
	end
	for k,v in pairs(game.unsavedgems["blue"]) do
		game.collectedgems["blue"][k] = v
		game.unsavedgems["blue"][k] = nil
	end
	for k,v in pairs(game.unsavedgems["red"]) do
		game.collectedgems["red"][k] = v
		game.unsavedgems["red"][k] = nil
	end
	game.collectedgems_bluecount = game.collectedgems_bluecount + game.unsavedgems_bluecount
	game.collectedgems_redcount = game.collectedgems_redcount + game.unsavedgems_redcount
	game.unsavedgems_bluecount = 0
	game.unsavedgems_redcount = 0
	statusbar.update("gems")
end

function game.loadLevel(room, entrance, save, firstlevel, full, docutscenemovement)
	local success = true
	
	game.levelpaths = {}
	game.do_cutscene_movement_after_respawn = not not docutscenemovement
	if save == nil then save = true end
	
	local preexisting = false
	local baselevel
	if type(room) == "table" then
		--print("ok, been passed a room")
		baselevel = room
		baselevel.won = false
		preexisting = true
	else
		local roomname = room
		local x
		local y
		local filepath = (full and roomname) or game.levelsetpath .. "/levels/" .. roomname
		if full then
			local roomnamesplit = split(roomname, "/")
			roomname = roomnamesplit[#roomnamesplit]
		end
		if string.find(roomname, ",") then
			local coords = split(roomname, ",")
			x = tonumber(coords[1])
			y = tonumber(coords[2])
			if x and (math.modf(x)) ~= x then x = nil; print ("loaded room with non-int x (" .. tostring(x) .. ") - nuh-uh") end
			if y and (math.modf(y)) ~= y then y = nil; print ("loaded room with non-int y (" .. tostring(y) .. ") - nuh-uh") end
			--x and y can be nil if the level's filename isn't two integers separated by a comma. if either are nil then edge exits will not be automatic
		end
		local leveldata = love.filesystem.read(filepath..".txt")
		if leveldata == nil then
			print("hey your level file ain't jack shit (" .. roomname .. ", " .. filepath .. ")")
			leveldata = love.filesystem.read("void.txt");
			save = false;
			success = false
		end
		leveldata = correctnewlines(leveldata)
		game.levelpaths[1] = filepath
		baselevel = level:new(leveldata, filepath, x, y)
		baselevel.name = roomname
	end
	if firstlevel and not save then save = true end --firstlevel arg is a kludge that makes it so, if this is the very first level, entering it WILL save, even if the level is invalid and you get sent to a voidroom
	if save then
		game.save_entrance = entrance
		if not firstlevel then
			game.savegems()
		end
	end
	if not success then baselevel.voidroom = true end
	baselevel:gamify(entrance)
	if save then
		game.templatelevels = {}
		game.activelevels = {}
		game.templatelevels[1] = baselevel
		game.activelevels[1] = baselevel
		game.start_in_different_level = false
		for k,v in pairs(game.solid_colors) do
			game.solid_colors_start[k] = game.solid_colors[k]
		end
		game.switchpadstate_start = game.switchpadstate
		game.switchpad2state_start = game.switchpad2state
	else
		game.activelevels = {}
		game.activelevels[1] = baselevel
		game.start_in_different_level = true
	end
	if not (game.editormode and not editor.playingamemusic) and (baselevel.music ~= audio.activesong or baselevel.options.playmusicevenifsame) then
		--audio.playsong(baselevel.music, false, baselevel.options.pauseoldmusic, true)
		audio.playsong(baselevel.music, false, baselevel.options.pauseoldmusic)
		game.currentsong = baselevel.music
	end
	return success
end

function game.loadSavefile(path)
	--"saves/" .. game.currentlevelset.foldername .."/slot1.txt"
	if not love.filesystem.getInfo(path) then
		return false
	end
	
	local room
	local entrance = {}
	local colors = {}
	local switchpads = {}
	local music = false
	local gems = {}
	
	local field = nil
	--local kind = "blue"
	
	writer.thread:wait()
	writer.multithread:wait()
	local savefile = love.filesystem.read(path)
	local phase = nil
	savefile = correctnewlines(savefile)
	savefile = split(savefile, "\n")
	
	
	local function firstchar(s, c)
		return string.sub(s, 1, 1) == c
	end
	local function checkfieldsanity(s) --this could be in main
		if firstchar(s, "_") then
			error("underscore at the start of field in " .. tostring(phase) .. "phase of loading savefile")
		end
	end
	
	
	for i, row in ipairs(savefile) do
		if string.sub(row, 1, 3) == "===" and string.sub(row, -3, -1) == "===" then
			phase = string.sub(row, 4, -4)
		else
			--lua doesn't have switch statements :(
			--technically it does in the form of indexing functions from a table but it's pretty clunky
			if phase == "ROOM" then
				if string.sub(row, 1, 1) == "|" then
					room = string.sub(row, 2, -1)
				end
			elseif phase == "ENTRANCE" then
				if string.sub(row, 1, 1) == ">" then
					field = string.sub(row, 2, -1)
					checkfieldsanity(field)
				else
					local stuff = split(row, "|", 1)
					if stuff[1] == "nil" then stuff[2] = nil
					elseif stuff[1] == "number" then stuff[2] = assert(tonumber(stuff[2]), "you told me this thing in settings is a number but it's not")
					elseif stuff[1] == "boolean" then stuff[2] = strtobool(stuff[2])
					elseif stuff[1] ~= "string" then error("unsupported type in save file, " .. stuff[1]) end
					entrance[field] = stuff[2]
				end
			elseif phase == "COLORS" then
				if string.sub(row, 1, 1) == ">" then
					field = string.sub(row, 2, -1)
					checkfieldsanity(field)
				elseif string.sub(row, 1, 1) == "|" then
					colors[field] = strtobool(string.sub(row, 2, -1))
				end
			elseif phase == "SWITCHPADS" then --the state of switchpads that control colors, which can be independent from the colors themselves for two-color switchpads (when each color they control is switched by a one-color switchpad). purely visual
				if string.sub(row, 1, 1) == ">" then
					field = string.sub(row, 2, -1)
					checkfieldsanity(field)
				elseif string.sub(row, 1, 1) == "|" then
					switchpads[field] = strtobool(string.sub(row, 2, -1))
				end
			elseif phase == "MUSIC" then
				if string.sub(row, 1, 1) == "|" then
					music = string.sub(row, 2, -1)
				end
			--[[elseif phase == "GEMS" then
				if firstchar(row, ">") then
					kind = string.sub(row, 2, -1)
					if kind ~= "blue" and kind ~= "red" then error("in savefile, invalid gem kind: " .. kind) end
				elseif firstchar(row, "|") then
					field = string.sub(row, 2, -1)
					checkfieldsanity(field)
					if not string.find(field, ",") then error("in savefile, invalid gem location: " .. field) end
					gems[kind][field] = true
				end]]
			end
		end
	end
	return true, room, entrance, colors, switchpads, music, gems
end

function game.setviewport(w, h)
	local vp = game.viewport
	w = tonumber(w) or game.width
	h = tonumber(h) or game.height
	if w > game.width then w = game.width end
	if h > game.height then h = game.height end
	if vp.width == tonumber(w) and vp.height == tonumber(h) then
		print("game viewport is already this size!")
		return
	end
	vp.width = w
	vp.height = h
	vp.x = math.floor((game.width - vp.width) / 2)
	vp.y = math.floor((game.height - vp.height) / 2)
	if vp.width ~= game.width or vp.height ~= game.height then
		vp.trivial = false
	else
		vp.trivial = true
	end
	if not vp.trivial then
		dingbats.initMaps()
		dingbats.place()
	end
end

local switchpads_default = {["redblue"] = true, ["cyanyellow"] = true}
function game.loadLevelset(levelset) --major changes here should also be copied to making a new levelset/accessing a levelset in the "edit" submenu
	game.currentlevelset = levelset
	game.levelsetpath = levelset.filepath
	audio.fetchcustommusic(levelset.filepath .. "/music")
	stageselect.fetchstages()
	local vp = game.viewport
	game.setviewport(levelset.info.viewportwidth or levelset.info.screenwidth or game.width, levelset.info.viewportheight or levelset.info.screenheight or game.height)
	game.flipscreen = (levelset.info.flipscreen == "true")
	
	local savename = "saves/" .. game.currentlevelset.foldername .."/slot" .. game.saveslot
	local saveexists, room, entrance, colors, switchpads, music = game.loadSavefile(savename .. ".txt")
	gemsave.read(savename .. ".j4g", true) --true argument automatically sets the relevant variables
	if not room then room = levelset.info.firstroom end
	if not entrance then entrance = 0 end
	game.save_entrance = entrance
	
	local E = {}
	--[[if gems then
		for k,v in pairs(gems["blue"] or E) do
			game.collectedgems["blue"][k] = true
		end
		for k,v in pairs(gems["red"] or E) do
			game.collectedgems["red"][k] = true
		end
	end]]
	for k,v in pairs(colors or game.solid_colors_default) do
		game.solid_colors[k] = v
		game.solid_colors_start[k] = v
	end
	for k,v in pairs(switchpads or switchpads_default) do
		if k == "redblue" then
			game.switchpadstate = v
			game.switchpadstate_start = v
		elseif k == "cyanyellow" then
			game.switchpad2state = v
			game.switchpad2state_start = v
		end
	end
	if music then
		audio.playsong(music)
		game.currentsong = music
	end
	
	local success = game.loadLevel(room, entrance, true, true, false, true)
	if not success then
		if not saveexists then
			game.activelevels[1]:placeascii("starting room \"" .. tostring(room) .. "\"\ndoes not exist!", 3, 5)
			--game.templatelevels[1]:placeascii("starting room \"" .. tostring(room) .. "\"\ndoes not exist!", 3, 5)
		else
			game.activelevels[1]:placeascii("from savefile: room \"" .. tostring(room) .. "\"\ndoes not exist!", 3, 5)
			--game.templatelevels[1]:placeascii("from savefile: room \"" .. tostring(room) .. "\"\ndoes not exist!", 3, 5)
			game.save_entrance = 0
		end
	end
	local transitenabled = true --= not settings.notransit
	if transitenabled then
		--mirror changes here to transit.lua loadlevelset
		transit.active = true
		transit.reverse = true
		transit.progress = 0
		transit.type = "fade" --REDUNDANT ACCESS if the condition two lines below is met! garbage program! start over
		local focus = game.activelevels[1].camerafocus
		if focus and consts.oppositedirs[focus.edge] then --index to get whether this is a valid direction
			transit.type = "wipe" .. focus.edge
		end
	end
end

function game.update(dt)
	local doreset = false
	if (not game.pausedformenu) and (not game.viewport.trivial) then
		dingbats.update()
	end
	if transit.active then
		transit.update()
	elseif not game.paused then
		--[=[
		for i=1, #objectnamelist do
			if game.loadedobjects[objectnamelist[i]] ~= nil then
				for ii=1, #game.loadedobjects[objectnamelist[i]] do
					game.loadedobjects[objectnamelist[i]][ii]:update()
				end
			end
		end
		]=]
		for i,corpse in ipairs(game.corpses) do
			corpse:update()
		end
		for i,level in ipairs(game.activelevels) do
			for k,v in pairs(game.alreadydone_temp) do game.alreadydone_temp[k] = nil end --wow, i barely remember this. looks like an earlier version of temps
			level:update()
			if level.doreset then doreset = true end
		end
	end
	if game.backtomenu then game.gobacktomenu()
	elseif game.youwin then game.gotoyouwin()
	elseif doreset then
		doreset = false
		print("resetting!")
		game.resetLevel()
	end
	for k,v in pairs(game.keymap) do
		if game.removefromkeymap[k] then
			game.keymap[k] = -1
			game.removefromkeymap[k] = nil
		else
			game.keymap[k] = game.keymap[k] + 1
			if game.keymap[k] == 0 then game.keymap[k] = nil end
		end
	end
	game.anykeypressed = false
	for k,v in pairs(game.removefromkeymap) do
		print("leftover removefromkeymap: " .. tostring(k))
		game.removefromkeymap[k] = nil
	end
end

function game.showtextbox(text, name, color)
	if game.pausedfortextbox then
		game.textboxqueue[#game.textboxqueue + 1] = {text, name, color}
	else
		game.shownamebox = false
		
		game.pausedfortextbox = true
		game.paused = true
		game.textboxtext:setf(text, game.textboxwraplimit, "left")
		game.textboxheight = game.textboxtext:getHeight() + (game.textboxpadding * 2)
		game.textboxwidth = game.textboxtext:getWidth() + (game.textboxpadding * 2)
		game.textboxX = math.floor((game.width - game.textboxwidth) / 2)
		game.textboxY = math.floor((game.height - game.textboxheight) / 2)
		if name then
			if color then
				game.nameboxcolor = color
			else
				game.nameboxcolor = false
			end
			game.shownamebox = true
			game.nameboxtext:setf(name, game.textboxwraplimit, "left")
			game.nameboxheight = game.nameboxtext:getHeight() + (game.nameboxpadding_v * 2)
			game.nameboxwidth = game.nameboxtext:getWidth() + (game.nameboxpadding_h * 2)
			if game.textboxwidth < game.nameboxwidth + game.nameboxXoffset then
				game.textboxwidth = game.nameboxwidth + game.nameboxXoffset
				game.textboxX = math.floor((game.width - game.textboxwidth) / 2)
			end
			game.nameboxX = game.textboxX + game.nameboxXoffset
			game.nameboxY = game.textboxY - game.nameboxheight
		end
	end
end

function game.pause(silent)
	menu.picker = 1
	game.pausedformenu = true
	game.paused = true
	if not silent then audio.playsfx("pause") end
	menu.fact = facts[love.math.random(1, #facts)]
	menu.facttext:setf("FACT: " .. menu.fact, game.width / 2, "center")
	if audio.activesong ~= nil then
		local song = audio.loadedsongs[audio.activesong]
		song:setFilter(audio.lowpass)
	end
	statemachine.setoverlaystate("menu")
	if menu.substateTemplate ~= "paused" then menu.changeSubstate("paused") end
end

function game.unpause()
	menu.picker = 1
	game.pausedformenu = false
	game.paused = false
	if game.pausedfortextbox then game.paused = true end
	audio.playsfx("pause")
	if audio.activesong ~= nil then
		local song = audio.loadedsongs[audio.activesong]
		song:setFilter()
	end
end

function game.keyWasPressed(key)
	return game.keymap[key] == 1
end

function game.keypressed(key)
	--if key == "kp/" then game.whitescreen = not game.whitescreen end
	--[[
	if not (game.paused or transit.active) then
		for i,level in ipairs(game.activelevels) do
			level:keypressed(key)
		end
	end
	]]
	
	game.anykeypressed = true
	game.removefromkeymap[key] = nil
	if not game.keymap[key] then
		game.keymap[key] = 1
	end
	--if key == "g" then game.showtextbox("hello. this is my incredibly lengthy text that is ridiculously lengthy") end
	--if (game.editormode) and (key == "escape" or key == "tab") then
	if game.editormode and key == "tab" then
		editor.scrollVert_HoldMoveDelayCurrent = 20
		editor.scrollHori_HoldMoveDelayCurrent = 20
		statemachine.setstate("editor")
	elseif key == "escape" then
		game.pause()
	elseif key == "return" and game.pausedfortextbox then
		game.pausedfortextbox = false
		if game.current_textbox > #game.textboxqueue then
			game.current_textbox = 1
			game.textboxqueue = {}
		else
			game.current_textbox = game.current_textbox + 1
			local text = game.textboxqueue[game.current_textbox - 1][1]
			local name = game.textboxqueue[game.current_textbox - 1][2]
			local color = game.textboxqueue[game.current_textbox - 1][3]
			game.showtextbox(text, name, color)
		end
	--elseif key == "p" then
	--	game.pause()
	--elseif love.keyboard.isDown("lctrl") and key == "s" then --debug
	--	print("DEBUG save")
	--	writeto("save")
	elseif cheat.isactive("smallsteps") then
		if key == "[" then
			for _,level in ipairs(game.activelevels) do
				for _,object in ipairs(level.objects) do
					if object.type == "ogmo" then
						object.x = object.x - 1
					end
				end
			end
		elseif key == "]" then
			for _,level in ipairs(game.activelevels) do
				for _,object in ipairs(level.objects) do
					if object.type == "ogmo" then
						object.x = object.x + 1
					end
				end
			end
		end
	end
	--[[
   if key == "rctrl" then
      debug.debug()
   end
   ]]--
	if game.pausedfortextbox or game.pausedformenu then game.paused = true else game.paused = false end
end

function game.keyreleased(key)
	if game.keymap[key] == 1 then
		game.removefromkeymap[key] = true --if the key was just pressed, then don't recognize it was released until this frame is over (so pressing a key and then releasing it fast enough doesn't cause it to get dropped)
	else
		game.keymap[key] = -1 --negative or 0 value represents cooldown, since i want ingame objects to detect key presses "discretely", as though "key was pressed" just means "key is held this frame but not the previous frame".  e.g. if you press a key on frame 1, you shouldn't be able to press it again on frame 2.
	end
	--sylvie made a neat post about how "1-frame inputs" are inherently shaky: https://cohost.org/sylvie/post/2454127-notes-on-1-frame-inp . which doesn't really have anything to do with j4r except that i'm intentionally preventing presses and releases from happening in the same frame (though for reasons of preferring discreteness and *maybe* making replay files easier)
end

function game.mousepressed(x, y, button)
	if ((game.editormode and game.emptyclique) or cheat.isactive("clique")) and button == 1 then
		local toplevel = game.activelevels[#game.activelevels]
		local targetx = (x + toplevel.cameraX - (game.viewport.x + toplevel.x))
		local targety = (y + toplevel.cameraY - (game.viewport.y + toplevel.y))
		if	targetx >= 0 and targetx < toplevel.width and
			targety >= 0 and targety < toplevel.height
		then
			game.emptyclique = false
			toplevel.noplayers = false
			local newogmo = ogmo:setup(targetx - (tilesize / 2), targety - (tilesize / 2), {}, toplevel)
			table.insert(toplevel.objects, newogmo)
			newogmo.level = toplevel
			toplevel:updateobjmap(newogmo)
			toplevel.liveplayeramt = toplevel.liveplayeramt + 1
		end
	elseif cheat.isactive("scroller1") and button == 3 then
		for _,level in ipairs(game.activelevels) do
			for _,object in ipairs(level.objects) do
				if object.type == "ogmo" then
					if love.mouse.isDown(2) then
						object.vmom = object.vmom * 2
						object.hmom = object.hmom * 2
					else
						object.vmom = 0
						object.hmom = 0
					end
				end
			end
		end
	end
end

function game.wheelmoved(x, y)
	if cheat.isactive("scroller1") then
		if love.mouse.isDown(2) then x, y = y, x end
		for _,level in ipairs(game.activelevels) do
			for _,object in ipairs(level.objects) do
				if object.type == "ogmo" then
					object.vmom = object.vmom - y
					object.hmom = object.hmom + (x * 2)
				end
			end
		end
	end
end

function game.draw()
	local r, g, b, a = love.graphics.getColor()
	--love.graphics.setColor(hextocolor(game.activelevels[1].background))
	--love.graphics.rectangle("fill", 0, 0, love.graphics.getWidth(), love.graphics.getHeight())
	love.graphics.setColor(0.5, 0.5, 0.5)
	love.graphics.rectangle("fill", 0, 0, game.width, game.height)
	love.graphics.setColor(r, g, b, a)
	--local dingbatsdrawn = 0
	if not game.viewport.trivial then
	--[[	for y=1, #game.dingbats do
			for x=1, #game.dingbats[y] do
				local dingbat = game.dingbats[y][x]
				if dingbat then
					dingbatsdrawn = dingbatsdrawn + 1
					love.graphics.draw(game.dingbats_img, game.dingbatquads[dingbat], (x - 1) * 16, (y - 1) * 16)
					--love.graphics.draw(game.dingbats_img, game.dingbatquads[dingbat])
				end
			end
		end]]
		dingbats.draw()
	end
	--if frameadvance then
	--	print("dingbats drawn: " .. tostring(dingbatsdrawn))
	--end
	
	for _,level in ipairs(game.activelevels) do
		level:draw()
	end
	if #game.corpses > 0 then
		love.graphics.push()
		local lvl = game.activelevels[1]
		local vp = game.viewport
		local smaller_width = (vp.width < lvl.width and vp.width) or lvl.width
		local smaller_height = (vp.height < lvl.height and vp.height) or lvl.height
		love.graphics.setScissor(vp.x + lvl.x, vp.y + lvl.y, smaller_width, smaller_height)
		love.graphics.translate((-1 * lvl.cameraX) + vp.x + lvl.x, (-1 * lvl.cameraY) + vp.y + lvl.y)
		for i,corpse in ipairs(game.corpses) do
			corpse:draw()
		end
		love.graphics.pop()
		love.graphics.setScissor()
	end
	if transit.active then
		transit.draw()
	end
	if game.editormode then
		printWithOutline("testing level! press tab to return to editor")
	end
	statusbar.draw()
	if game.pausedfortextbox then
		love.graphics.setColor(0, 0, 0, 1)
		love.graphics.rectangle("fill", game.textboxX, game.textboxY, game.textboxwidth, game.textboxheight)
		love.graphics.setColor(0.2, 0.2, 0.2, 1)
		love.graphics.rectangle("line", game.textboxX, game.textboxY, game.textboxwidth, game.textboxheight)
		love.graphics.setColor(r, g, b, a)
		love.graphics.draw(game.textboxtext, game.textboxX + game.textboxpadding, game.textboxY + game.textboxpadding)
		love.graphics.printf("(press enter)", game.textboxX, (game.textboxY + game.textboxheight) - (math.floor(game.textboxpadding / 2)), game.textboxwidth / 0.75, "right", 0, 0.75)
		--if #game.textboxqueue > 0 then
		--	love.graphics.printf(tostring(game.currenttextbox) .. "/" .. tostring(#game.textboxqueue + 1), game.textboxX, (game.textboxY + game.textboxheight) - (math.floor(game.textboxpadding / 2)), game.textboxwidth / 0.75, "right", 0, 0.75)
		--end
		if game.shownamebox then
			love.graphics.setColor(0.2, 0.2, 0.2, 1)
			love.graphics.rectangle("fill", game.nameboxX, game.nameboxY, game.nameboxwidth, game.nameboxheight)
			love.graphics.setColor(0.4, 0.4, 0.4, 1)
			love.graphics.rectangle("line", game.nameboxX, game.nameboxY, game.nameboxwidth, game.nameboxheight)
			if game.nameboxcolor then
				love.graphics.setColor(game.nameboxcolor)
			else
				love.graphics.setColor(1, 1, 1, a)
			end
			love.graphics.draw(game.nameboxtext, game.nameboxX + game.nameboxpadding_h, game.nameboxY + game.nameboxpadding_v)
			love.graphics.setColor(r, g, b, a)
		end
	end
	if game.editormode then
		love.graphics.setColor(0.2, 0.2, 0.2)
		love.graphics.rectangle("fill", 0, game.height, game.width + editor.addedwidth, game.height)
		love.graphics.rectangle("fill", game.width, 0, editor.addedwidth, game.height)
		love.graphics.setColor(r, g, b, a)
		if game.emptyclique then
			love.graphics.print([[
no players spawned.
click anywhere in the level to spawn an ogmo.]], 160,490)
		end
	end
	if game.pausedformenu then
		love.graphics.setColor(0, 0, 0, 0.5)
		love.graphics.rectangle("fill", 0, 0, love.graphics.getWidth(), love.graphics.getHeight())
		love.graphics.setColor(r, g, b, a)
		--printAsTooltip("fact: " .. game.fact)
	end
	--"testing level! press tab to return" print was originally here
	if game.whitescreen then
		love.graphics.rectangle("fill", 0, 0, love.graphics.getWidth(), love.graphics.getHeight())
	end
end

function game.gobacktomenu()
	resizeWindow(baseWidth, baseHeight)
	statemachine.setstate("menu")
end

function game.gotoyouwin()
	if not game.editormode then resizeWindow(baseWidth, baseHeight) end
	statemachine.setstate("youwin")
end

function game.win(exit, level, save, dir)
	local transitenabled = not settings.notransit
	if transit.active then print("win from transit") end
	if save and transitenabled and not transit.active then
		print("transiting")
		transit.queuedwin = {exit, level, save}
		transit.progress = 0
		transit.active = true
		if dir and consts.oppositedirs[dir] then --index to get whether this is a valid direction
			transit.type = "wipe" .. dir
		else
			transit.type = "fade"
		end
		for _,level in ipairs(game.activelevels) do
			level.won = true
		end
		return
	end
	--theoretically this could be called twice, which is stupid. i don't think it would do anything bad though - just load a level and then immediately load another
	game.youwin = false
	local id = exit.id
	if statemachine.currentstate == game and not level.frozen then
		for _,level in ipairs(game.activelevels) do
			level.won = true
		end
		--if game.editormode then
		if false then
			editor.scrollVert_HoldMoveDelayCurrent = 20
			editor.scrollHori_HoldMoveDelayCurrent = 20
			statemachine.setstate("editor")
		else
			if id == nil then id = 1; print("level exit id is nil, setting to 1") end
			--if not level.exits[number] then error("invalid level exit " .. number, 2) end
			if not level.exits[id] then
				print("level exit " .. tostring(id) .. " (type " .. type(id) .. ") is nonexistent. entering void")
				print("listing exits:")
				for k,v in pairs(level.exits) do
					print(k,v)
				end
				game.loadLevel("void", id, false, false, true)
				game.activelevels[1]:placeascii("exit " .. tostring(id) .. " (" .. type(id) .. ")\nof room " .. tostring(level.roomX) .. "," .. tostring(level.roomY) .. "\nis nonexistent", 3, 5)
				game.activelevels[1].voidroom = true
			else
				print ("won w/ id " .. tostring(id) .. "! time to load " .. level.exits[id])
				local newlevel = level.exits[id]
				if newlevel == "WINLEVELSET" then
					print("YOU WON THE LEVELSET!!!!!")
					game.youwin = true
				else
					local pauseoldmusic = false
					local playmusicevenifsame = false
					
					--if playtesting from editor, try to load unsaved room before regular version of room
					if game.editormode then
						local coords = newlevel:split(",")
						local x = tonumber(coords[1])
						local y = tonumber(coords[2])
						print(x, y)
						print(editor.unsavedrooms[y])
						if editor.unsavedrooms[y] then print(editor.unsavedrooms[y][x]) end
						if x and y and editor.unsavedrooms[y] and editor.unsavedrooms[y][x] then
							local unsavedroom = editor.unsavedrooms[y][x]
							local found = game.loadLevel(unsavedroom, exit, save)
							if found then
								print("(found unsaved room)")
								return
							else print("unsaved room at [y][x] exists but couldn't be loaded...") end
						end
					end
					
					local success = false
					local filepath = game.levelsetpath .. "/levels/" .. newlevel
					local exists = love.filesystem.getInfo(filepath..".txt")
					if exists then
						success = game.loadLevel(newlevel, exit, save)
					end
					if not success then
						game.loadLevel("void", id, false, false, true)
						game.activelevels[1]:placeascii("room \"" .. newlevel .. "\" does not exist!\nfrom exit " .. tostring(id) .. " (" .. type(id) .. ")\nof room " .. tostring(level.roomX) .. "," .. tostring(level.roomY), 3, 5)
						game.activelevels[1].voidroom = true
					end
				end
			end
		end
	end
end

function game.trysave()
	if (statemachine.currentstate == game) and (not game.editormode) and type(game.save_entrance) == "table" then
		if not game.new_gems_this_session then
			print("saving")
			writeto("save")
		else
			print("saving (+ gems)")
			writeto("save")
			local gemfile = "saves/" .. game.currentlevelset.foldername .. "/slot" .. tostring(game.saveslot) .. ".j4g"
			local gemunionfile = "saves/" .. game.currentlevelset.foldername .. "/union.j4g"
			gemsave.doubledelivery(gemfile, gemunionfile)
			--writeto_multi("save", "gems") --lol no.
		end
	end
end

function game.stop(newstate)
	game.keymap = {}
	game.removefromkeymap = {}
	game.trysave()
	--game.flipscreen = false
	transit.reset()
	game.backtomenu = false
	game.youwin = false
	for k,v in pairs(game.solid_colors) do
		game.solid_colors[k] = game.solid_colors_default[k]
		game.solid_colors_start[k] = game.solid_colors_default[k]
	end
	game.switchpadstate = true
	game.switchpad2state = true
	game.corpses = {}
	game.templatelevels = {}
	game.activelevels = {}
	--for k,v in pairs(game.collectedgems["blue"]) do game.collectedgems["blue"][k] = nil end
	--for k,v in pairs(game.collectedgems["red"]) do game.collectedgems["red"][k] = nil end
	--for k,v in pairs(game.unsavedgems["blue"]) do game.unsavedgems["blue"][k] = nil end
	--for k,v in pairs(game.unsavedgems["red"]) do game.unsavedgems["red"][k] = nil end
	game.collectedgems["blue"] = {}
	game.collectedgems["red"] = {}
	game.unsavedgems["blue"] = {}
	game.unsavedgems["red"] = {}
	game.collectedgems_bluecount = 0
	game.collectedgems_redcount = 0
	game.unsavedgems_bluecount = 0
	game.unsavedgems_redcount = 0
	game.new_gems_this_session = false
	--game.levelsetpath = ""
	--game.currentlevelset = nil
	if game.editormode then editor.returningfromgame = true end
	if newstate ~= "youwin" then game.editormode = false end
	audio.stoploopingsfxall()
	game.paused = false
	game.pausedformenu = false
	game.pausedfortextbox = false
	game.textboxqueue = {}
	if audio.activesong ~= nil then
		song = audio.loadedsongs[audio.activesong]
		song:setFilter()
	end
end