local header_reserved_length = 12 --gemsave files have a 16-byte header containing info the menu will want to access, so it doesn't need to read the whole file. of these 16 bytes, 4 are currently used:
--blue gem count (2 bytes)
--red gem count (2 bytes)
--so 12 bytes remain.
gemsave = {}
function gemsave.readamts(filename)
	if not love.filesystem.getInfo(filename, "file") then
		print("gemsave.readamts: file " .. tostring(filename) .. " not found; supposing it's empty")
		return 0, 0, false
	end
	local file = love.filesystem.newFile(filename)
	print("gemsave.readamts: reading from gemsave: " .. tostring(filename))
	file:open("r")
	local blue_amt = love.data.unpack(">H", (file:read(2))) --uint16 big endian. note that read HAS to be put in parens, otherwise the second argument returned by File:read, the amount of bytes read, will be used as the "pos" argument for unpack - skipping all of the string!
	local red_amt = love.data.unpack(">H", (file:read(2)))
	file:close()
	print("gemsave.readamts: finished reading")
	return blue_amt, red_amt, file
end

function gemsave.read(filename, autoset)
	if not love.filesystem.getInfo(filename, "file") then
		print("gemsave.read: file " .. tostring(filename) .. " not found; supposing it's empty")
		if autoset then
			game.collectedgems_bluecount = 0
			game.collectedgems_redcount = 0
			game.collectedgems.blue = {}
			game.collectedgems.red = {}
			return
		else
			return 0, 0, {}, {}, false
		end
	end
	local file = love.filesystem.newFile(filename)
	print("gemsave.read: reading from gemsave: " .. tostring(filename))
	file:open("r")
	local blue_amt = love.data.unpack(">H", (file:read(2))) --uint16 big endian
	local red_amt = love.data.unpack(">H", (file:read(2)))
	file:read(header_reserved_length) --reserved
	local blue_map = {}
	local red_map = {}
	for i=1, blue_amt do
		local x = love.data.unpack(">h", (file:read(2))) --int16 big endian
		local y = love.data.unpack(">h", (file:read(2)))
		blue_map[x .. "," .. y] = true
	end
	for i=1, red_amt do
		local x = love.data.unpack(">h", (file:read(2)))
		local y = love.data.unpack(">h", (file:read(2)))
		red_map[x .. "," .. y] = true
	end
	if not file:isEOF() then
		file:close()
		error("excess data in gemsave: " .. tostring(filename))
	end
	file:close()
	print("gemsave.read: finished reading")
	if autoset then
		game.collectedgems_bluecount = blue_amt
		game.collectedgems_redcount = red_amt
		game.collectedgems.blue = blue_map
		game.collectedgems.red = red_map
	else
		return blue_amt, red_amt, blue_map, red_map, file
	end
end

do --doubledelivery
	local buffer = require("string.buffer")
	
	--when a level's gem savefile is written to, the corresponding union savefile is also either written to or checked to see if there's anything new to write. we do both writes from one function to save on iterating through currentgems.
	function gemsave.doubledelivery(filename, unionfilename)
		--currently this takes the current collectedgems tables as a base, then removes redundant entries from union, rather than using union as a base and removing redundant entries from the collectedgems tables. i impulsively thought "wait, but if i did it the other way around, then that would mean the last entries in the union are the newest ones, which would be nice" but that's not the case because pairs() order isn't stable. i wrote words.
		showsave_nonthread()
		print("begin double delivery")
		assert(filename, "missing filename for gemsave.doubledelivery")
		assert(unionfilename, "missing union filename for gemsave.doubledelivery")
		local blue_amt = game.collectedgems_bluecount
		local red_amt = game.collectedgems_redcount
		local blue_amt_observed = 0
		local red_amt_observed = 0
		local blue_map = game.collectedgems.blue
		local red_map = game.collectedgems.red
		
		local union
		local union_blue_amt = 0
		local union_red_amt = 0
		
		local bluestring = buffer.new(blue_amt)
		local redstring = buffer.new(red_amt)
		local unionblueextra = buffer.new(blue_amt)
		local unionredextra = buffer.new(red_amt)
		local unionfile
		local skipunionwrite = true
		do
			local unionblue
			local unionred
			_,_,unionblue,unionred,unionfile = gemsave.read(unionfilename)
			if not unionfile then unionfile = love.filesystem.newFile(unionfilename) end
			print("continue double delivery")
			union = {
				blue = unionblue,
				red = unionred,
			}
		end
		
		--handle current gems, and remove them from union to avoid further iteration later
		for k,v in pairs(blue_map) do
			blue_amt_observed = blue_amt_observed + 1
			local coords = k:split(",")
			local x = love.data.pack("string",">h",tonumber(coords[1])) --int16 big endian
			local y = love.data.pack("string",">h",tonumber(coords[2]))
			bluestring:put(x)
			bluestring:put(y)
			if skipunionwrite and not union.blue[k] then skipunionwrite = false end
			union.blue[k] = nil
		end
		if blue_amt then
			if blue_amt_observed > blue_amt then
				print("MORE blue gems found while iterating collectedgems.blue than stored in collectedgems_bluecount!")
			elseif blue_amt_observed < blue_amt then
				print("LESS blue gems found while iterating collectedgems.blue than stored in collectedgems_bluecount!")
			end
		end
		
		for k,v in pairs(red_map) do
			red_amt_observed = red_amt_observed + 1
			local coords = k:split(",")
			local x = love.data.pack("string",">h",tonumber(coords[1]))
			local y = love.data.pack("string",">h",tonumber(coords[2]))
			redstring:put(x)
			redstring:put(y)
			if skipunionwrite and not union.red[k] then skipunionwrite = false end
			union.red[k] = nil
		end
		if red_amt then
			if red_amt_observed > red_amt then
				print("MORE red gems found while iterating collectedgems.red than stored in collectedgems_redcount!")
			elseif red_amt_observed < red_amt then
				print("LESS red gems found while iterating collectedgems.red than stored in collectedgems_redcount!")
			end
		end
		
		--handle union, though it's more like exclusive disjunction at this point
		union_blue_amt = blue_amt_observed
		for k,v in pairs(union.blue) do
			union_blue_amt = union_blue_amt + 1
			local coords = k:split(",")
			local x = love.data.pack("string",">h",tonumber(coords[1]))
			local y = love.data.pack("string",">h",tonumber(coords[2]))
			unionblueextra:put(x)
			unionblueextra:put(y)
		end
		union_red_amt = red_amt_observed
		for k,v in pairs(union.red) do
			union_red_amt = union_red_amt + 1
			local coords = k:split(",")
			local x = love.data.pack("string",">h",tonumber(coords[1]))
			local y = love.data.pack("string",">h",tonumber(coords[2]))
			unionredextra:put(x)
			unionredextra:put(y)
		end
		
		local levelset = game.currentlevelset
		levelset.gemcounts["union"]["blue"] = union_blue_amt
		levelset.gemcounts["union"]["red"] = union_red_amt
		levelset.gemcounts[game.saveslot]["blue"] = blue_amt_observed
		levelset.gemcounts[game.saveslot]["red"] = red_amt_observed
		
		local reservedstring = string.rep("\255", header_reserved_length)
		
		local fullstring = buffer.new()
		fullstring:put(love.data.pack("string", ">H", blue_amt_observed)) --uint16 big endian
		fullstring:put(love.data.pack("string", ">H", red_amt_observed))
		fullstring:put(reservedstring) --reserved
		fullstring:put(tostring(bluestring))
		fullstring:put(tostring(redstring))
		
		local fullunionstring = buffer.new()
		fullunionstring:put(love.data.pack("string", ">H", union_blue_amt))
		fullunionstring:put(love.data.pack("string", ">H", union_red_amt))
		fullunionstring:put(reservedstring) --reserved
		fullunionstring:put(tostring(bluestring))
		fullunionstring:put(tostring(unionblueextra))
		fullunionstring:put(tostring(redstring))
		fullunionstring:put(tostring(unionredextra))
		
		local file = love.filesystem.newFile(filename)
		print("writing to gems...")
		file:open("w")
		file:write(fullstring:tostring())
		print("finished writing to gems")
		file:close()
		if not skipunionwrite then
			print("writing to union gems...")
			unionfile:open("w")
			unionfile:write(fullunionstring:tostring())
			unionfile:close()
			print("finished writing to union gems")
		end
		
		print("double delivery finished")
	end
end