local defaultbox = {
	title = "my awesome box",
	tabs = {
		{
			name = "tab one",
			elements = {
				{
					type = "text",
					content = "This is an example box."
				},
				{
					type = "tickbox",
					content = "enable textfield",
					id = "enabletextfield"
				},
				{
					type = "text",
					content = "type some stuff here."
				},
				{
					type = "textfield",
					--content = "type some text here",
					--length = 16,
					req = "enablething",
					id = "id1",
				},
				{
					type = "tickbox",
					content = "enable radio buttons",
					id = "enableradio"
				},
				{
					type = "radio",
					--buttons = {
					--	{content = "thingmabobs"},
					--	{content = "doohickeys"},
					--	{content = "gizmos"}
					--},
					content = {
						{name = "Thingmabobs (best option)", id = "thingmabobs"},
						{name = "Doohickeys (just ok)", id = "doohickeys"},
						{name = "Gizmos (awful)", id = "gizmos"}
					},
					req = "enableradio",
					id = "id2"
				},
				{
					type = "text",
					content = "how many friends do you have"
				},
				{
					type = "numberfield",
					max = 200,
					min = -1,
					integer = false
				}
			}
		},
		{
			name = "tab two",
			elements = {
				{
					type = "text",
					content = "this tab has nothing lol"
				},
			}
		}
	},
--	initialized = false,
--	focus = 1,
--	activetab = 1,
--	width = 160,
--	height = 160,
}

local font = love.graphics.getFont()

local elementlist = {
	["text"] = {
		draw = function(self, x, y)
			love.graphics.draw(self.text, x, y)
		end,
		create = function(content, id, box, w)
			local self = {}
			self.text = love.graphics.newText(font)
			self.text:setf({consts.black, content}, w, "left")
			self.height = self.text:getHeight()
			return self
		end,
	},
	["tickbox"] = {
		draw = function(self, x, y)
			love.graphics.draw(self.text, x + 16, y)
			if self.box.vars[self.id] then
				love.graphics.draw(ih8ui.img, ih8ui.quads["tickbox_ticked"], x, y)
			else
				love.graphics.draw(ih8ui.img, ih8ui.quads["tickbox_empty"], x, y)
			end
		end,
		create = function(content, id, box, w)
			local self = {}
			self.id = id
			if not box.vars[self.id] then
				box.vars[self.id] = false
			end
			self.text = love.graphics.newText(font)
			self.text:setf({consts.black, content}, w - 20, "left")
			self.height = self.text:getHeight()
			if self.height < 16 then self.height = 16 end
			return self
		end,
		onclick = function(self, x, y, button)
			if button == 1 and x <= 16 and y <= 16 then
				self.box.changesmade = true
				self.box.vars[self.id] = not self.box.vars[self.id]
			end
		end
	},
	["textline"] = {
		draw = function(self, x, y)
			local r,g,b,a = love.graphics.getColor()
			love.graphics.rectangle("fill", x, y, self.width, self.height)
			love.graphics.setColor(0, 0, 0, a)
			self.tl:draw(consts.lightgray, ih8ui.focused_tl_holder == self, x, y)
			love.graphics.setColor(r, g, b, a)
			if self.hint and (#self.tl.text == 0) then
				love.graphics.draw(self.hint, x, y)
			end
		end,
		create = function(content, id, box, w)
			local self = {}
			self.id = id
			if not box.vars[self.id] then
				box.vars[self.id] = ""
			end
			self.width = content.width or w
			if content.hint then
				self.hint = love.graphics.newText(font)
				self.hint:setf({consts.gray, content.hint}, self.width, "left")
			end
			self.tl = textline:new(love.graphics.getFont(), 0, 0, self.width, 16, "", content.properties)
			self.height = 16
			return self
		end,
		onclick = function(self, x, y, button)
			if button == 1 then
				self.box.changesmade = true
				love.keyboard.setKeyRepeat(true)
				ih8ui.focused_tl_holder = self
				self.tl:mousepressed(x, y, button)
			end
		end
	},
}

local img = graphics.load("ui/ih8ui")

ih8ui = { --Interface Helper 8: Usable? Incredibly!
	defaultbox = defaultbox,
	box = defaultbox,
	defaultwidth = 240,
	showingbox = false,
	box_grabbed = false,
	box_closeheld = false,
	focused_tl_holder = false,
	errortext = love.graphics.newText(smallishfont, "oopsie daisy"),
	padding = 8,
	tabpadding = 4,
	spacing = 5,
	horizontal_inbounds = 16,
	titlebar_height = 16,
	close_button_size = 16,
	yesno_height = 24,
	yesno_label_offset = 4,
	tabheight = 16,
	img = img,
	quads = {
		["close"] = quad(0, 0, 16, 16, img),
		["tickbox_empty"] = quad(16, 0, 16, 16, img),
		["tickbox_ticked"] = quad(32, 0, 16, 16, img),
		["page_left"] = quad(48, 0, 16, 16, img),
		["page_right"] = quad(64, 0, 16, 16, img),
	},
}

ih8ui.easybox = {
	title = "easy box",
	tabs = {
		[1] = {
			name = "first tab",
			elements = {
				{
					type = "text",
					content = "welcome to my tab"
				},
				{
					type = "text",
					content = "this is a short one but i want to include at least one text that causes a line wrap",
				}
			}
		},
		[2] = {
			name = "second tab",
			elements = {
				{
					type = "text",
					content = "woah a tick box"
				},
				{
					type = "tickbox",
					content = "enable tickiness? (NOTE: enabling tickness will cause the world to explode)",
					id = "ticktest"
				},
				{
					type = "text",
					content = "favorite word?"
				},
				{
					type = "textline",
					content = {
						hint = "type here",
					},
					id = "tltest",
					check = function(self)
						local input = self.box.vars[self.id]
						
						if input and string.find(input, " ") then
							return false, "word shouldn't have spaces"
						end
						return true
					end
				},
				{
					type = "text",
					content = "favorite color?"
				},
				{
					type = "textline",
					content = {
						width = 108,
						hint = "type a hex code",
						properties = {
							onlychars = consts.chars_hex,
							maxlength = 6,
						}
					},
					id = "tltest2",
					check = function(self)
						local input = self.box.vars[self.id]
						
						if input and #input ~= 6 then
							return false, "hex code should be length 6"
						end
						return true
					end
				},
				{
					type = "text",
					content = "thank you for using this box"
				},
			}
		},
		[3] = {
			name = "third tab",
			elements = {
				{type = "text", content = "the more tabs the better"},
				{type = "text", content = "omg a prefilled textbox!"},
				{
					type = "textline",
					content = {
						hint = "woah...",
					},
					id = "prefilled",
				},
				{type = "text", content = "here's another prefilled one that's supposed to be a hex code but isn't"},
				{
					type = "textline",
					content = {
						width = 108,
						hint = "type a hex code",
						properties = {
							onlychars = consts.chars_hex,
							maxlength = 6,
						},
					},
					id = "invalid",
					check = function(self)
						local input = self.box.vars[self.id]
						
						if input and #input ~= 6 then
							return false, "hex code should be length 6"
						end
						return true
					end
				}
			}
		},
		[4] = {
			name = "fourth tab",
			elements = {
				--{type = "text", content = "more tabs more tabs more tabs"},
				{type = "tickbox", content = "enable the text field", id = "enablejank"},
				{type = "text", content = "maximum jank", req = "enablejank"},
				{
					type = "textline",
					content = {
						hint = "a d v a n c e d",
					},
					id = "janktext",
					req = "enablejank",
				}
			}
		},
		[5] = {
			name = "fifth tab",
			elements = {
				{type = "text", content = "this may be the best tab"},
				{
					type = "tickbox",
					content = "no it isn't (i am lying)",
					id = "liar",
					check = function(self)
						local input = self.box.vars[self.id]
						
						if input then
							return false, "untick this right now >:("
						end
						return true
					end
				},
			}
		},
		[6] = {
			name = "sixth tab",
			elements = {
				{
					type = "tickbox",
					content = "don't tick me",
					id = "notick",
					check = function(self)
						local input = self.box.vars[self.id]
						
						if input then
							return false, "uhh... Fail!"
						end
						return true
					end
				},
				{
					type = "tickbox",
					content = "random tickbox #1",
					id = "random1",
				},
				{
					type = "tickbox",
					content = "random tickbox #2",
					id = "random2",
				},
				{
					type = "tickbox",
					content = "random tickbox #3! the fun never ends",
					id = "random3",
				},
			}
		},
	},
	hasyesno = true,
	initvars = function(vars)
		vars["random1"] = (love.math.random() < 0.5)
		vars["random2"] = (love.math.random() < 0.5)
		vars["random3"] = (love.math.random() < 0.5)
		vars["prefilled"] = "omg there's text here already!"
		vars["invalid"] = "more text woahh"
	end,
	onok = function(self)
	
	end
}

local normalfont = love.graphics.getFont()

ih8ui.pagecontrol = {
	height = 16,
	width = 0,
	y = 0,
	text = love.graphics.newText(font),
	colortext = {consts.black, "placeholder"},
	
	setPageText = function(self, n)
		self.colortext[2] = tostring(n)
		self.text:setf(self.colortext, 16, "center")
	end,
	onclick = function(self, x, y, button)
		if x >= (self.width - 48) then
			local box = ih8ui.box
			local tab = box.tabs[box.activetab]
			if x < (self.width - 32) then --page left
				if button == 2 then
					ih8ui.switchPage(tab.minpage)
				else
					local page = tab.currentpage - 1
					if page < tab.minpage then
						if tab.wrappages then page = tab.maxpage
						else return end
					end
					ih8ui.switchPage(page)
				end
			elseif x >= (self.width - 16) then --page right
				if button == 2 then
					ih8ui.switchPage(tab.maxpage)
				else
					local page = box.tabs[box.activetab].currentpage + 1
					if page > tab.maxpage then
						if tab.wrappages then page = tab.minpage
						else return end
					end
					ih8ui.switchPage(page)
				end
			elseif x < self.width then --text
				ih8ui.pagegoto = true
				self:setPageText("_")
			end
		end
	end,
	draw = function(self, x, y)
		local box = ih8ui.box
		local tab = box.tabs[box.activetab]
		local r,g,b,a = love.graphics.getColor()
		
		if tab.minpage and (not tab.wrappages) and tab.currentpage == tab.minpage then
			love.graphics.setColor(r,g,b,0.5)
		end
		love.graphics.draw(ih8ui.img, ih8ui.quads["page_left"], x + (self.width - 48), y)
		love.graphics.setColor(r,g,b,a)
		love.graphics.draw(self.text, x + (self.width - 32), y)
		if tab.maxpage and (not tab.wrappages) and tab.currentpage == tab.maxpage then
			love.graphics.setColor(r,g,b,0.5)
		end
		love.graphics.draw(ih8ui.img, ih8ui.quads["page_right"], x + (self.width - 16), y)
		love.graphics.setColor(r,g,b,a)
	end,
}

ih8ui.initElements = function(box, e, y) --ey!
	box = box or ih8ui.box
	for i,v in ipairs(e) do
		if elementlist[v.type] then
			v.object = elementlist[v.type].create(v.content, v.id, box, box.width - (2 * ih8ui.padding))
			--so the point of this is that the original tables are modified to include the actual objects alongside their specifications
			--but there's some redundant stuff stored for no reason klike the id, the check function, and the req
			--and for some reason i accounted for if the object decides to change its id (which it shouldn't??)
			if not v.object.id then v.object.id = v.id end --unnecessary and redundant, id doesn't change
			if not v.object.check then v.object.check = v.check end --redundant
			v.object.req = v.req --ditto
			v.object.box = box
			v.object.x = ih8ui.padding
			v.object.y = y
			if not v.object.height then
				error("error: in ui box \"" .. tostring(box.title) .. "\", object of type \"" .. tostring(v.type) .. "\" did not return a height")
			end
			if not v.object.width then v.object.width = box.width - (2 * ih8ui.padding) end
			y = y + v.object.height + ih8ui.spacing
			v.object.draw = elementlist[v.type].draw
			v.object.onclick = elementlist[v.type].onclick
			print("created ih8ui object of type " .. tostring(v.type), "index " .. tostring(i))
		else error("in ui box, unknown type " .. tostring(v.type)) end
	end
	return y --returned if we want to gauge the height the elements take up in the tab
end

ih8ui.initBox = function(box)
	if box.initialized then print("box \"" .. tostring(box.title) .. "\" already initialized"); return box; end
	box.initialized = true
	
	box.x = 0
	box.y = 0
	
	box.vars = {}
	
	box.errors = {}
	box.pre_errors = {}
	box.clicked = {}
	box.clicked_byid = {}
	box.clicked_bytab = {}
	box.changesmade = false
	
	box.focus = 1 --which element is focused
	box.activetab = 1
	if not box.width then box.width = 240 end
	if not box.title then box.title = "(unnamed box)" end
	local boxheight = 0
	local tallest_tab
	--local i = 1
	--local tab_amt = #box.tabs
	--while i <= tab_amt do
	
	--set up tab bar
	box.tabspots = {}
	if #box.tabs > 1 then
		local tabnames = {}
		local tabx = 0
		local taby = 0
		local tabrowwidth = 0
		for i,t in ipairs(box.tabs) do
			if not t.name then error("this tab (#" .. tonumber(i) .. ") doesn't have a name") end
			local tabspot = {}
			tabspot.name = t.name
			tabspot.width = normalfont:getWidth(t.name) + (2 * ih8ui.tabpadding)
			tabspot.height = ih8ui.tabheight
			
			tabrowwidth = tabrowwidth + tabspot.width
			if tabrowwidth > box.width then
				tabrowwidth = 0
				tabx = 0
				taby = taby + ih8ui.tabheight
			end
			
			tabspot.x = tabx
			tabspot.y = taby
			
			tabx = tabx + tabspot.width
			box.tabspots[#box.tabspots + 1] = tabspot
		end
		box.tabbar_height = taby + ih8ui.tabheight
	else
		box.tabbar_height = 0
	end
	
	ih8ui.pagecontrol.y = ih8ui.titlebar_height + box.tabbar_height
	
	--actually initialize tab contents now that height of tab bar is known
	box.tabs_with_weakpages = {}
	for i,t in ipairs(box.tabs) do
		if t.initialized then print("tab \"" .. tostring(tab.name) .. "\" already initialized") end
		if not t.initialized then
			t.i = i
			--if not t.name then t.name = "(unnamed tab)" end
			local y = ih8ui.titlebar_height + box.tabbar_height + ih8ui.spacing
			if t.weakpages then table.insert(box.tabs_with_weakpages, i) end
			if t.createPage and not t.pages then t.pages = {} end
			if t.pages then
				--t.initialized_pages = {}
				t.haspagecontrol = true
				y = y + ih8ui.pagecontrol.height --note: no spacing before pagecontrol
				if not t.minpage then t.minpage = 1 end
				t.currentpage = t.minpage
				if not t.pages[t.minpage] then
					if t.elements then t.pages[t.minpage] = t.elements
					else t.pages[t.minpage] = t.createPage(t.minpage)
					end
				end
				if not t.maxpage then t.maxpage = #t.pages end
				t.elements = t.pages[t.minpage]
			end
			local pages = t.pages or {t.elements}
			for ii,page in pairs(pages) do --PAIRS, not ipairs, since if pages are only filled in by createpage then there might be gaps
				local y_ = y
				y_ = ih8ui.initElements(box, page, y_)
				t.height = y_
				if t.height > boxheight then 
					boxheight = t.height
					tallest_tab = t
				end
			end
			t.initialized = true
		end
	end
	if box.hasyesno then
		box.yesno_y = boxheight
		box.yesno_buttonwidth = ((box.width - (2 * ih8ui.padding)) / 2) - math.floor(ih8ui.padding / 2) --2 * ih8ui.padding represents the padded sides of the box; the last ih8ui.padding represents the padding between the buttons
		box.yesno_button1x = box.x + ih8ui.padding
		box.yesno_button2x = box.x + ih8ui.padding + box.yesno_buttonwidth + ih8ui.padding
		boxheight = boxheight + ih8ui.yesno_height + ih8ui.spacing
	end
	box.height = boxheight
	if box.height > editor.height then --originally planned to manually split tabs if they have too many elements but how about i don't. users are't going to be making ui boxes anyway
		--error(etc)
		print("box \"" .. tostring(box.title) .. "\" a bit too tall for the screen buddy :S (tallest tab is \"" .. tostring(tallest_tab.title) .. "\", tab #" .. tostring(tallest_tab.i))
	end
	box.initialized = true
	return box
end

ih8ui.precheckElements = function(box, e, i, ii) --eiii! okay this is annoying. i means tab number, ii means page number. but "tab" and "page" are generally reserved for the actual tables used by tabs and pages (mainly "tab"). i could write "tabno" but that would be sluggish. maybe i and ii is sluggisher and i just haven't realized it yet. idk
	box = box or ih8ui.box
	for iii,v in ipairs(e) do
		if v.type == "textline" and box.vars[v.id] == nil then
			error("nil var for textline of id " .. tostring(v.id) .. " (should be empty string)")
			--not sure whose responsibility it should be to correct it
		end
		if v.type == "textline" and box.vars[v.id] and box.vars[v.id] ~= v.object.tl:getText() then
			v.object.tl:setText(box.vars[v.id])
		end
		if not v.object then
			print("in precheck, found something w/o object. let's see where it leads.")
			print(iii, v)
			for k,vv in pairs(v) do print(k,vv) end
			do return end
			print("NO OBJECT.")
			print(iii, v, v.type)
			error("no object (see console)")
		end
		if v.object.check then
			local success, message = v.object:check()
			if not success then
				if not box.pre_errors[i] then box.pre_errors[i] = {} end
				if not box.pre_errors[i][ii] then box.pre_errors[i][ii] = {} end
				box.pre_errors[i][ii][iii] = message
			end
		end
	end
end

ih8ui.showBox = function(box)
	if ih8ui.showingbox then
		print("already showing a box!")
		return false
	end
	local state = statemachine.getactivestate()
	if state == editor then
		editor.tooltip = nil
		editor.returnfromshowbox = true
	end
	if not box.initialized then
		ih8ui.initBox(box)
	end
	local gw = love.graphics.getWidth()
	local gh = love.graphics.getHeight()
	box.x = math.floor((gw - box.width) / 2)
	box.y = math.floor((gh - box.height) / 2)
	--ih8ui.pagecontrol.width = box.width - (2 * ih8ui.padding)
	ih8ui.pagecontrol.width = box.width
	if box.initvars then box.initvars(box.vars) end
	for i,t in ipairs(box.tabs) do
		local pages = t.pages or {t.elements}
		for ii,page in pairs(pages) do --PAIRS, not ipairs, since if pages are only filled in by createpage then there might be gaps
			ih8ui.precheckElements(box, page, i, ii)
		end
	end
	ih8ui.box = box
	ih8ui.showingbox = true
	ih8ui.switchTab(1)
	return true
end

ih8ui.drawBox = function(box)
	local r,g,b,a = love.graphics.getColor()
	love.graphics.setColor(0.9, 0.9, 0.9, 1)
	love.graphics.rectangle("fill", box.x, box.y, box.width, box.height)
	love.graphics.setColor(0.6, 0.6, 0.6, 1)
	love.graphics.rectangle("fill", box.x, box.y, box.width, ih8ui.titlebar_height + box.tabbar_height)
	love.graphics.setColor(0, 0, 0, 1)
	love.graphics.print(box.title, box.x, box.y)
	love.graphics.setColor(r,g,b,a)
	love.graphics.draw(ih8ui.img, ih8ui.quads["close"], box.x + (box.width - 16), box.y)
	if #box.tabspots > 0 then
		love.graphics.setColor(0.8, 0.8, 0.8, 1)
		for i,t in ipairs(box.tabspots) do
			local selectedheight = 0
			if box.activetab == i then
				love.graphics.setColor(0.9, 0.9, 0.9, 1)
				selectedheight = 1
			end
			love.graphics.rectangle("fill", box.x + t.x + 1, box.y + t.y + ih8ui.titlebar_height + (2 - selectedheight), t.width - 2, (t.height - 2) + selectedheight)
			love.graphics.print({consts.black, t.name}, box.x + t.x + ih8ui.tabpadding, box.y + t.y + ih8ui.titlebar_height + 1)
			if box.activetab == i then
				love.graphics.setColor(0.8, 0.8, 0.8, 1)
			end
		end
	end
	local tab = box.tabs[box.activetab]
	if tab.haspagecontrol then
		love.graphics.setColor(0.75, 0.75, 0.75, 1)
		love.graphics.rectangle("fill", box.x, box.y + ih8ui.pagecontrol.y, box.width, ih8ui.pagecontrol.height)
		love.graphics.setColor(r,g,b,a)
		--ih8ui.pagecontrol:draw(box.x + ih8ui.padding, box.y + ih8ui.pagecontrol.y)
		ih8ui.pagecontrol:draw(box.x, box.y + ih8ui.pagecontrol.y)
	end
	love.graphics.setColor(r,g,b,a)
	for i,v in ipairs(tab.elements) do
		local errorshere = (box.errors[box.activetab] and box.errors[box.activetab][(tab.currentpage) or 1])
		local pre_errorshere = (box.pre_errors[box.activetab] and box.pre_errors[box.activetab][(tab.currentpage) or 1])
		if	errorshere or pre_errorshere --pre-errors are errors present from initvars (e.g. if a level is loaded with an invalid value in some field). these are not required to be corrected like normal errors are
		then
			local err = errorshere and errorshere[i]
			local pre = pre_errorshere and pre_errorshere[i]
			if err then
				love.graphics.setColor(1, 0, 0)
			elseif pre then
				love.graphics.setColor(1, 0.75, 0)
			else
				print(errorshere, pre_errorshere)
				print(err, pre)
				error("either errorshere or pre_errorshere were truthy, but neither errorshere[i] nor pre_errorshere[i] were")
			end
			local obj = v.object
			love.graphics.rectangle("line", box.x + obj.x, box.y + obj.y, obj.width, obj.height)
			ih8ui.errortext:set({consts.black, err or pre})
			local eth = ih8ui.errortext:getHeight() + 1
			local etw = ih8ui.errortext:getWidth()
			if err then
				love.graphics.setColor(1, 0.5, 0.5, 0.75)
			elseif pre then
				love.graphics.setColor(1, 0.875, 0.5, 0.75)
			end
			love.graphics.rectangle("fill", (box.x + obj.x) + obj.width - etw, box.y + obj.y - eth, etw, eth)
			love.graphics.setColor(r,g,b,a)
			love.graphics.draw(ih8ui.errortext, (box.x + obj.x) + obj.width - etw, box.y + obj.y - (ih8ui.errortext:getHeight() + 1))
		end
		local obj = v.object
		if obj.req and ((not box.vars[obj.req]) or box.vars[obj.req] == "") then
			love.graphics.setColor(r,g,b,0.5)
		end
		obj:draw(box.x + obj.x, box.y + obj.y)
		if obj.req and ((not box.vars[obj.req]) or box.vars[obj.req] == "") then
			love.graphics.setColor(r,g,b,a)
		end
	end
	if box.hasyesno then
		--love.graphics.setColor(0.95, 0.95, 0.95, 1)
		love.graphics.setColor(0.8, 0.8, 0.8, 1)
		local buttonwidth = box.yesno_buttonwidth
		local button1x = box.yesno_button1x
		local button2x = box.yesno_button2x
		if box.yesno_button1hovered then
			love.graphics.setColor(0.85, 0.85, 0.85, 1)
		end
		love.graphics.rectangle("fill", box.x + button1x, box.y + box.yesno_y, buttonwidth, ih8ui.yesno_height)
		
		if next(box.errors) then
			love.graphics.setColor(1, 0, 0)
			love.graphics.rectangle("line", box.x + button1x, box.y + box.yesno_y, buttonwidth, ih8ui.yesno_height)
		end
		if box.yesno_button2hovered then
			love.graphics.setColor(0.85, 0.85, 0.85, 1)
		else
			love.graphics.setColor(0.8, 0.8, 0.8, 1)
		end
		love.graphics.rectangle("fill", box.x + button2x, box.y + box.yesno_y, buttonwidth, ih8ui.yesno_height)
		love.graphics.setColor(0, 0, 0, 1)
		love.graphics.printf("ok", box.x + button1x, box.y + box.yesno_y + ih8ui.yesno_label_offset, buttonwidth, "center")
		love.graphics.printf("cancel", box.x + button2x, box.y + box.yesno_y + ih8ui.yesno_label_offset, buttonwidth, "center")
		love.graphics.setColor(r,g,b,a)
	end
end

ih8ui.switchTab = function(n)
	assert(ih8ui.showingbox, "ih8ui: asked to switch tabs while not showing a box")
	local box = ih8ui.box
	box.activetab = n
	local tab = box.tabs[n]
	if tab.pages then
		ih8ui.switchPage(tab.currentpage)
		--ih8ui.pagecontrol:setPageText(tab.currentpage)
	end
	return tab
end

ih8ui.switchPage = function(n)
	assert(ih8ui.showingbox, "ih8ui: asked to switch pages while not showing a box")
	local box = ih8ui.box
	local tab = box.tabs[box.activetab]
	if not tab.pages then tab.pages = {} end
	if tab.pages[n] then
		tab.currentpage = n
		tab.elements = tab.pages[n]
	else
		if not tab.createPage then
			error("this tab was asked to switch to page " .. tostring(n) .. ", but it doesn't exist, and there's no createpage function")
		end
		local page = tab.createPage(n)
		if not page then
			error("tab.createPage did not supply a page for n=" .. tostring(n))
		end
		tab.pages[n] = page
		tab.elements = page
		local y = ih8ui.titlebar_height + box.tabbar_height + ih8ui.spacing + (ih8ui.pagecontrol.height - ih8ui.spacing)
		ih8ui.initElements(box, tab.elements, y)
		ih8ui.precheckElements(tab.elements, box.activetab, n)
	end
	ih8ui.pagecontrol:setPageText(n)
	return tab.pages[n]
end

ih8ui.closeBox = function()
	local box = ih8ui.box
	ih8ui.box_grabbed = false
	ih8ui.box_closeheld = false
	ih8ui.showingbox = false
	box.focus = 1
	--box.activetab = 1
	love.keyboard.setKeyRepeat(false)
	ih8ui.pagegoto = false
	ih8ui.focused_tl_holder = false
	box.yesno_button1hovered = false
	box.yesno_button2hovered = false
	box.errors = {}
	box.pre_errors = {}
	box.clicked = {}
	box.clicked_byid = {}
	box.clicked_bytab = {}
	box.changesmade = false
	for i,v in ipairs(box.tabs_with_weakpages) do --tabs_with_weakpages itself is not cleared since it's set by initBox
		local t = box.tabs[v]
		if not t.createPage then
			print("this tab has no createPage function; why is it marked as weakpages")
		else
			--t.elements = t.pages[t.minpage]
			--hey np++ renders = slightly different inside comments. what gives
			t.elements = nil --will be set by switchPage() next time the tab is opened
			t.pages = {}
		end
	end
end

ih8ui.keypressed = function(key)
	if key == "esc" then
		ih8ui.closeBox()
		return
	end
	if ih8ui.focused_tl_holder then
		ih8ui.focused_tl_holder.tl:keypressed(key)
	elseif ih8ui.pagegoto then
		local success = false
		local tab = ih8ui.tabs[ih8ui.activetab]
		if tonumber(key) then
			local page = tonumber(key)
			if page > tab.maxpage then page = tab.maxpage
			elseif page < tab.minpage then page = tab.minpage end
			ih8ui.switchPage(page)
			success = true
		end
		ih8ui.pagegoto = false
		if not success then
			ih8ui.pagecontrol:setPageText(tab.currentpage)
		end
	end
end

ih8ui.textinput = function(t)
	if ih8ui.focused_tl_holder then
		ih8ui.focused_tl_holder.tl:textinput(t)
	end
end

ih8ui.mousepressed = function(x, y, button)
	if button > 3 then return end --when we check whether no mouse buttons are down after ih8ui.mousereleased it's easier to check love.mouse.isDown(1, 2, 3) than love.mouse.isDown(1, 2, 3, 4, 5, 6, 7, 8) or however many weird extra mouse buttons sdl can recognize
	local box = ih8ui.box
	local tab = box.tabs[box.activetab]
	
	local clearpagegoto = ih8ui.pagegoto
	ih8ui.pagegoto = false
	if ih8ui.focused_tl_holder then
		local fth = ih8ui.focused_tl_holder
		if not (
			y >= box.y + fth.y and y < box.y + fth.y + fth.height and
			x >= box.x + fth.x and x < box.x + fth.x + fth.width
		) then
			print("defocusing textline")
			box.vars[fth.id] = fth.tl.text
			
			love.keyboard.setKeyRepeat(false)
			ih8ui.focused_tl_holder = false
		else
			fth.tl:mousepressed(x - (box.x + fth.x), y - (box.y + fth.y), button)
		end
	end
	
	if	y >= box.y and y < box.y + box.height and
		x >= box.x and x < box.x + box.width
	then
		if y < box.y + ih8ui.titlebar_height then --grabbing or closing
			if x < box.x + box.width - ih8ui.close_button_size then
				ih8ui.box_grabbed = true
			else
				ih8ui.box_closeheld = true
			end
		elseif (not box.hasyesno) or y < box.y + box.yesno_y then --clicking elements or tab bar
			if y < box.y + ih8ui.titlebar_height + box.tabbar_height then --clicking tab bar
				--print("clicked tab bar")
				for i,t in ipairs(box.tabspots) do
					local ty_rel = t.y + ih8ui.titlebar_height
					--print(t.name)
					--print(x, y)
					--print(box.x + t.x, box.y + ty_rel)
					--print(box.x + t.x + t.width, box.y + ty_rel + t.height)
					if	x >= box.x + t.x and x < box.x + t.x + t.width and
						y >= box.y + ty_rel and y < box.y + ty_rel + t.height
					then
						--print("box active tab is " .. tostring(i))
						ih8ui.switchTab(i)
						clearpagegoto = false --done in switchtab
						break
					end
				end
			elseif tab.haspagecontrol and y < box.y + ih8ui.pagecontrol.y + ih8ui.pagecontrol.height then --clicking page control
				ih8ui.pagecontrol:onclick(x - box.x, y - (box.y + ih8ui.pagecontrol.y), button) --if that y value isn't right, i'll never notice, since pagecontrol onclick doesn't care about y
			else --clicking elements
				for i,v in ipairs(tab.elements) do
					local obj = v.object
					local objx_rel = (obj.x + box.x)
					local objy_rel = (obj.y + box.y)
					if	y > objy_rel and y < objy_rel + obj.height and
						x > objx_rel and x < objx_rel + obj.width
					then
						if (not obj.req) or (obj.req and box.vars[obj.req] and box.vars[obj.req] ~= "") then
							if obj.onclick then
								--box.clicked is used purely internally to iterate through interacted objects for error-checking. all the other clicked variables are for onok's convenience so it knows what the user did or did not interact with and thus what can be skipped over when storing/updating everything
								if tab.pages then
									if not box.clicked_bytab[tab.name] then box.clicked_bytab[tab.name] = {} end
									box.clicked_bytab[tab.name][tab.currentpage] = true
								else
									box.clicked_bytab[tab.name] = true
								end
								if obj.id then
									box.clicked_byid[obj.id] = true
								end
								local page = tab.currentpage or 1
								if not box.clicked[box.activetab] then
									box.clicked[box.activetab] = {}
								end
								if not box.clicked[box.activetab][page] then
									box.clicked[box.activetab][page] = {}
								end
								box.clicked[box.activetab][page][i] = true
								local pre_errorshere = box.pre_errors[box.activetab] and box.pre_errors[box.activetab][tab.currentpage or 1]
								if pre_errorshere then
									pre_errorshere[i] = false
								end
								obj:onclick(x - objx_rel, y - objy_rel, button)
							end
						end
						break
					end
				end
			end
		elseif box.hasyesno and y < box.y + box.yesno_y + ih8ui.yesno_height then --clicking yesno
			if x >= box.x + box.yesno_button1x and x < box.x + box.yesno_button1x + box.yesno_buttonwidth then
				print("oking")
				ih8ui.ok()
			elseif x >= box.x + box.yesno_button2x and x < box.x + box.yesno_button2x + box.yesno_buttonwidth then
				print("cancel :(")
				ih8ui.closeBox()
			end
		end
	end
	if clearpagegoto and not ih8ui.pagegoto then
		ih8ui.pagecontrol:setPageText(tab.currentpage)
	end
end

ih8ui.ok = function()
	local box = ih8ui.box
	box.errors = {}
	local errored = false
	local errorspot = nil
	for i,t in ipairs(box.tabs) do
		local clicked_tab = box.clicked[i]
		if clicked_tab then
			local pages = t.pages or {t.elements}
			for ii,page in pairs(pages) do --PAIRS, not ipairs, since if pages are only filled in by createpage then there might be gaps
				--box.clicked[i][ii]
				local clicked_page = clicked_tab[ii]
				if clicked_page then
					for iii,v in ipairs(page) do
						--box.clicked[i][ii]
						if clicked_page[iii] and v.object.check then
--						if box.clicked[i][ii] and v.object.check and not (box.pre_errors[i] and box.pre_errors[i][ii]) then --pre-errors check is redundant since pre-errors go away as soon as you click on them
							local success, message = v.object:check()
							if not success then
								errored = true
								errorspot = {i, ii}
								if not box.errors[i] then box.errors[i] = {} end
								if not box.errors[i][ii] then box.errors[i][ii] = {} end
								box.errors[i][ii][iii] = message
							end
						end
					end
				end
			end
		end
	end
	if not errored then
		print("oked ok!")
		if box.onok then box:onok() end
		ih8ui.closeBox()
	else
		print("un-ok!!")
		local i = errorspot[1]
		local ii = errorspot[2]
		if not (i == box.activetab and ((not box.tabs[i].currentpage) or ii == box.tabs[i].currentpage)) then
			local tab = ih8ui.switchTab(errorspot[1])
			if tab.pages then ih8ui.switchPage(errorspot[2]) end
		end
		--[[if (not box.errors[box.activetab]) or (not next(box.errors[box.activetab])) then
			for i,tab in ipairs(box.tabs) do
				if box.errors[i] then
					--box.activetab = i
					ih8ui.switchTab(i)
					print("saw an error in this tab")
					if tab.pages then
						print("has pages...")
						for ii,_ in pairs(tab.pages) do --PAIRS, not ipairs, since if pages are only filled in by createpage then there might be gaps
							print(box.errors[i][ii])
							if box.errors[i][ii] then
								ih8ui.switchPage(ii)
							end
							break
						end
					end
					break
				end
			end
		end]]
	end
end

ih8ui.mousereleased = function(x, y, button)
	if not love.mouse.isDown(1, 2, 3) then
		ih8ui.box_grabbed = false
		if ih8ui.box_closeheld then
			ih8ui.closeBox()
		end
	end
	if ih8ui.focused_tl_holder then
		local fth = ih8ui.focused_tl_holder
		fth.tl:mousereleased()
	end
end

ih8ui.boxinbounds = function()
	local gw = love.graphics.getWidth()
	local gh = love.graphics.getHeight()
	local box = ih8ui.box
	--local tbh = ih8ui.titlebar_height --tbh
	
	if gh - box.y < ih8ui.titlebar_height then
		box.y = gh - ih8ui.titlebar_height
	elseif box.y < 0 then
		box.y = 0
	end
	if gw - box.x < ih8ui.horizontal_inbounds then
		box.x = gw - ih8ui.horizontal_inbounds
	elseif box.x + box.width < (ih8ui.horizontal_inbounds + ih8ui.close_button_size) then
		box.x = (ih8ui.horizontal_inbounds + ih8ui.close_button_size) - box.width
	end
end

ih8ui.mousemoved = function( x, y, dx, dy, istouch )
	local box = ih8ui.box
	if ih8ui.focused_tl_holder then
		local fth = ih8ui.focused_tl_holder
		fth.tl:mousemoved(x - (box.x + fth.x), y - (box.y + fth.y), dx, dy, istouch )
	end
	box.yesno_button1hovered = false
	box.yesno_button2hovered = false
	if ih8ui.box_grabbed then
		ih8ui.box.x = ih8ui.box.x + dx
		ih8ui.box.y = ih8ui.box.y + dy
		ih8ui.boxinbounds()
		if	x > love.graphics.getWidth() or x < 0 or
			y > love.graphics.getHeight() or y < 0
		then
			ih8ui.box_grabbed = false
		end
	elseif ih8ui.box_closeheld then
		if	x < box.x + box.width - ih8ui.close_button_size or
			x > box.x + box.width or
			y < box.y or
			y > box.y + ih8ui.titlebar_height
		then
			ih8ui.box_closeheld = false
		end
	else
		if y >= box.y + box.yesno_y and y < box.y + box.yesno_y + ih8ui.yesno_height then
			if x >= box.x + box.yesno_button1x and x < box.x + box.yesno_button1x + box.yesno_buttonwidth then
				box.yesno_button1hovered = true
			elseif x >= box.x + box.yesno_button2x and x < box.x + box.yesno_button2x + box.yesno_buttonwidth then
				box.yesno_button2hovered = true
			end
		end
	end
end

ih8ui.update = function(dt)
	if ih8ui.focused_tl_holder then
		ih8ui.focused_tl_holder.tl:update()
	end
end

ih8ui.initBox(ih8ui.easybox)