commandswithlevelsymbolatend = {
	["rei"] = true,
}
level = class:new()

--[[
	objects = {},
	tilemap = {}, --multiple tiles actually can go on each coordinate so each square here is represented by a table
	options = {},
	filename = nil,
	exits = {},
	width = 512,
	height = 512,
	background = "#341160",
	frozen = false
]]

level.dontclone = set("tilemap", "objects", "objects_w_earlyupdate", "symbolmaps", "tagmaps", "particles", "particles_set", "animtimer", "temps", "won") --tables are excluded (though not all exclusions are tables). 
--ogmo_spawns is also a table, but it's a table that will never have any practical reason to be modified, so don't bother making a special recursive case for it
--npcs is only modified to split instances of \n into (actual newline)

--i think level.clone is only used for editor clipboard now. previously it was used to very roughly store the initial state of the level for clipboards (re-initializing the level from its preliminary elements turned out to be a lot less buggy)
function level.clone(base, early)
	local newlevel = level:new()
	if not early then
		error("deprecated sorry")
	elseif early ~= "ctrlc" then
		--copies a pre-gamified level but doesn't care about tagmaps and symbolmaps (and other stuff)
		for k,v in pairs(base) do
			newlevel[k] = v --this includes tagmaps and symbolmaps and uses the exact same tables, which is fine because those tables should never be modified ingame after the level initialization
		end
	else
		for k,v in pairs(base) do
			if not level.dontclone[k] then
				newlevel[k] = v
			end
		end
		--recursion
		--this basically achieves the same thing as the more manual copying of tilemap/objmap when cloning gamified levels but i forgor the specifics of how tagmaps and symbolmaps are structured
		local recursecopy
		recursecopy = function(t, depth)
			local new = {}
			for k,v in pairs(t) do
				if type(v) == "table" then
					if depth > 1 then
						new[k] = recursecopy(v, depth - 1)
					else
						new[k] = v
					end
				else
					new[k] = v
				end
			end
			return new
		end
		--symbolmaps is a table of matrices
		
		--depth 1: copying a table with non-tables
		--depth 2: copying a table of tables (matrix)
		--depth 3: copying a table of tables of tables (table of matrices)
		newlevel.symbolmaps = recursecopy(base.symbolmaps, 3)
		
		--tagmaps is a table of matrices, where each cell in the matrix is a table (since one cell can have multiple tags)
		--depth 4: copying a table of tables of tables of tables (table of matrices of tables)
		newlevel.tagmaps = recursecopy(base.tagmaps, 4)
		
		--god i hope this is right
		--i could just copy everything with recursecopy(blah, 10) - including other stuff like tilemaps in gamified clones, not just the tagmaps and symbolmaps - but 1. the only nested tables pre-gamification levels are supposed to have are symbolmaps and tagmaps and 2. i want to still kind of know what i'm doing and be responsible for any mistakes
		--main reason i don't just want to recursively copy everything with depth 100 is that it takes one instance of a table referencing itself for some reason in order to fuck things up. user input will eventually be a thing and i cannot trust it
		--(though obj:new() in the cloning gamified objects thing is almost definitely a weird case of "well it started that way and i'm able to work around it so i'm not changing it" though. this is literally my first lua project ever
	end
	
	return newlevel
end

function level:findcamerafocus() --wip, ideally a player coming out of an exit should have some special "prioritize me" property. additionally there should be a hierarchy for camera priority: ogmo, then blob, then have it set up so new player-type objects that cam should follow can have priority similarly set
	self.lastcamerafocusX = nil
	self.lastcamerafocusY = nil
	local newfocus
	for i,obj in ipairs(self.objects) do
		if obj.player and (not obj.ghost) and obj.alive then
			newfocus = obj
			break
		end
	end
	self.camerafocus = newfocus
	statusbar.update(nil, self)
end

function level:updatecamera()
	if self.liveplayeramt > 0 and ((not self.camerafocus) or (not self.camerafocus.alive)) then self:findcamerafocus() end
	if self.camerafocus then
		if
			self.lastcamerafocusX == self.camerafocus.x and
			self.lastcamerafocusY == self.camerafocus.y --for completeness should include checking against width and height. but in this game objects don't tend to squish and stretch much
		then
			return
		end
		self.lastcamerafocusX = self.camerafocus.x
		self.lastcamerafocusY = self.camerafocus.y
		local vp = game.viewport
		self.cameraX = (self.camerafocus.x_clamped or self.camerafocus.x) + (self.camerafocus.width / 2) - (vp.width / 2)
		--self.cameraX = self.camerafocus.x + (self.camerafocus.width / 2) - (vp.width / 2)
		self.cameraX = math.floor(self.cameraX + .5)
		if not self.cameraX then print("self.cameraX is nil") end
		if not vp.width then print("vp.width is nil") end
		if not self.width then print("self.width is nil") end
		if self.cameraX + vp.width > self.width then self.cameraX = self.width - vp.width end
		if self.cameraX < 0 then self.cameraX = 0 end
		
		local focusY = self.camerafocus.y_clamped or self.camerafocus.y
		if self.camerafocus.calculate_y_drawat then focusY = self.camerafocus:calculate_y_drawat() end
		local focusheight = self.camerafocus.height -- kludge to ignore reduced height from duck, which can affect the positioning
		if self.camerafocus.ducking then focusheight = self.camerafocus.height + 3 end
		self.cameraY = focusY + (focusheight / 2) - (vp.height / 2)
		self.cameraY = math.floor(self.cameraY + .5)
		if self.cameraY + vp.height > self.height then self.cameraY = self.height - vp.height end
		if self.cameraY < 0 then self.cameraY = 0 end
	else
		statusbar.nofocus()
	end
end

--lame stuff
function level:getMax(kind)
	if not self.counts[kind] then return 0 end
	return self.counts[kind]["max"] or 0
end
function level:getAmt(kind)
	if not self.counts[kind] then return 0 end
	return self.counts[kind]["amt"] or 0
end
function level:amtIsMax(kind)
	if not self.counts[kind] then return false end
	return ((self.counts[kind]["amt"] or 0) == (self.counts[kind]["max"] or 0))
end
function level:setMax(kind, amt)
	if not self.counts[kind] then self.counts[kind] = {amt = 0, ["max"] = 0} end
	self.counts[kind]["max"] = amt
end
function level:setAmt(kind, amt)
	if not self.counts[kind] then self.counts[kind] = {amt = 0, ["max"] = 0} end
	self.counts[kind]["amt"] = amt
end
function level:incMax(kind)
	if not self.counts[kind] then self.counts[kind] = {amt = 0, ["max"] = 0} end --npp highlighting scares me
	self.counts[kind]["max"] = self.counts[kind]["max"] + 1
end
function level:incAmt(kind)
	if not self.counts[kind] then self.counts[kind] = {amt = 0, ["max"] = 0} end
	self.counts[kind]["amt"] = self.counts[kind]["amt"] + 1
end
function level:decMax(kind)
	if not self.counts[kind] then self.counts[kind] = {amt = 0, ["max"] = 0} end
	self.counts[kind]["max"] = self.counts[kind]["max"] - 1
	if self.counts[kind]["max"] < 0 then error("decremented " .. tostring(kind) .. " max under 0") end
end
function level:decAmt(kind)
	if not self.counts[kind] then self.counts[kind] = {amt = 0, ["max"] = 0} end
	self.counts[kind]["amt"] = self.counts[kind]["amt"] - 1
	--if self.counts[kind]["amt"] < 0 then error("decremented " .. tostring(kind) .. " amt under 0? seems bad? idk i just wanted u to know") end
end
function level:resetAmt(kind)
	if not self.counts[kind] then self.counts[kind] = {amt = 0, ["max"] = 0} end
	self.counts[kind]["amt"] = 0
end
--lame stuff over (yay)

function level:marktemp(t, k)
	if not self.temps[t] then self.temps[t] = {} end
	self.temps[t][k] = true
end

function level:checktemp(t, k)
	if not self.temps[t] then return false end
	return self.temps[t][k]
end

function level:init(levelstring, filepath, x, y)
	--print("NEW LEVEL AT " .. tostring(x) .. "," .. tostring(y) .. " (" .. tostring(self) .. ")")
	if levelstring then
		if type(levelstring) ~= "string" then error("provided a non-string argument to level.init (type " .. type(levelstring) .. ", tostring " .. tostring(levelstring) .. ")", 3) end
		--if levelstring == "" then error("gave empty level string to level:new()! bad sign", 3) end
		local phase = nil
		local subphase = nil
		
		levelstring = levelstring:correctnewlines()
		levelstring = split(levelstring, "\n")
		
		self.roomX = x
		self.roomY = y
		self.exits = {}
		--print(x, y)
		--[[if x and y then --moved to gamify
			print("x and y exist")
			self.exits["up"] =    tostring(x) .. "," .. tostring(y - 1)
			self.exits["down"] =  tostring(x) .. "," .. tostring(y + 1)
			self.exits["left"] =  tostring(x - 1) .. "," .. tostring(y)
			self.exits["right"] = tostring(x + 1) .. "," .. tostring(y)
			
		end]]
		self.exits[0] = "WINLEVELSET"
		
		self.filepath = filepath
		self.comments = {}
		self.symbolmaps = {}
		self.tagmaps = {}
		self.music = nil
		self.options = {}
		self.background = nil
		self.hints = {}
		self.metalevels = {}
		self.npcs = {}
		
		local readingtags = false
		
		local y_tiled = nil
		local maplength = nil
		local mapheight = nil
		
		local symbolmapisempty = true
		
		local currentnpc = nil
		local npctext = nil
		
		for i, row in ipairs(levelstring) do
			local justsetphase = false
			if string.sub(row, 1, 3) == "===" and string.sub(row, -3, -1) == "===" then
				readingtags = false
				if phase == "MAP" and symbolmapisempty then
					print("uh oh, layer " .. subphase .. " was empty")
					self.symbolmaps[subphase].isempty = true
				end
				if currentnpc then
					table.insert(self.npcs, {name = currentnpc, text = npctext})
				end
				currentnpc = nil
				npctext = nil
				local phasedata = split(string.sub(row, 4, -4), ":")
				phase, subphase = phasedata[1], phasedata[2]
				if phase == "MAP" then
					if y_tiled and not mapheight then mapheight = y_tiled
					elseif y_tiled and mapheight and mapheight ~= y_tiled then
						print("mapheight not consistent at new map (mapheight: " .. tostring(mapheight) .. ", y_tiled: " .. tostring(y_tiled) .. ", subphase: " .. tostring(subphase) .. ")")
						mapheight = y_tiled
					end
					if not subphase then subphase = 1 end
					if cheat.isactive("stackem") and statemachine.currentstate ~= editor and not game.editormode then subphase = 1 end
					if type(subphase) ~= "number" then subphase = tonumber(subphase) end
					self.tagmaps[subphase] = false
					y_tiled = nil
				end
				if tonumber(subphase) ~= nil then subphase = tonumber(subphase) end
				justsetphase = true
				symbolmapisempty = true
			else
				local mode = "none"
				if string.sub(row, 1, 1) == "|" then mode = "data"; row = string.sub(row, 2, -1)
				elseif string.sub(row, 1, 1) == ">" then mode = "command"; row = string.sub(row, 2, -1)
				end
				
				if mode == "command" then
					local commandtable = split(row, ":", 1)
					local command = commandtable[1]
					local argument = commandtable[2]
					if command == "len" then
						maplength = tonumber(argument)
						self.width = maplength * tilesize
					end
					if phase == "MAP" then
						if command == "row" then
							row = string.rep(argument, maplength)
							mode = "data"
						elseif command == "rep" then
							if y_tiled == nil then error("y_tiled is nil when rep command called - pretty sure you don't have a row to rep?") end
							for ii=1, tonumber(argument) do
								local sm = table.copy(self.symbolmaps[subphase][y_tiled])
								y_tiled = y_tiled + 1
								table.insert(self.symbolmaps[subphase], sm)
								if maplength == nil then
									maplength = #row / 2
									if math.modf(maplength) ~= maplength then error("length of a row of this map (in ascii chars, excluding opening |) is odd?") end
									self.width = maplength * tilesize
								end
							end
						elseif command == "tag" then
							readingtags = true
							if y_tiled and not mapheight then mapheight = y_tiled
							elseif y_tiled and mapheight and mapheight ~= y_tiled then
								print("mapheight not consistent at tagging (mapheight: " .. tostring(mapheight) .. ", y_tiled: " .. tostring(y_tiled) .. ", subphase: " .. tostring(subphase) .. ")")
								mapheight = y_tiled
							end
						else print("unrecognized level command \"" .. command .. "\"") end
					elseif phase == "NPCS" then
						if currentnpc then
							table.insert(self.npcs, {name = currentnpc, text = npctext})
						end
						currentnpc = row
						npctext = {}
					end
				end
				
				if mode == "data" then --this is NOT an elseif because commands can transition to data mode
					if not phase then
						table.insert(self.comments, row)
					elseif phase == "MAP" then
						if not readingtags then	
							if not y_tiled then y_tiled = 0 end
							y_tiled = y_tiled + 1
							if maplength == nil then
								maplength = #row / 2
								if math.modf(maplength) ~= maplength then error("length of a row of this map (in ascii chars, excluding opening |) is odd?") end
								self.width = maplength * tilesize
							--elseif maplength ~= #row then print("mate this row length is inconsistent... subphase: " .. subphase .. ", map length: " .. maplength .. " row length: " .. #row .. ", row (following line):\n" .. row .. "|end")
							end
							if #row < (maplength * 2) then
								row = row .. string.rep("  ", maplength - #row)
							end
							if #row > (maplength * 2) then
								error("row is longer than expected length in chars per row (set by first row in first layer or >len) - #row is " .. tostring(#row) .. ", maplength is " .. tostring(maplength), 2)
							end
							
							local splitrow = nwidesplit(row, "", 2)
							if symbolmapisempty then
								for ii=1, #splitrow do
									if splitrow[ii] ~= "  " then
										symbolmapisempty = false
										break
									end
								end
							end
							if not self.symbolmaps[subphase] then self.symbolmaps[subphase] = {} end
							table.insert(self.symbolmaps[subphase], splitrow)
						else
							if not self.tagmaps[subphase] then
								if not cheat.isactive("stackem") then
									self.tagmaps[subphase] = editor.makeEmptyTagMap(maplength, mapheight)
								else
									--local symbolmapsamt = 0
									--for k,v in pairs(self.symbolmaps) do if not v.isempty then symbolmapsamt = symbolmapsamt + 1 end end
									--self.tagmaps[subphase] = editor.makeEmptyTagMap(maplength, mapheight * symbolmapsamt)
									self.tagmaps[subphase] = editor.makeEmptyTagMap(maplength, mapheight)
								end
							end
							local tagdata = split(row, ":")
							local tag = tagdata[1]
							local tagcoords = split(tagdata[2], "-")
							local tagcoords1 = split(tagcoords[1], ",")
							local tagcoords2 = split(tagcoords[2], ",")
							local tagmap = self.tagmaps[subphase]
							--local y1, y2 = tagcoords1[2], tagcoords2[2]
							--local x1, x2 = tagcoords1[1], tagcoords2[1]
							--if cheat.isactive("stackem") then
							--
							--end
							for y=tagcoords1[2], tagcoords2[2] do
								for x=tagcoords1[1], tagcoords2[1] do
									if not tagmap[y][x] then
										tagmap[y][x] = {tag}
									else
										table.insert(tagmap[y][x], tag)
									end
								end
							end
						end
					elseif phase == "EXITS" then
						table.insert(self.exits, row)
					elseif phase == "HINTS" then
						table.insert(self.hints, row)
					elseif phase == "METALEVELS" then
						table.insert(self.metalevels, row)
					elseif phase == "MUSIC" then
						--music is only supposed to be one row because you can't have more than one track playing
						if self.music then print("loadlevel: changing music when it's already defined??? old val: " .. self.music) end
						self.music = row
					elseif phase == "OPTIONS" then
						--table.insert(self.options, row)
						self.options[row] = true
					elseif phase == "BACKGROUND" then
						if self.background then print("loadlevel: changing background when it's already defined??? old val: " .. self.background) end
						self.background = row
					elseif phase == "NAME" then
						if self.name then print("loadlevel: i guess a level can have two names but can they be on 1 line please. old val: " .. self.name) end
						self.name = row
					elseif phase == "NPCS" then
						table.insert(npctext, row)
					else
						--if not self[string.lower(phase)] then self[string.lower(phase)] = {} end
						--table.insert(self[string.lower(phase)], row)
					end
				end
			end
		end
		if currentnpc then
			table.insert(self.npcs, {name = currentnpc, text = npctext})
		end
		for k,v in pairs(self.npcs) do
			if not next(v.text) then
				self.npcs[k] = nil
			end
		end
		if not self.background then self.background = "#341160" end
		if not self.music then self.music = "" end
		self.maplength = maplength --could potentially help out editor a bit to already know what the map length is and then only have to recalculate it when it places
		--i'm pretty sure self.maplength actually ends up 1 larger than the actual map length
	end
end

function level:invertvoltage()
	local tilemap = self.tilemap
	--local invertkey = level.voltageinv_tilekey
	local processedcombos = {}
	for y,_ in ipairs(tilemap) do
		for x,_ in ipairs(tilemap[y]) do
			for i,tile in ipairs(tilemap[y][x]) do
				if type(tile) == "string" then
					if tiles[tile].voltageinv then
						tilemap[y][x][i] = tiles[tile].voltageinv
					end
				elseif tile.voltageinv and not processedcombos[tile] then
					--neat shortcut: combos (tagged tiles) each share the same table to save on memory, so for each combination of tags, only one table has to be changed
					local newtile = makecomplextile(tiles[tile.type].voltageinv, x, y)
					for k,v in pairs(newtile) do
						if k ~= "fake" and k ~= "tags" then
							tile[k] = v
						end
					end
					processedcombos[tile] = true
				end
			end
		end
	end
	for i,obj in ipairs(self.objects) do
		if obj.invertvoltage then
			obj:invertvoltage()
		end
	end
end

function level:cleanupfromgame() --For returning to the editor from a gamified level, when you don't need the stuff from gamification anymore. Saves on memory (if GC comes by, anyway) and resets some variables that need to be reset between gamifications.
	--important
	self.skips = nil
	
	--memory-savers
	self.objects = nil
	self.objects_w_earlyupdate = nil
	self.tilemap = nil
	self.objmap = nil
	self.particles = nil
	self.particles_set = nil
end

--local empty = set("  "," #")

function level:gamify(entrance)
	self.allgems = {}
	self.validexits = {}
	self.doflipscreenexit = false
	local pos
	if self.roomX and self.roomY then --hopefully always???
		local x = self.roomX
		local y = self.roomY
		pos = tostring(x) .. "," .. tostring(y)
		if game.unsavedgems["blue"][pos] or game.collectedgems["blue"][pos] then self.allgems["blue"] = true end
		if game.unsavedgems["red"][pos] or game.collectedgems["red"][pos] then self.allgems["red"] = true end
		
		local up =    tostring(x) .. "," .. tostring(y - 1)
		local down =  tostring(x) .. "," .. tostring(y + 1)
		local left =  tostring(x - 1) .. "," .. tostring(y)
		local right = tostring(x + 1) .. "," .. tostring(y)
		self.exits["up"] = up
		self.exits["down"] = down
		self.exits["left"] = left
		self.exits["right"] = right
		
		if game.flipscreen then --make it so it doesn't have to call love.filesystem.exists a bunch. have some "flipscreenextantrooms" variable that's a bunch of string keys that gets filled out when it checks for love.filesystem.exists and tries to check there first
		--oops i mean getInfo
			if love.filesystem.getInfo(game.levelsetpath .. "/levels/" .. (up .. ".txt")) then
				self.validexits["up"] = true
			end
			if love.filesystem.getInfo(game.levelsetpath .. "/levels/" .. (down .. ".txt")) then
				self.validexits["down"] = true
			end
			if love.filesystem.getInfo(game.levelsetpath .. "/levels/" .. (left .. ".txt")) then
				self.validexits["left"] = true
			end
			if love.filesystem.getInfo(game.levelsetpath .. "/levels/" .. (right .. ".txt")) then
				self.validexits["right"] = true
			end
		end
	end
	--print("GAMIFYING LEVEL WITH ROOMX " .. tostring(self.roomX) .. " AND ROOMY " .. tostring(self.roomY) .. "(" .. tostring(self) .. ")")
	
	self.animtimer = 0
	self.cameraX = 0
	self.cameraY = 0
	if type(entrance) == "table" then self.entrance = entrance
	else
		if tonumber(entrance) then entrance = tonumber(entrance) end
		self.entrance = {
			id = entrance,
			careening = false,
			jumpsleft = 1,
			hmom = 0,
			vmom = 0
		}
		--if type(entrance) ~= "number" then self.entrance.number = 0; self.entrance.miscdata = entrance; end
	end
	self.won = false
	self.frozen = false
	self.objects = {}
	self.objects_w_earlyupdate = {}
	self.tilemap = {}
	self.objmap = {}
	self.updateobjmap_queued = {}
	self.counts = {}
	self.gravity = 1
	self.playeramt = 0
	self.ogmo_spawns = {}
	self.particles = {}
	self.particles_set = {}
	
	self.temps = {}
	
	local layerkeys = {}
	for k,_ in pairs(self.symbolmaps) do
		table.insert(layerkeys, k)
	end
	table.sort(layerkeys, function(a,b) return a < b end)
	
	--"Skips" are a way of indicating which cells contain no tiles or objects and can be skipped on future gamifications of this level.
	--This is to bring the method of resetting levels by re-gamifying them up to the same speed as the old method of recursive-copying self.tilemap and self.objects. I.e. if there's a bunch of mostly-empty layers, that shouldn't significantly increase how much time it takes to load everything.
	--In practice this is probably an atrocious micro-optimization. But it's a *sound* atrocious micro-optimization.
	local makeskips = false
	if not self.skips then
		makeskips = true
		self.skips = {}
	else
		print("layer skips found!")
	end
	local lastskip = {}
	
	local e = {}
	
	local directions = {"down", "right", "up", "left"}
	local late_init = {}
	
	if self.options.resetvoltage then game.solid_colors.voltageinv = false end
	if self.options.resetelectricity then game.solid_colors.electricity = true end
	if self.options.resetredblue then
		game.solid_colors.red = true
		game.solid_colors.blue = false
	end
	if self.options.resetcyanyellow then
		game.solid_colors.cyan = true
		game.solid_colors.yellow = false
	end
	
	--actual map parsing
	local fluidedges = {}
	local tagcombos = {} --save on creating extra tables for tiles with the same combination of tags (e.g. visual tags)
	local firstlayer = true --Gets set to false at the end of the outer loop below.
	for _,i in ipairs(layerkeys) do
		local map = self.symbolmaps[i]
		if makeskips then
			self.skips[i] = {}
		end
		for y_tiled,row in ipairs(map) do
			if self.tilemap[y_tiled] == nil then
				self.tilemap[y_tiled] = {}
				self.objmap[y_tiled] = {}
			end
			
			local x_tiled = 1
			local rowlength = #row
			--for x_tiled=1, #row do
			while x_tiled <= rowlength do --x_tiled is incremented at the end of the loop, unless skip is true.
				local skip = false
				local rawsymbol
				local symbol
				if makeskips then --Generate or extend a skip.
					rawsymbol = row[x_tiled]
					symbol = levelsymbols[rawsymbol]
					if not symbol then error ("not a valid symbol: " .. row[x_tiled]) end
					if #symbol.tiles == 0 and #symbol.objects == 0 then
						if next(lastskip) then
							self.skips[i][lastskip.y][lastskip.x] = self.skips[i][lastskip.y][lastskip.x] + 1
						else
							lastskip.y = y_tiled
							lastskip.x = x_tiled
							if not self.skips[i][y_tiled] then self.skips[i][y_tiled] = {} end
							self.skips[i][y_tiled][x_tiled] = 1
						end
						if not firstlayer then --It's important none of the tiles on the first layer are skipped, so the objmap and tilemap can be properly established. (This is something the old method half-assed; it just copied over the original objmap.)
							skip = true
							x_tiled = x_tiled + 1 --Since skipping prevents x_tiled from being incremented.
						end
					else
						lastskip.y = nil
						lastskip.x = nil
					end
				elseif (not firstlayer) and self.skips[i][y_tiled] and self.skips[i][y_tiled][x_tiled] then
					local skipamt = self.skips[i][y_tiled][x_tiled]
					--print("skip of " .. tostring(skipamt))
					x_tiled = x_tiled + skipamt --skip prevents x_tiled from being incremented, so x_tiled will be advanced by exactly this amount for the next iteration.
					skip = true
				end
				if not skip then --No continue statement. Boo. I guess there's goto.
					rawsymbol = rawsymbol or row[x_tiled]
					symbol = symbol or levelsymbols[rawsymbol]
					if not symbol then error ("not a valid symbol: " .. row[x_tiled]) end
					for ii=1, #symbol.objects do
						local obj = symbol.objects[ii]
						local notghost = true
						local options = {}
						local temp = split(obj, "|")
						if #temp >  1 then
							obj = temp[1]
							for iii=2, #temp do
								temp[iii] = split(temp[iii], ":")
								options[temp[iii][1]] = temp[iii][2] or "yes"
								if temp[iii][1] == "ghost" then notghost = false end
							end
						end
						--[[
						local temp = split(obj, ";")
						if #temp >  1 then
							obj = temp[1]
							temp = split(temp[2], "|")
							for iii=1, #temp do
								table.insert(options, temp[iii])
							end
						end
						]]
						
						
						if objects[obj] == nil then print("THIS AIN'T AN OBJECT CHAMP: " .. obj) end
						
						local tags = table.copy((self.tagmaps[i] and self.tagmaps[i][y_tiled] and self.tagmaps[i][y_tiled][x_tiled]) or e)
						local tags_reverse = {}
						
						local spawnids = {}
						local dospawn = true
						
						if symbol.hasplayer and fluidedges[y_tiled] and fluidedges[y_tiled][x_tiled] then
							table.insert(tags, "fluid")
							table.insert(tags, "spawn" .. consts["oppositedirs"][fluidedges[y_tiled][x_tiled]])
						end
						
						for i,tag in ipairs(tags) do
							tags_reverse[tag] = true
							if string.sub(tag, 1, 5) == "spawn" then
								local id = string.sub(tag, 6)
								if tonumber(id) then id = tonumber(id) end
								if id ~= "blue" and id ~= "red" then
									spawnids[id] = true
								else
									if not game.collectedgems[id][pos] then
										dospawn = false
										break
									end
								end
							end
						end
						
						if dospawn and obj == "ogmo" and notghost then
							for k,v in pairs(spawnids) do
								self.ogmo_spawns[k] = self.ogmo_spawns[k] or {}
								table.insert(self.ogmo_spawns[k], {(x_tiled - 1) * tilesize, (y_tiled - 1) * tilesize})
							end
						end
						
						if dospawn and ( (not next(spawnids)) or spawnids[self.entrance.id] ) then --if there are no "only spawn if player came from entrance x" tags, or there is at least one tag such but the entrance is one of those tags, then proceed
							local edge = nil
							local magicvalue = nil
							local iscamerafocus = false
							for i,v in ipairs(directions) do
								if tags_reverse["edge" .. v] then edge = v end
							end
							if tags_reverse["edgenone"] then edge = false end
							
							if tags_reverse["camerafocus"] or tags_reverse["fluid"] then iscamerafocus = true end
							
							if edge == nil then
								if     y_tiled == #map then
									if     x_tiled == 1    and map[y_tiled - 1][x_tiled] == rawsymbol then edge = "left"
									elseif x_tiled == #row and map[y_tiled - 1][x_tiled] == rawsymbol then edge = "right"
									else edge = "down" end
								elseif y_tiled == 1 then
									if     x_tiled == 1    and map[y_tiled + 1][x_tiled] == rawsymbol then edge = "left"
									elseif x_tiled == #row and map[y_tiled + 1][x_tiled] == rawsymbol then edge = "right"
									else edge = "up" end
								elseif x_tiled == 1 then edge = "left"
								elseif x_tiled == #row then edge = "right" end
							end
							
							if symbol.fluidwin then
								if not fluidedges[y_tiled] then fluidedges[y_tiled] = {} end
								fluidedges[y_tiled][x_tiled] = edge --wins on the corner of a level aren't generally supported
							end
							
							local newobj = objects[obj]:setup((x_tiled - 1) * tilesize, (y_tiled - 1) * tilesize, options, self, edge, tags_reverse)
							if newobj then --object creation functions can reject creating an object at all
								if not newobj.tags then newobj.tags = tags_reverse end
								--if tags_reverse["invisible"] and not newobj.no_autoinvisible then newobj.invisible = true end
								--if tags_reverse["silhouette"] and not newobj.no_autosilhouette then newobj.silhouette = true end
								if newobj.tags["fake"] then newobj.fake = true end
								if newobj.level then print("object overrode its level?") end
								if newobj.level == nil then newobj.level = self end --unsure why you'd want to override level but that's fine
								table.insert(self.objects, newobj)
								if newobj.earlyUpdate then table.insert(self.objects_w_earlyupdate, newobj) end
								if newobj.late_init then table.insert(late_init, newobj) end
								if iscamerafocus then self.camerafocus = newobj end
								if game.solid_colors.voltageinv and newobj.invertvoltage then
									newobj:invertvoltage()
								end
							end
						end
					end
					local temptilesarray = {}
					for ii=1, #symbol.tiles do
					
						local tags = (self.tagmaps[i] and self.tagmaps[i][y_tiled] and self.tagmaps[i][y_tiled][x_tiled]) or e
						local tile = symbol.tiles[ii]
						if game.solid_colors.voltageinv and tiles[tile].voltageinv then tile = tiles[tile].voltageinv end
						if #tags == 0 then
							table.insert(temptilesarray, tile)
						else
							local tags_reverse = {}
							
							local spawnids = {}
							local dospawn = true
							
							for i,tag in ipairs(tags) do
								tags_reverse[tag] = true
								if string.sub(tag, 1, 5) == "spawn" then
									local id = string.sub(tag, 6)
									if tonumber(id) then id = tonumber(id) end
									if id ~= "blue" and id ~= "red" then
										spawnids[id] = true
									else
										if not game.collectedgems[id][pos] then
											dospawn = false
											break
										end
									end
								end
							end
							--combos
							if dospawn and ( (not next(spawnids)) or spawnids[self.entrance.id] ) then
								--local c = table.concat(tags, ",") .. tile
								local c = table.concat{table.concat(tags, ","), ";", tile}
								--TODO: make sure tags can't contain , or ;
								--unless user just doesn't get to provide tags.
								if not tagcombos[c] then
									local combo = makecomplextile(tile, x_tiled, y_tiled)
									combo.tags = tags_reverse
									if tags_reverse["fake"] then combo.solid = false; combo.fake = true; end
									tagcombos[c] = combo
								end
								local tile = tagcombos[c]
								--if tags_reverse["invisible"] then tile.invisible = true end
								--if tags_reverse["silhouette"] then tile.silhouette = true end
								table.insert(temptilesarray, tile)
							end
						end
					end
					if self.tilemap[y_tiled][x_tiled] == nil then
						self.tilemap[y_tiled][x_tiled] = temptilesarray
						self.objmap[y_tiled][x_tiled] = {}
					else
						for k,v in ipairs(temptilesarray) do
							table.insert(self.tilemap[y_tiled][x_tiled], v)
						end
					end
					x_tiled = x_tiled + 1
				end
			end
			lastskip.y = nil
			lastskip.x = nil
		end
		firstlayer = false
	end
	
	self.height = #self.tilemap * tilesize
	assert(self.width, "no width for this level")
	self.x = 0
	self.y = 0
	local vp = game.viewport
	if vp.width > self.width then
		self.x = math.floor((vp.width - self.width) / 2)
	end
	if vp.height > self.height then
		self.y = math.floor((vp.height - self.height) / 2)
	end
	
	if self.entrance.flipscreen then
		--insert an ogmo
		local x = 0
		local y = 0
		if self.entrance.id ~= "down" then
			y = self.height - 16
		end
		if self.entrance.id == "left" then
			x = self.width - 16
		end
		local tags = {["fluid"] = true}
		--print("====FLIPSCREEN!!!======")
		--print("initializing ogmo at " .. tostring(x) .. ", " .. tostring(y))
		local newobj = objects["ogmo"]:setup(x, y, {}, self, consts.oppositedirs[self.entrance.id], tags)
		if newobj then --aaaa copypasting!!!! most of this is just for ogmo specifically though so i got to remove stuff. though i'm also keeping stuff anyway in case 1000 years from now i decide to change ogmo's setup function weirdly
			--print("ogmo decided to be at " .. tostring(newobj.x) .. ", " .. tostring(newobj.y))
			if not newobj.tags then newobj.tags = tags end
			if newobj.tags["fake"] then newobj.fake = true end
			if newobj.level == nil then newobj.level = self end
			table.insert(self.objects, newobj)
			self.camerafocus = newobj
		else
			print("flipscreen happened but ogmo decided not to appear! oops?")
		end
	end
	
	game.emptyclique = false
	print("found " .. tostring(self.playeramt) .. " players")
	if self.playeramt == 0 then
		print("no players found! entrance ID is " .. tostring(self.entrance.id))
		self.noplayers = true
		if game.editormode then game.emptyclique = true end
	else
		self.noplayers = false
	end
	
	self.metadepth = 1
	self.liveplayeramt = self.playeramt
	self.objmap_usedcoords = {}
	self.objmap_usedcoords_byobj = {}
	setmetatable(self.objmap_usedcoords_byobj, weakkey)
	self:updatecamera()
	self:updateobjmap()
	
	if discard_templates then
		self.symbolmaps = nil
		self.tagmaps = nil
	end
	
	for i,v in ipairs(late_init) do
		v:late_init()
	end
	
	if next(self.temps) ~= nil then
		self.temps = {}
	end
	
	if statusbar.out and not self.noplayers then
		print("couldn't find camerafocus for statusbar???")
	end
	
	return self
end

function level:freeze()
	self.frozen = true
end

function level:unfreeze()
	self.frozen = false
end

function level:placeascii(text, x, y)
	if type(text) ~= "string" then
		error("asked to place ascii text " .. tostring(text) .. " of type " .. tostring(type(text)), 2)
	end
	if y > #self.tilemap or x > #self.tilemap[1] then
		error("asked to place ascii text outside of level bounds (at " .. tostring(x) .. ", " .. tostring(y) .. "; width is " .. #self.tilemap[1] .. " and height is " .. #self.tilemap .. ")", 2)
	end
	text = text:correctnewlines()
	local x_orig = x
	for i,v in ipairs(text:split("")) do
		if v == "\n" then
			x = x_orig
			y = y + 1
		else
			if not tiles["ascii_" .. v] then
				v = "?"
			end
			if x > #self.tilemap[y] then
				x = x_orig
				y = y + 1
			end
			if y > #self.tilemap then break end
			if v ~= " " then
				local t = self.tilemap[y][x]
				t[#t + 1] = "ascii_" .. v
			end
			x = x + 1
		end
	end
end

function level:includeparticle(name)
	assert(name ~= nil, "including nil particles?!")
	if not self.particles_set[name] then
		self.particles_set[name] = true
		_G.particles[name]:reset()
		self.particles[#self.particles + 1] = name
	end
end

function level:updateobjmap(obj) --objmap is a matrix like tilemap that denotes which cells are occupied by each object in self.objects. it's used to make collision/overlap detection quicker by being able to just look for objects in relevant areas of the matrix instead of iterating through every object
	if not obj then
		for k,v in pairs(self.objmap_usedcoords) do
			if v then
				local coords = split(k, ",")
				self.objmap[tonumber(coords[2])][tonumber(coords[1])] = {}
			end
			self.objmap_usedcoords[k] = nil
		end
		for k,v in pairs(self.objmap_usedcoords_byobj) do self.objmap_usedcoords_byobj[k] = nil end
		for i,obj in ipairs(self.objects) do
			if not obj.irrelevant then
				local topleftX = math.floor(((obj.x) / tilesize) + 1)
				local topleftY = math.floor(((obj.y) / tilesize) + 1)
				--if objmap is fucked up, first chekc the > #self.objmap[1]/self.objmap lines in topleft (and the < 1 lines in bottomright?)
				if topleftX > #self.objmap[1] then topleftX = #self.objmap[1] end
				if topleftX < 1 then topleftX = 1 end
				if topleftY > #self.objmap then topleftY = #self.objmap end
				if topleftY < 1 then topleftY = 1 end
				local bottomrightX = math.ceil(((obj.x + ((obj.width or tilesize))) / tilesize))
				local bottomrightY = math.ceil(((obj.y + ((obj.height or tilesize))) / tilesize))
				if bottomrightX > #self.objmap[1] then bottomrightX = #self.objmap[1] end
				if bottomrightX < 1 then bottomrightX = 1 end
				if bottomrightY > #self.objmap then bottomrightY = #self.objmap end
				if bottomrightY < 1 then bottomrightY = 1 end
				for y=topleftY, bottomrightY do
					for x=topleftX, bottomrightX do
						self.objmap_usedcoords[tostring(x) .. "," .. tostring(y)] = true
						if not self.objmap_usedcoords_byobj[obj] then self.objmap_usedcoords_byobj[obj] = {} end
						self.objmap_usedcoords_byobj[obj][tostring(x) .. "," .. tostring(y)] = true
						if (not self.objmap[y]) or (not self.objmap[y][x]) then
							print("objmap failed????")
							print(x, y)
							print(self.objmap[y])
							if self.objmap[y] then
								print(self.objmap[y][x])
							end
						end
						table.insert(self.objmap[y][x], obj)
					end
				end
			end
		end
	else
		if not self.objmap_usedcoords_byobj[obj] then
			self.objmap_usedcoords_byobj[obj] = {}
		else
			for k,v in pairs(self.objmap_usedcoords_byobj[obj]) do
				if v then
					local coords = split(k, ",")
					local mysquare = self.objmap[tonumber(coords[2])][tonumber(coords[1])]
					for i,obj2 in ipairs(mysquare) do
						if obj2 == obj then
							table.remove(mysquare, i)
							break
						end
					end
				end
				self.objmap_usedcoords_byobj[obj][k] = nil
			end
		end
		if not obj.irrelevant then
			local topleftX = math.floor(((obj.x) / tilesize) + 1)
			local topleftY = math.floor(((obj.y) / tilesize) + 1)
			--if objmap is fucked up, first chekc the > #self.objmap[1]/self.objmap lines in topleft (and the < 1 lines in bottomright?)
			if topleftX > #self.objmap[1] then topleftX = #self.objmap[1] end
			if topleftX < 1 then topleftX = 1 end
			if topleftY > #self.objmap then topleftY = #self.objmap end
			if topleftY < 1 then topleftY = 1 end
			local bottomrightX = math.ceil(((obj.x + ((obj.width or tilesize))) / tilesize))
			local bottomrightY = math.ceil(((obj.y + ((obj.height or tilesize))) / tilesize))
			if bottomrightX > #self.objmap[1] then bottomrightX = #self.objmap[1] end
			if bottomrightX < 1 then bottomrightX = 1 end
			if bottomrightY > #self.objmap then bottomrightY = #self.objmap end
			if bottomrightY < 1 then bottomrightY = 1 end
			for y=topleftY, bottomrightY do
				for x=topleftX, bottomrightX do
					self.objmap_usedcoords[tostring(x) .. "," .. tostring(y)] = true
					self.objmap_usedcoords_byobj[obj][tostring(x) .. "," .. tostring(y)] = true
					if (not self.objmap[y]) or (not self.objmap[y][x]) then
						print("objmap failed????")
						print(x, y)
						print(self.objmap[y])
						if self.objmap[y] then
							print(self.objmap[y][x])
						end
					end
					table.insert(self.objmap[y][x], obj)
				end
			end
		end
	end
end

function level:removefromobjmap(obj)
	for k,v in pairs(self.objmap_usedcoords_byobj[obj]) do
		if v then
			local coords = split(k, ",")
			local mysquare = self.objmap[tonumber(coords[2])][tonumber(coords[1])]
			for i,obj2 in ipairs(mysquare) do
				if obj2 == obj then
					table.remove(mysquare, i)
					break
				end
			end
		end
		self.objmap_usedcoords_byobj[obj][k] = nil
	end
	self.objmap_usedcoords_byobj[obj] = nil
end

function level:update()
	self.animtimer = self.animtimer + 1
	if self.animtimer > 8 then self.animtimer = 1 end
	self.doreset = false
	for _,p in ipairs(self.particles) do
		particles[p]:update(1) --1 frame, not 1 second
	end
	if not (self.frozen or self.won) then
		local foundcamerafocus = false
		
		for i=1, #self.objects_w_earlyupdate do
			if self.objects_w_earlyupdate[i] then
				local obj = self.objects_w_earlyupdate[i]
				obj:earlyUpdate() --earlyupdates are generally only for objects with the "on overlap" hook that want to do something while knowing it won't be after on overlap is called
				--it's also assumed the object won't move, hence updateobjmap not being called, but this can change
			end
		end
		--for i=1, #self.objects do
		local i = 1
		local objmax = #self.objects
		while i <= objmax do --this means that objects added mid-update will not be updated in the current update
			local obj = self.objects[i]
			if not obj then print("nil obj lol") end
			if obj.irrelevant then
				table.remove(self.objects, i)
				self:removefromobjmap(obj)
				i = i - 1
				objmax = objmax - 1
			elseif obj.update then --first bit is there due to an arcane ancient bug where the length of this would be nonzero even though the table's contents are all nil. probably safe to remove
			--reading the above comment a while later, that might have been from the length operator being erroneous on a table with gaps. maybe i didn't clean it all the way. idk
				if obj.level ~= self then
					print("---")
					print(obj.level)
					print("-")
					print(obj.level.__baseclass)
					print(self)
					print("-")
					print(self.__baseclass)
					print(level)
					error("this object that's inside me isn't inside me :/ -level (checking " .. obj.type .. ")")
				end
				local oldx = obj.x
				local oldy = obj.y
				obj:update()
				if self.won then return end
				if obj.irrelevant then
					table.remove(self.objects, i)
					i = i - 1
					self:removefromobjmap(obj)
					objmax = objmax - 1
				else
					if oldx ~= obj.x or oldy ~= obj.y or self.updateobjmap_queued[obj] then
						self:updateobjmap(obj)
					end
					if obj == self.camerafocus then foundcamerafocus = true end
				end
			end
			i = i + 1
		end
		--self:updateobjmap()
		if not foundcamerafocus then self.camerafocus = nil end
		self:updatecamera()
		if (not self.noplayers) and self.liveplayeramt <= 0 and (#game.corpses == 0 or settings.quickdeaths) and self.playeramt > 0 then
		--also, meta levels need logic so that entering a meta block with a corresponding active level should just bring you to the level instead of cloning it, and make sure they don't end until no more players are inside of it. also make it so you can't enter other meta levels while one meta level is active
			local levelname = self.name or "(unnamed lol)"
			print("all players in level " .. levelname .. " are dead! :(")
			self.doreset = true
		end
		if next(self.temps) ~= nil then
			self.temps = {}
		end
		if next(self.updateobjmap_queued) ~= nil then
			self.updateobjmap_queued = {}
		end
	end
end

function level:keypressed(key)
	if not self.frozen then
		for i=1, #self.objects do
			if self.objects[i].keypressed then
				self.objects[i]:keypressed(key)
			end
		end
	end
end

function level:draw()
	love.graphics.push()
	local r, g, b, a = love.graphics.getColor()
	local vp = game.viewport
	
	--screenborder time
	if not vp.trivial then
		local x = vp.x
		local y = vp.y - 16
		love.graphics.setScissor(0, 0, vp.x + vp.width, game.height)
		while x < (vp.x + vp.width) do
			love.graphics.draw(game.screenborder_img, game.screenborderquads["d"], x, y)
			x = x + 16
		end
		love.graphics.setScissor(0, 0, game.width, vp.y + vp.height)
		x = vp.x + vp.width
		love.graphics.draw(game.screenborder_img, game.screenborderquads["dl"], x, y)
		y = y + 16
		while y < (vp.y + vp.height) do
			love.graphics.draw(game.screenborder_img, game.screenborderquads["l"], x, y)
			y = y + 16
		end
		love.graphics.setScissor(vp.x, 0, game.width, game.height)
		y = vp.y + vp.height
		love.graphics.draw(game.screenborder_img, game.screenborderquads["ul"], x, y)
		x = x - 16
		while x > (vp.x - 16) do
			love.graphics.draw(game.screenborder_img, game.screenborderquads["u"], x, y)
			x = x - 16
		end
		love.graphics.setScissor(0, vp.y, game.width, game.height)
		x = vp.x - 16
		love.graphics.draw(game.screenborder_img, game.screenborderquads["ur"], x, y)
		y = y - 16
		while y > (vp.y - 16) do
			love.graphics.draw(game.screenborder_img, game.screenborderquads["r"], x, y)
			y = y - 16
		end
		love.graphics.setScissor()
		y = vp.y - 16
		love.graphics.draw(game.screenborder_img, game.screenborderquads["dr"], x, y)
	end
	
	--if not vp.trivial then
	love.graphics.translate(vp.x, vp.y)
	love.graphics.setScissor(vp.x, vp.y, vp.width, vp.height)
	--end
	
	love.graphics.setColor(hextocolor(self.background))
	love.graphics.rectangle("fill", 0, 0, game.width, game.height)
	love.graphics.setColor(0, 0, 0, 0.5)
	love.graphics.rectangle("fill", 0, 0, game.width, game.height)
	love.graphics.setColor(hextocolor(self.background))
	love.graphics.rectangle("fill", (-1 * self.cameraX) + self.x, (-1 * self.cameraY) + self.y, self.width, self.height)
	love.graphics.setColor(r, g, b, a)
	
	love.graphics.push()
	love.graphics.translate((-1 * self.cameraX) + self.x, (-1 * self.cameraY) + self.y)
	local smaller_width = (vp.width < self.width and vp.width) or self.width
	local smaller_height = (vp.height < self.height and vp.height) or self.height
	
	
	love.graphics.setScissor(vp.x + self.x, vp.y + self.y, smaller_width, smaller_height)
	--love.graphics.setScissor()
	--uncomment when testing tile culling
	
	--determine bounds for tilemap drawing
	local topbound
	local bottombound
	local leftbound
	local rightbound
	
	--inefficient
	--[[
	topbound = 1
	bottombound = #self.tilemap
	leftbound = 1
	rightbound = #self.tilemap[1]
	]]
	
	--efficient
	topbound = math.floor(self.cameraY / tilesize) + 1
	if topbound < 1 then topbound = 1 end
	bottombound = math.ceil(topbound + (vp.height / tilesize))
	if bottombound > #self.tilemap then bottombound = #self.tilemap end --len twice! this probably gets optimized away. i love being lazy. the machine will do everything for me.
	--SHUT UP. BE HAPPY. ACCEPT ALL COMPILER IMPROVEMENTS WITHOUT QUESTION. THE COMFORT YOU'VE DEMANDED IS NOW MANDATORY. AT LAST. EVERYTHING IS DONE... FOR YOU.
	--actually if __len were set in metatable it wouldn't necessarily be able to be optimized away. it's not, though.
	leftbound = math.floor(self.cameraX / tilesize) + 1
	if leftbound < 1 then leftbound = 1 end
	rightbound = math.ceil(leftbound + (vp.width / tilesize))
	if rightbound > #self.tilemap[1] then rightbound = #self.tilemap[1] end --len twice AND access twice, golly
	
	if advancingframe then
		print("TILEMAP CULLING")
		print("TOP:", topbound)
		print("BOTTOM:", bottombound)
		print("LEFT:", leftbound)
		print("RIGHT:", rightbound)
		print("-----")
	end
	
	--tilemap is like map but instead of containing just symbols it just contains the tile names the symbols would point to
	--for each row in the game.tilemap:
	for y_tiled=topbound, bottombound do
		--for each entry of the row
		for x_tiled=leftbound, rightbound do
			--for each tile in the entry (each entry can have multiple tiles)
			for i=1, #self.tilemap[y_tiled][x_tiled] do
				local content = self.tilemap[y_tiled][x_tiled][i]
				local tile = (type(content) == "string" and tiles[content]) or content
				if tile.color and not game.solid_colors[tile.color] then
					tile = (type(tile.alt) == "string" and tiles[tile.alt]) or tile.alt
				end
				if settings.seetheunseeable or (not tile.tags) or not tile.tags.invisible then
					local resetcolor = false
					local resetblend = false
					if tile.tags then resetcolor, resetblend = prepcolorfromtags(tile) end
					local graphics = tile.graphics
					if tile.anim and self.animtimer > 4 then graphics = tile.graphics_anim end
					for ii,graphic in ipairs(graphics) do
						love.graphics.draw(graphic.reference, graphic.quad, ((x_tiled - 1) * tilesize) + graphic.ingameoffset[1], ((y_tiled - 1) * tilesize) + graphic.ingameoffset[2])
					end
					if resetcolor then
						love.graphics.setColor(1, 1, 1, 1)
					end
					if resetblend then
						love.graphics.setBlendMode("alpha")
					end
				end
			end
		end
	end
	for _,p in ipairs(self.particles) do
		love.graphics.draw(particles[p])
	end
	local objects_nonsolid = {}
	local objects_solid = {}
	local objects_late = {}
	local trydraw = function(obj)
		if transit.active and self.camerafocus == obj and obj.edge and obj.edge ~= "none" then return end --don't draw the object if it's the camera focus, it's on an edge of the level, and the transit animation is active. this is so ogmo doesn't show up partially off the edge of the screen before the transit animation completes.
		if (not obj.tags) or (not obj.tags.invisible) then
			local resetcolor = false
			local resetblend = false
			if obj.tags then resetcolor,resetblend = prepcolorfromtags(obj) end
			obj:draw()
			if resetcolor then
				love.graphics.setColor(1, 1, 1, 1)
			end
			if resetblend then
				love.graphics.setBlendMode("alpha")
			end
		end
	end
	for _,obj in ipairs(self.objects) do
		if not obj.irrelevant then
			if obj.drawlate then table.insert(objects_late, obj)
			--elseif not obj.solid then table.insert(objects_nonsolid, obj)
			elseif (not obj.solid) or obj.drawearly then trydraw(obj)
			else table.insert(objects_solid, obj) end
		end
	end
	--[[
	for i=1, #objects_nonsolid do
		objects_nonsolid[i]:draw()
	end
	]]
	for i=1, #objects_solid do
		trydraw(objects_solid[i])
	end
	for i=1, #objects_late do
		trydraw(objects_late[i])
	end
	--[[
	for y,v1 in pairs(self.objmap) do
		if v1 then
			for x,v2 in pairs(v1) do
				if type(v2) == "table" then
					for _,__ in pairs(v2) do
						love.graphics.draw(graphics.load("marker"), (x - 1) * tilesize, (y - 1) * tilesize)
					end
				end
			end
		end
	end
	]]
	love.graphics.pop()
	love.graphics.pop()
	love.graphics.setScissor()
	--[[
	love.graphics.setColor(0, 0, 0, a)
	love.graphics.rectangle("fill", 400, 0, 400, 16)
	love.graphics.setColor(r, g, b, a)
	love.graphics.print(self.camerafocus.x, 400)
	love.graphics.print(self.camerafocus.y, 416)
	love.graphics.print(booltostr(self.camerafocus), 432)
	]]
end