mob = object:new()
--the difference between mobs and objects is that mobs have to have certain variables like vmom and hmom that imply they're moving objects. for this they get access to functions like nudge and... right now it's just nudge
--furthetmore just because an object has those variables doesn't mean it needs to be a mob; if it chooses to just be an object then it won't have nudge. objects that want to nudge another object need to check if it has nudge in the first place
--nudge could have gone into mobtools i guess. though unlike functions in mobtools, nudge is not called by the mob itself, it's called by another object that wants to move the mob against its will (e.g. a platform)

function mob:nudge(h,v,diy)
	self.level.updateobjmap_queued[self] = true
	local unblocked_h = true
	local unblocked_v = true
	local blockinfo = {}
	if h and h ~= 0 then
		local old_hmom = self.hmom
		self.hmom = h
		blockinfo.walled, blockinfo.newxposition, blockinfo.walledby = mobtools.doCollisionScan("horizontal", self)
		self.hmom = old_hmom
		
		unblocked_h = (blockinfo.walled == "none")
		if not diy then
			if unblocked_h then
				self.x = self.x + h
				self.x_clamped = math.floor(self.x + .5)
			else
				self.x = blockinfo.newxposition
				self.x_clamped = math.floor(self.x + .5)
				if self.walled ~= nil then self.walled = blockinfo.walled end
				if self.walledby ~= nil then self.walledby = blockinfo.walledby end
				local notdead = (#blockinfo.walledby == 0)
				for _,collider in ipairs(blockinfo.walledby) do --this line marks something particularly stupid in that the terms "collidee" and "collider" should be reversed almost everywhere they're used
					if not collider.deathly then
						notdead = true
					end
					if collider.onCollide then
						collider:onCollide(self, blockinfo.walled)
					end
				end
				if (not notdead) and self.die and not (self.type == "ogmo" and cheat.isactive("agodami")) then
					self:die()
				end
			end
		end
	end
	if v and v ~= 0 then --Looks like nudge isn't fully implemented because nothing nudges objects vertically yet.
		local old_vmom = self.vmom
		self.vmom = v
		blockinfo.verticaled, blockinfo.newyposition, blockinfo.verticaledby = mobtools.doCollisionScan("vertical", self)
		self.vmom = old_vmom
		unblocked_v = (blockinfo.verticaled == "none")
		if not diy then
			if unblocked_v then
				self.y = self.y + v
			else
				self.y = blockinfo.newyposition
			end
		end
	end
	return unblocked_h, unblocked_v, blockinfo
end

function mob:handleriders(cancrush) --NOTE: does not handle the ridable moving upward and catching objects that are in midair. ideally this should be a simpler "nudge objects that are on top of this by this object's x and y speed" function, and then everything related to PUSHING things should be a separate function entirely
	--although, also note that catching midair objects is 99% pointless for gates since they never yield and most moving objects will snap to the surface if they're inside something and have momentum. ...although, if you're going upward and a gate moves upward into you, will you snap to the bottom of it?
	local am_i_in = function(self, obj)
		for i,v in ipairs(obj.verticaledby) do
			if v == self then return true end
		end
	end
	if not self.riding then self.riding = {} end
	if #self.riding > 0 then
		local yield = false
		local tonudge = {}
		local tocrush = {}
		local toremove = {}
		for i,obj in ipairs(self.riding) do
			if obj.grounded and (obj.verticaledby[1] == self or am_i_in(self, obj)) then --having objects set up an inverse table for verticaledby would be faster
				self.level.updateobjmap_queued[obj] = true
				local unblocked_h, unblocked_v, blockinfo = obj:nudge(self.hmom, self.vmom, true)
				local willcrush = false
				if (not unblocked_v) and self.vmom < 0 then
					if cancrush and obj.crushcheck then
						local crushable = obj:crushcheck(blockinfo)
						if crushable then table.insert(tocrush, {["obj"] = obj, ["blockinfo"] = blockinfo}); willcrush = true;
						else yield = true; break; end
					else
						yield = true; break
					end
				end
				if not willcrush then table.insert(tonudge, {["obj"] = obj, ["blockinfo"] = blockinfo}) end
			else
				table.insert(toremove, i)
			end
		end
		for i,v in ipairs(toremove) do
			table.remove(self.riding, v)
		end
		if not yield then
			for _,t in ipairs(tonudge) do
				if not t.unblocked_v then
					t.obj.verticaled = t.blockinfo.verticaled
					t.obj.verticaledby = t.blockinfo.verticaledby
				else
					t.obj.y = t.blockinfo.newyposition
				end
				if not t.unblocked_h then
					t.obj.walled = t.blockinfo.walled
					t.obj.walledby = t.blockinfo.walledby
				else
					t.obj.x = t.blockinfo.newxposition
				end
			end
			for _,t in ipairs(tocrush) do
				if t.obj.crush then
					t.obj.x = t.blockinfo.newxposition
					t.obj.y = t.blockinfo.newyposition
					t.obj:crush(t.blockinfo)
				end
			end
		end
		return yield
	end
	return false
end

--[[ this function assumes the mod doing the riding is going to be calling it and not the mob on which it's riding (when it moves) which is really weird
function mob:handleriding() --the mob calling this is the one doing the riding
	local crush = false
	if not self.riding then self.riding = {} end
	if #self.riding > 0 then
		local highest_hmom = self.riding[1].hmom
		local lowest_hmom = self.riding[1].hmom
		local highest_vmom = self.riding[1].vmom
		local highest_vmom_obj = self.riding[1]
		local lowest_vmom = self.riding[1].vmom
		for _,obj in ipairs(self.riding) do
			if obj.hmom < lowest_hmom then lowest_hmom = obj.hmom
			elseif obj.hmom > highest_hmom then highest_hmom = obj.hmom end
			if obj.vmom < lowest_vmom then lowest_vmom = obj.vmom
			elseif obj.vmom > highest_vmom then highest_vmom = obj.vmom; highest_vmom_obj = obj; end
		end
		local h = highest_hmom
		if math.abs(lowest_hmom) > highest_hmom then h = lowest_hmom end
		local v = highest_vmom
		--if math.abs(lowest_vmom) > highest_vmom then v = lowest_vmom end --this doesn't make sense. if the rider is straddling two objects, one going down and one going up, it shouldn't follow the one going down just because it's going faster. this is, of course, assuming normal gravity direction
		if h ~= 0 and v ~= 0 then
			local unblocked_h, unblocked_v = self:nudge(h,v)
			if (not unblocked_v) and v < 0 then
				crush = true
			end
		end
	end
	return crush, highest_vmom_obj
end
]]