cannon = object:new()

function cannon.editordraw(x, y, options)
	local dir = "right"
	if options.dir then dir = options.dir end
	love.graphics.draw(graphics.load("cannon"), cannon.quads[options.kind or "normal"][dir], x, y)
end

local img = graphics.load("cannon")

cannon.quads = {
	normal = {
		["right"] = 			love.graphics.newQuad(0, 0, 16, 16, img),
		["down"] = 			love.graphics.newQuad(16, 0, 16, 16, img),
		["left"] = 			love.graphics.newQuad(16, 16, 16, 16, img),
		["up"] = 			love.graphics.newQuad(0, 16, 16, 16, img),
	},
	fire = {
		["right"] = 			love.graphics.newQuad(0, 32, 16, 16, img),
		["down"] = 			love.graphics.newQuad(16, 32, 16, 16, img),
		["left"] = 			love.graphics.newQuad(16, 48, 16, 16, img),
		["up"] = 				love.graphics.newQuad(0, 48, 16, 16, img),
	}
}

function cannon:init(x, y, dir, kind, timer)
	self.type = "cannon"
	self.x = x
	self.y = y
	self.hmom = 0
	self.vmom = 0
	self.width = 14
	self.height = 12
	self.solid = true
	self.dir = dir
	self.kind = kind
	self.timerstart = 100
	self.timer = timer
end

function cannon:setup(x, y, options, level, edge, tags)
	local dir = options.dir or "right"
	if tags["faceup"] then dir = "up"
	elseif tags["faceleft"] then dir = "left"
	elseif tags["faceright"] then dir = "right"
	elseif tags["facedown"] then dir = "down"
	end
	local kind = options.kind or "normal"
	local timer = 100
	if next(tags) then
		for i=0, 9 do
			if tags["variant" .. tostring(i)] then
				timer = math.modf(100 * (i/10))
				break
			end
		end
	end
	return cannon:new(x + 1, y + 2, dir, kind, timer)
end

function cannon:update()
	self.timer = self.timer - 1
	if self.timer <= 0 then
		local cb = objects["cannonball"]:setup(self.x - 1, self.y - 2, {["kind"] = self.kind}, self.level, "none", {["face" .. self.dir] = true})
		cb.level = self.level
		cb.tags = self.tags
		table.insert(self.level.objects, cb)
		self.level:updateobjmap(cb)
		if self.timer == -1 then --if timer started on 0 (and got decremented to -1) then the cannon should've already fired before the level began, so we simulate this by updating the cannonball twice. this works because the level hasn't been drawn yet
			cb:update()
			self.level:updateobjmap(cb)
		end
		self.timer = self.timerstart + self.timer
	end
end

function cannon:draw()
	love.graphics.draw(graphics.load("cannon"), self.quads[self.kind][self.dir], self.x - 1, self.y - 2)
end

return cannon