cannonball = object:new()

local img = graphics.load("cannon")

local noshine = quad(32, 16, 10, 10, img)
local fire1 = quad(32, 26, 10, 10, img)
local fire2 = quad(32, 36, 10, 10, img)

cannonball.quads = {
	normal = {
		quad(48, 0, 10, 10, img),
		quad(48, 10, 10, 10, img),
		quad(48, 20, 10, 10, img),
		quad(48, 30, 10, 10, img),
		quad(48, 40, 10, 10, img),
		quad(48, 50, 10, 10, img),
		quad(58, 0, 10, 10, img),
		quad(58, 10, 10, 10, img),
		quad(58, 20, 10, 10, img),
		quad(58, 30, 10, 10, img),
		quad(58, 40, 10, 10, img),
		quad(58, 50, 10, 10, img),
		noshine,
		noshine,
		noshine,
		noshine,
		noshine,
		noshine,
		noshine,
		noshine,
		noshine,
		noshine,
		noshine,
		noshine,
	},
	fire = {
		fire1,
		fire1,
		fire1,
		fire1,
		fire1,
		fire1,
		fire2,
		fire2,
		fire2,
		fire2,
		fire2,
		fire2,
		fire1,
		fire1,
		fire1,
		fire1,
		fire1,
		fire1,
		fire2,
		fire2,
		fire2,
		fire2,
		fire2,
		fire2,
	},
}

function cannonball.editordraw(x, y, options)
	love.graphics.draw(graphics.load("cannon"), cannonball.frames[options.kind or "normal"][13], x + 3, y + 3)
end

function cannonball:init(x, y, hmom, vmom, kind)
	self.type = "cannonball"
	self.x = x
	self.y = y
	self.hmom = hmom
	self.vmom = vmom
	self.width = 10
	self.height = 10
	self.kind = kind
	self.solid = false
	self.deathly = false
	if self.kind == "fire" then self.deathly = true end
	self.dontkillfollowers = true
	self.frame = 1
	self.lastframe = 24
	self.animtimer = 1
	self.animtimer_start = 1
	self.backward = (self.hmom < 0 or self.vmom > 0)
	if self.backward then self.frame = 12 end
	self.collisioncondition = function(self, collidee)
		if collidee.player then return false end
		return true
	end
end

function cannonball:setup(x, y, options, level, edge, tags)
	local hmom = 0
	local vmom = 0
	x = x + 3
	y = y + 3
	if tags["facerandom"] then tags[pick{"faceup","faceleft","facedown","faceright"}] = true end
	if tags["faceup"] then vmom = -3; y = (y + 3) - 16 end
	if tags["faceleft"] then hmom = -3; x = (x + 3) - 16 end
	if tags["facedown"] then vmom = 3; y = (y - 3) + 16 end
	if tags["faceright"] then hmom = 3; x = (x - 3) + 16 end
	return cannonball:new(x, y, hmom, vmom, options.kind or "normal")
end

cannonball.collisionfunctions = {
	horizontal = function(self)
		local walled, position, walledby = mobtools.doCollisionScan("horizontal", self)
		if position == nil then
			self.x = self.x + self.hmom
		else
			for _,collider in ipairs(walledby) do
				if collider.onCollide then
					collider:onCollide(self, walled)
				end
			end
			self.irrelevant = true
		end
	end,
	vertical = function(self)
		local verticaled, position, verticaledby = mobtools.doCollisionScan("vertical", self)
		if position == nil then
			self.y = self.y + self.vmom
		else
			for _,collider in ipairs(verticaledby) do
				if collider.onCollide then
					collider:onCollide(self, verticaled)
				end
			end
			self.irrelevant = true
		end
	end,
}

function cannonball:onOverlap(obj)
	if self.kind == "normal" and obj.type == "ogmo" then
		if obj.y + obj.height < self.y + self.height then
			if not (obj.grounded or obj.physicallyinsidelauncher) then
				self:playsfx("cannonball stomp")
				self.irrelevant = true
				obj.vmom = -1 * obj.jumpforce
			end
		else
			obj.mustdie = true
		end
	end
end

function cannonball:update()
	if self.animtimer > 0 then
		self.animtimer = self.animtimer - 1
	end
	if self.animtimer == 0 then
		if not self.backward then
			self.frame = self.frame + 1
			if self.frame > self.lastframe then self.frame = 1 end
			self.animtimer = self.animtimer_start
		else
			self.frame = self.frame - 1
			if self.frame < 1 then self.frame = self.lastframe end
			self.animtimer = self.animtimer_start
		end
	end
	if self.hmom ~= 0 then self.collisionfunctions.horizontal(self) end
	if self.vmom ~= 0 then self.collisionfunctions.vertical(self) end
	if self.x + self.width < 0 or self.y + self.height < 0 or self.y > self.level.height or self.x > self.level.width then
		self.irrelevant = true
	end
end

function cannonball:draw()
	love.graphics.draw(graphics.load("cannon"), cannonball.quads[self.kind or "normal"][self.frame], self.x, self.y)
end

return cannonball