checkpoint = object:new()

checkpoint.automask = true
checkpoint.maskimagename = "gem mask"

function checkpoint.editordraw(x, y, options)
	if not options["savepoint"] then
		love.graphics.draw(graphics.load("gems"), checkpoint.quads["outline"], x, y)
		love.graphics.draw(graphics.load("gems"), checkpoint.quads["inside"], x, y)
	else
		love.graphics.draw(graphics.load("gems"), checkpoint.quads["outline_save"], x, y)
		love.graphics.draw(graphics.load("gems"), checkpoint.quads["inside_save"], x, y)
	end
end

checkpoint.quads = {
	["outline"] = love.graphics.newQuad(0, 32, 16, 16, graphics.load("gems")),
	["inside"] = love.graphics.newQuad(16, 32, 16, 16, graphics.load("gems")),
	["outline_save"] = love.graphics.newQuad(0, 48, 16, 16, graphics.load("gems")),
	["inside_save"] = love.graphics.newQuad(16, 48, 16, 16, graphics.load("gems")),
}

checkpoint.drawlate = true --so that ogmoscan renders on top of everything

checkpoint.ogmoscanquads = {}
print("checkpoint ogmoscanquads time")
for i=0, 31 do
	--these equations could both probably be rearranged to be more consistent with each other. i think the 4 * appearing twice in y's equation is equivalent to the 16 * at the start of x's equation?
	local x = (16 * (i % 4)) + (4 * (i - (i % 16)))
	local y = (4 * (i - (i % 4))) - (4 * (i - (i % 16)))
	
	checkpoint.ogmoscanquads[i + 1] = quad(x, y, 16, 16, graphics.load("ogmoscan"))
	--yes i mental algebra'd this instead of just writing a bunch of coordinates manually. i'm not sure why
	--print("frame " .. i + 1 .. ": x offset " .. tostring(x) .. ", y offset " .. tostring(y))
end

function checkpoint:init(x, y, spawnnum, savepoint)
	self.type = "checkpoint"
	self.x = x
	self.y = y
	self.hmom = 0
	self.ymom = 0
	self.width = tilesize
	self.height = tilesize
	self.solid = false
	self.spawnnum = spawnnum
	self.invalid = false
	self.passive = false
	self.used = false --setting this to "used" is handled in late_init
	self.scan_anim_timer = 0
	self.savepoint = savepoint
end

function checkpoint:late_init() --checkpoint isn't going to know whether it's valid or not until all ogmos have been placed
	--if not self.savepoint and
	if (self.level.entrance.id == self.spawnnum) or (game.templatelevels[1] and game.templatelevels[1].filepath == self.level.filepath and game.templatelevels[1].entrance.id == self.spawnnum) then
		self.used = true
	else
		self.used = false
	end
	if not self.level.ogmo_spawns[self.spawnnum] then
		self.invalid = true
	else
		self.invalid = false
	end
end

function checkpoint:setup(x, y, options, level, edge, tags)
	local spawnnum = 1
	for i=0, 9 do
		if tags["variant" .. i] then
			spawnnum = i; break;
		end
	end
	local savepoint = not not options["savepoint"]
	return checkpoint:new(x, y, spawnnum, savepoint)
end

function checkpoint:update()
	if self.scan_anim_timer > 0 then
		self.scan_anim_timer = self.scan_anim_timer - 1
	end
end

function checkpoint:onOverlap(obj)
	if (not self.used) and (not self.invalid) and obj.player then
		local entrance = {
			--number = self.exitno or 1,
			id = self.spawnnum,
			careening = obj.careening,
			jumpsleft = obj.jumps,
			hmom = obj.hmom,
			vmom = obj.vmom,
			gray = (obj.starstate and obj.starstate.gray),
			purple = (obj.starstate and obj.starstate.purple),
			icestar = (obj.starstate and obj.starstate.ice),
		}
		local lvl = game.activelevels[1]
		if self.savepoint then
			if not game.editormode then
				game.save_entrance = entrance
			end
			for k,v in pairs(game.solid_colors) do
				game.solid_colors_start[k] = game.solid_colors[k]
			end
			game.switchpadstate_start = game.switchpadstate
			game.switchpad2state_start = game.switchpad2state
			game.savegems()
		end
		game.start_in_different_level = false
		game.activelevels[1].entrance = entrance
		game.templatelevels[1] = game.activelevels[1]
		self.used = true
		self.scan_anim_timer = 32
		self:playsfx("gem collect")
	end
end

function checkpoint:draw()
	local outline = "outline"
	local inside = "inside"
	if self.savepoint then outline = "outline_save"; inside = "inside_save"; end
	love.graphics.draw(graphics.load("gems"), checkpoint.quads[outline], self.x, self.y)
	if (not self.used) and (not self.invalid) then love.graphics.draw(graphics.load("gems"), checkpoint.quads[inside], self.x, self.y) end
	if self.scan_anim_timer > 0 then
		for i,v in ipairs(self.level.ogmo_spawns[self.spawnnum]) do
			local scan = "ogmoscan"
			local frame = 33 - self.scan_anim_timer
			if self.savepoint then
				scan = "ogmoscan_green"
				frame = self.scan_anim_timer
			end
			love.graphics.draw(graphics.load(scan), checkpoint.ogmoscanquads[frame], v[1], v[2])
		end
	end
end

return checkpoint