conveyor = object:new()

local img = graphics.load("conveyor")

conveyor.quads = {
	["right1"] = quad(0,  0, 16, 16, img),
	["right2"] = quad(16, 0, 16, 16, img),
	["right3"] = quad(32, 0, 16, 16, img),
	["right4"] = quad(48, 0, 16, 16, img),
	["right5"] = quad(64 + 0,  0, 16, 16, img),
	["right6"] = quad(64 + 16, 0, 16, 16, img),
	["right7"] = quad(64 + 32, 0, 16, 16, img),
	["right8"] = quad(64 + 48, 0, 16, 16, img),
	["left1"] = quad(0,  16, 16, 16, img),
	["left2"] = quad(16, 16, 16, 16, img),
	["left3"] = quad(32, 16, 16, 16, img),
	["left4"] = quad(48, 16, 16, 16, img),
	["left5"] = quad(64 + 0,  16, 16, 16, img),
	["left6"] = quad(64 + 16, 16, 16, 16, img),
	["left7"] = quad(64 + 32, 16, 16, 16, img),
	["left8"] = quad(64 + 48, 16, 16, 16, img),
}

function conveyor.editordraw(x, y, options)
	love.graphics.draw(graphics.load("conveyor"), conveyor.quads[options.dir .. "1"], x, y)
end

function conveyor:init(x, y, dir)
	self.type = "conveyor"
	self.x = x
	self.y = y
	self.hmom = 0
	self.vmom = 0
	self.width = tilesize
	self.height = tilesize
	self.solid = true
	self.speed = 1
	self.direction = dir
	self.currentquad = 8
end

function conveyor:setup(x, y, options, level, edge, tags)
	return conveyor:new(x, y, options.dir)
end

function conveyor:invertvoltage()
	self.direction = consts.oppositedirs[self.direction]
	self.currentquad = 9 - self.currentquad
end

function conveyor:update()
	if game.solid_colors.electricity then
		self.currentquad = self.currentquad + 1
		if self.currentquad > 8 then self.currentquad = 1 end
	end
end

function conveyor:onCollide(collidee, side)
	if game.solid_colors.electricity then
		if (side == "down" and collidee.nudge) or (side == "up" and collidee.type == "ogmo" and (not collidee.ghost) and self.level.gravity < 0) then
			local sign = 1
			if self.direction == "left" then sign = -1 end
			if side == "up" then sign = sign * - 1 end
			local temps = self.level.temps
			if not temps.conveyored then
				temps.conveyored = {}
			end
			local obj = collidee
			if not temps.conveyored[obj] then --each table in temps is cleared after every update
				--local unblocked_h, unblocked_v, blockinfo = obj:nudge(self.speed, 0, true)
				obj:nudge(self.speed * sign, 0)
				temps.conveyored[obj] = true
			end
		end
	end
end

function conveyor:draw()
	love.graphics.draw(graphics.load("conveyor"), self.quads[self.direction .. self.currentquad], self.x, self.y)
end

return conveyor