door = object:new()

function door.editordraw(x, y, options)
	love.graphics.draw(graphics.load("door"), x, y)
end

function door:init(x, y, exitid, saves, facing)
	self.type = "door"
	self.x = x
	self.y = y
	self.hmom = 0
	self.ymom = 0
	self.width = tilesize
	self.height = tilesize
	self.solid = false
	self.exitid = exitid
	self.showtext = false
	self.justoverlapped = true --kludginess since update might come before or after overlap depending on object order
	self.invalid = false
	self.passive = true
	self.saves = saves
	self.facing = facing
end

function door:setup(x, y, options, level, edge, tags)
	local exitid = nil
	local facing = false
	for i=0, 9 do
		if tags["variant" .. i] then
			exitid = i; break;
		end
	end
	if not exitid then
		for i,v in ipairs(consts.directions) do
			if tags["face" .. v] then
				exitid = v
				facing = true
				break
			end
		end
	end
	if not exitid then exitid = 1 end
	local saves = not tags["nosave"]
	local newdoor = door:new(x, y, exitid, saves, facing)
	newdoor.tags = tags
	if newdoor.tags["invisible"] then
		newdoor.tags["invisible"] = false
		newdoor.invisible = true
	end
	return newdoor
end

function door:late_init()
	if (not (self.invalid or self.facing)) and self.level.exits[self.exitid] == "" or not self.level.exits[self.exitid] then
		self.invalid = true
	end
end

--door.clone_init = door.late_init

function door:update()
	if self.justoverlapped then
		self.justoverlapped = false
	else
		self.showtext = false
	end
end

function door:onOverlap(obj)
	if not self.invalid then
		if obj.player and (not obj.ghost) and obj.alive and obj.grounded then
			self.showtext = true
			self.justoverlapped = true
		end
	end
end

function door:onInteract(obj) --this can actually be called before update or earlyupdate since events like keypresses precede updates
	if not self.invalid then
		if obj.player and (not obj.ghost) and obj.grounded and obj.alive and not obj.level.frozen then
			self:playsfx("door")
			local exit = { --not actually sure why you'd want to preserve momentum when going through a door but you can if that's something you want
				id = self.exitid or 1,
				careening = obj.careening,
				jumpsleft = obj.jumps,
				hmom = obj.hmom,
				vmom = obj.vmom,
				xoffset = obj.x - self.x,
				yoffset = obj.y - self.y,
				gray = (obj.starstate and obj.starstate.gray),
				purple = (obj.starstate and obj.starstate.purple),
				icestar = (obj.starstate and obj.starstate.ice),
			}
			game.win(exit, self.level, self.saves)
		end
	end
end

function door:draw()
	if not self.invisible then
		love.graphics.draw(graphics.load("door"), self.x, self.y)
	end
	if self.showtext then
		local text = game.interactprompt
		local x = self.x + (tilesize / 2) - math.floor(text:getWidth() / 2)
		local y = self.y - tilesize
		local r,g,b,a = love.graphics.getColor()
		love.graphics.setColor(1,1,1,1) --you can't have color tags apply to interact prompts but you *can* have additive tags apply because i'm too lazy to disable additive blending lol.
		drawWithOutline(text, x, y)
		love.graphics.setColor(r,g,b,a)
		--printWithOutline(self.keytoprint, self.x + 4, self.y - tilesize) --+4 is completely arbitrary because i don't feel like centering it properly right now
	end
end

return door