elecswitch = object:new()

function elecswitch.editordraw(x, y, options)
	local myquad = "on"
	if options.voltage then myquad = "v_off" end
	love.graphics.draw(graphics.load("electricity"), elecswitch.quads[myquad], x, y + 4)
end

local img = graphics.load("electricity")

elecswitch.quads = {
	["off"] = 			love.graphics.newQuad(48, 0 + 4, 16, 12, img),
	["on"] = 			love.graphics.newQuad(48, 16 + 4, 16, 12, img),
	["v_off"] = 			love.graphics.newQuad(48, 0 + 4 + 64, 16, 12, img),
	["v_on"] = 			love.graphics.newQuad(48, 16 + 4 + 64, 16, 12, img),
}

function elecswitch:init(x, y, voltage)
	self.type = "elecswitch"
	self.x = x
	self.y = y
	self.hmom = 0
	self.vmom = 0
	self.width = tilesize
	self.height = 12
	self.solid = true
	self.voltage = voltage
end

function elecswitch:setup(x, y, options)
	return elecswitch:new(x, y + 4, not not options.voltage)
end

-- elecswitch["update"] = donothing --could cause dependency issues if donothing library is unpublished or rewritten to do something instead of nothing

function elecswitch:update()
	
end

function elecswitch:onCollide(collidee, side)
	if collidee.type == "ogmo" or collidee.type == "cannonball" then
		if not self.voltage then
			if side == "right" and not game.solid_colors.electricity then
				game.solid_colors.electricity = true
				self:playsfx("electricity switch")
			elseif side == "left" and game.solid_colors.electricity then
				game.solid_colors.electricity = false
				self:playsfx("electricity switch")
			end
		else
			if side == "right" and not game.solid_colors.voltageinv then
				game.solid_colors.voltageinv = true --voltage invert is very awkward because i initially conceived it as just a quick hack so mobtools collision/overlap check condition wouldn't have to get even more complicated (normal colors were complicated enough, and electricity would have to react based on the state of TWO colors (electricity itself and inverted voltage), which i wasn't prepared to implement) but now it actually is a color except instead of swapping graphics and collision in the draw/collision functions it actually replaces the swapped tiles entirely within the tilemap, which normal colors could also have done but which they didn't for efficiency, and jesus christ.
				self.level:invertvoltage()
				self:playsfx("electricity switch")
			elseif side == "left" and game.solid_colors.voltageinv then
				game.solid_colors.voltageinv = false
				self.level:invertvoltage()
				self:playsfx("electricity switch")
			end
		end
	end
end

function elecswitch:draw()
	local myquad = "off"
	if self.voltage then
		myquad = "v_off"
		if game.solid_colors.voltageinv then myquad = "v_on" end
	elseif game.solid_colors.electricity then myquad = "on"
	end
	love.graphics.draw(graphics.load("electricity"), self.quads[myquad], self.x, self.y)
end

return elecswitch