fallblock = object:new()

--note that the timers for fallblokcs, boomboxes, and xboxes can fluctuate by 1 frame depending on whether the object that collides with them is AFTER it in the object order or BEFORE it

local img = graphics.load("fallblock")

fallblock.quads = {
	["fallblock1"] = quad(0,  0, 16, 16, img),
	["fallblock2"] = quad(16, 0, 16, 16, img),
	["fallblock3"] = quad(32, 0, 16, 16, img),
	["fallblock4"] = quad(48, 0, 16, 16, img),
	["boomblock1"] = quad(0,  16, 16, 16, img),
	["boomblock2"] = quad(16, 16, 16, 16, img),
	["boomblock3"] = quad(32, 16, 16, 16, img),
	["boomblock4"] = quad(48, 16, 16, 16, img),
	["xblock1"] = quad(0,  32, 16, 16, img),
	["xblock2"] = quad(16, 32, 16, 16, img),
	["xblock3"] = quad(32, 32, 16, 16, img),
	["xblock4"] = quad(48, 32, 16, 16, img),
}

function fallblock.editordraw(x, y, options)
	local graphicname = "fallblock"
	if options["boombox"] then graphicname = "boomblock"
	elseif options["xbox"] then graphicname = "xblock" end
	love.graphics.draw(graphics.load("fallblock"), fallblock.quads[graphicname .. "1"], x, y)
end

function fallblock:init(x, y, boombox, xbox, hovering)
	self.type = "fallblock"
	self.x = x
	self.y = y
	self.y_clamped = y
	self.hmom = 0
	self.vmom = 0
	self.width = tilesize
	self.height = tilesize
	self.solid = true
	self.boombox = boombox
	self.xbox = xbox --holy fuck. naming it "xblock" was just a coincidence since the graphic i drew for it looked like an x
	self.hovering = hovering
	self.gravity = 0.2
	self.boomed = false
	self.irrelevant = false
	self.timer = -1
	if not self.hovering then self.timer = 0 end
	self.timer_start = 10
	self.quadnum = 1
	self.animtimer = 1
	self.animtimer_start = 5
	self.crushduckingogmo = true
end

function fallblock:setup(x, y, options, level, edge, tags)
	local boombox = options["boombox"]
	local xbox = options["xbox"]
	local hovering = not tags["variant1"]
	return fallblock:new(x, y, boombox, xbox, hovering)
end

function fallblock:update()
	if self.timer == -1 and self.animtimer > 0 then
		self.animtimer = self.animtimer - 1
	end
	if self.animtimer == 0 then
		self.quadnum = self.quadnum + 1
		if self.quadnum > 4 then self.quadnum = 1 end
		self.animtimer = self.animtimer_start
	end
	
	
	if self.timer > 0 then
		self.timer = self.timer - 1 --if this causes it to hit 0 then it will blow up/start moving down instantly
	end
	if self.timer == 0 and (self.hovering or (not self.boomed)) then
		if self.boombox or self.xbox then
			self.boomed = true
			self.solid = false
			self.irrelevant = true
			self:playsfx("boombox explode");
		else
			self.hovering = false
		end
	end
	if not self.hovering then
		self.vmom = self.vmom + self.gravity
		--collision
		local position
		local verticaled
		local verticaledby
		
		verticaled, position, verticaledby = mobtools.doCollisionScan("vertical", self)
		
		if position == nil then
			self.y = self.y + self.vmom
			self.y_clamped = math.floor(self.y + .5)
		else
			if self.y ~= self.y_clamped then self.y_clamped = math.floor(self.y + .5) end
			local bounce = false
			for _,collider in ipairs(verticaledby) do
				if collider.bounce then
					bounce = true
					break
				end
			end
			local nonogmos = {}
			self.y = position
			if bounce then
				if math.abs(self.vmom) < 5 then
					if self.vmom < 0 then self.vmom = self.vmom + (1/32) else self.vmom = self.vmom - (1/32) end
				end
				self.vmom = -self.vmom
				if math.abs(self.vmom) < 1 then self.vmom = 0 end
			else
				self.vmom = 0
			end
			for _,collider in ipairs(verticaledby) do
				if collider.onCollide then
					collider:onCollide(self, verticaled)
				end
			end
		end
	end
end

function fallblock:onCollide(collidee, side)
	if self.xbox then
		if self.timer == -1 then self:playsfx("falling block activate"); end
		self.timer = 2
	elseif self.timer == -1 and (self.boombox or side == "down") then --"down" actually refers to the top of the block, since the side argument is the side the collidee argument is colliding with, which is a little stupid. also it should be collider, not collidee, since another object has to call this function, but i copy-pasted this from hintblock so i'm leaving it as-is
		self.timer = self.timer_start
		self:playsfx("falling block activate");
	end
end

function fallblock:draw()
	local graphicname = "fallblock"
	if self.boombox then graphicname = "boomblock"
	elseif self.xbox then graphicname = "xblock" end
	love.graphics.draw(graphics.load("fallblock"), fallblock.quads[graphicname .. self.quadnum], self.x, self.y_clamped)
end

return fallblock