local mask_references = {
	["up"] = "spikesS",
	["right"] = "spikesE",
	["down"] = "spikesN",
	["left"] = "spikesW",
}

--idea for implementing line of sight:
--when overlapper finds something, do a collision test with simulated momentum to see if you would be able to collide with it, with collisioncondition temporarily disabled so you can collide with the player. to implement see-through tiles, swap it for another collisioncondition that disables colliding with anything with the "seethrough" property
--(note: ghostblocks should be see-through)

fallyspike = object:new()

local img = graphics.load("fallingspikes")

fallyspike.quads = {
	["up"] = love.graphics.newQuad(0, 0, 16, 16, img),
	["left"] = love.graphics.newQuad(16, 0, 16, 16, img),
	["down"] = love.graphics.newQuad(32, 0, 16, 16, img),
	["right"] = love.graphics.newQuad(48, 0, 16, 16, img),
	["mine"] = love.graphics.newQuad(64, 0, 16, 16, img),
	["strafer_loose"] = love.graphics.newQuad(80, 0, 16, 16, img),
	["strafer_rigid"] = love.graphics.newQuad(96, 0, 16, 16, img),
}

function fallyspike.editordraw(x, y, options)
	love.graphics.draw(graphics.load("fallingspikes"), fallyspike.quads[options.dir or "down"], x, y)
end

function fallyspike:init(x, y, dir, strafer)
	self.type = "fallyspike"
	self.x = x
	self.y = y
	self.x_clamped = self.x
	self.y_clamped = self.y
	self.hmom = 0
	self.vmom = 0
	self.hgrav = 0
	self.vgrav = 0
	self.width = tilesize
	self.height = tilesize
	self.solid = false
	self.deathly = true
	self.collisioncondition = function(self, collidee)
		if collidee.player then return false end
		return true
	end
	--self.dontkillfollowers = true --because you'll handle it on your side
	self.active = false
	self.gravity = 0.2
	self.dir = dir
	self.mine = false
	if self.dir == "mine" then self.mine = true end
	self.strafer = strafer
	if not self.mine then self.strafer = false end
	self.overlappers = {}
	self.maxspeed = 10
	
	if mask_references[self.dir] then
		self.mask = tiles[mask_references[self.dir]].mask
	end
end

function fallyspike:late_init()
	if not self.mine then
		if self.dir == "down" then self.vgrav = self.gravity
		elseif self.dir == "right" then self.hgrav = self.gravity
		elseif self.dir == "up" then self.vgrav = -1 * self.gravity
		elseif self.dir == "left" then self.hgrav = -1 * self.gravity
		end
		
		local x = self.x
		local y = self.y
		local w = tilesize
		local h = tilesize
		local x_ = math.floor(self.x / tilesize)
		local y_ = math.floor(self.y / tilesize)
		local axis = "vertical"
		if self.dir == "left" or self.dir == "right" then axis = "horizontal" end
		if self.level.objmap[y] and self.level.objmap[y_][x_] then
			for i,obj in ipairs(self.level.objmap[y_][x_]) do
				if obj.type == "fallyspike_overlapper_" .. axis then
					obj.subscribers[self] = true
					return
				end
			end
		end
		if axis == "vertical" then
			y = 0
			h = self.level.height
		else
			x = 0
			w = self.level.width
		end
		local ov = objects["overlapper"]:new(x, y, w, h)
		ov.type = "fallyspike_overlapper_" .. axis
		ov.subscribers[self] = true
		ov.level = self.level
		table.insert(self.level.objects, ov)
		self.level:updateobjmap(ov)
	else
		local ov1 = objects["overlapper"]:new(0, self.y, self.level.width, tilesize)
		ov1.type = "fallymine_overlapper_horizontal"
		ov1.subscribers[self] = true
		ov1.level = self.level
		table.insert(self.level.objects, ov1)
		self.level:updateobjmap(ov1)
		
		local ov2 = objects["overlapper"]:new(self.x, 0, tilesize, self.level.height)
		ov2.type = "fallymine_overlapper_vertical"
		ov2.subscribers[self] = true
		ov2.level = self.level
		table.insert(self.level.objects, ov2)
		self.level:updateobjmap(ov2)
		
		self.overlappers[1] = ov1
		self.overlappers[2] = ov2
	end
end

local function sign(x)
	if type(x) ~= "number" then error("getting sign of non-number (" .. tostring(x) .. ")", 2) end
	return (x < 0 and -1) or 1
end

fallyspike.collisionfunctions = {
	horizontal = function(self)
		local walled, position, walledby = mobtools.doCollisionScan("horizontal", self)
		if position == nil then
			self.x = self.x + self.hmom
			self.x_clamped = math.floor(self.x + .5)
		else
			self.x = position
			--if (not self.strafer) or sign(self.hmom) == sign(self.hgrav) then --makes it so strafers don't get stuck if they hit a wall in the direction they're accelerating away from
			if not self.strafer then
				self.hgrav = 0
				self.active = false
			else
				if self.hmom * self.hgrav > 0 then --cancel horizontal gravity if a wall was hit in the direction of the gravity
					self.hgrav = 0
				end
			end
			self.hmom = 0
			if self.x ~= self.x_clamped then self.x_clamped = math.floor(self.x + .5) end
			for _,collider in ipairs(walledby) do
				if collider.onCollide then
					collider:onCollide(self, walled)
				end
			end
			if not self.mine then
				self.irrelevant = true
			else
				self.overlappers[1].fake = false
				self.overlappers[2].fake = false
				self.overlappers[2].x = self.x
				self.level.updateobjmap_queued[self.overlappers[2]] = true
			end
		end
	end,
	vertical = function(self)
		local verticaled, position, verticaledby = mobtools.doCollisionScan("vertical", self)
		if position == nil then
			self.y = self.y + self.vmom
			self.y_clamped = math.floor(self.y + .5)
		else
			self.y = position
			--if (not self.strafer) or sign(self.vmom) == sign(self.vgrav) then --makes it so strafers don't get stuck if they hit a wall in the direction they're accelerating away from
			if not self.strafer then
				self.vgrav = 0
				self.active = false
			else
				if self.vmom * self.vgrav > 0 then --cancel vertical gravity if a wall was hit in the direction of the gravity
					self.vgrav = 0
				end
			end
			self.vmom = 0
			if self.y ~= self.y_clamped then self.y_clamped = math.floor(self.y + .5) end
			for _,collider in ipairs(verticaledby) do
				if collider.onCollide then
					collider:onCollide(self, verticaled)
				end
			end
			if not self.mine then
				self.irrelevant = true
			else
				self.overlappers[1].fake = false
				self.overlappers[2].fake = false
				self.overlappers[1].y = self.y
				self.level.updateobjmap_queued[self.overlappers[1]] = true
			end
		end
	end,
}

function fallyspike:activate(overlapper, obj)
	if (not obj.player) or (obj.ghost) then return end
	if
		self.mine or
		(self.dir == "down" and obj.y > self.y) or
		(self.dir == "up" and obj.y < self.y) or
		(self.dir == "left" and obj.x < self.x) or
		(self.dir == "right" and obj.x > self.x)
	then
		self.active = true
		if self.mine then
			--if i'm NOT aligned vertically and am moving horizontally:
				--stop moving horizontally and start moving vertically
			--except of course it's not actually that simple
			if overlapper.type == "fallymine_overlapper_horizontal" and not self.level:checktemp("fm_vert", self) then
				if self.x == obj.x then return end
				if (not self.strafer) and self.hmom ~= 0 then return end --for non-strafing mines: ignore if already moving
				self.hgrav = self.gravity * sign(obj.x - self.x)
				if not self.strafer then
					self.overlappers[2].fake = true
					self.vgrav = 0
				end
				if self.strafer == "rigid" then
					self.vgrav = 0
					self.vmom = 0
					self.level:marktemp("fm_hori", self)
					--[[
					if self.hmom == 0 and
					obj.x + obj.width > self.x and
					self.x + self.width > obj.x and
					obj.y + obj.height > self.y and
					self.y + self.height > obj.y then --if already overlapping the object, then move early
						self:update()
						self.level.updateobjmap_queued[self] = true
					end]]
				end
			elseif overlapper.type == "fallymine_overlapper_vertical" and not self.level:checktemp("fm_hori", self) then
				if self.y == obj.y then return end
				if (not self.strafer) and self.vmom ~= 0 then return end --for non-strafing mines: ignore if already moving
				self.vgrav = self.gravity * sign(obj.y - self.y)
				if not self.strafer then
					self.overlappers[1].fake = true
					self.hgrav = 0
				end
				if self.strafer == "rigid" then
					self.hgrav = 0
					self.hmom = 0
					self.level:marktemp("fm_vert", self)
					--[[
					if self.vmom == 0 and
					obj.x + obj.width > self.x and
					self.x + self.width > obj.x and
					obj.y + obj.height > self.y and
					self.y + self.height > obj.y then --if already overlapping the object, then move early
						self:update()
						self.level.updateobjmap_queued[self] = true
					end]]
				end
			end
		else
			overlapper.subscribers[self] = nil
		end
	end
end

function fallyspike:setup(x, y, options, level, edge, tags)
	local dir = options.dir or "down"
	local strafer
	if dir == "mine" then
		if tags["variant1"] then strafer = "loose" end
		if tags["variant2"] then strafer = "rigid" end
		if tags["variant1"] and tags["variant2"] then strafer = pick({"loose", "rigid"}) end
	end
	if dir ~= "mine" and next(tags) then
		if tags["facerandom"] then dir = pick(consts.directions)
		elseif tags["faceup"] then dir = "up"
		elseif tags["faceleft"] then dir = "left"
		elseif tags["faceright"] then dir = "right"
		elseif tags["facedown"] then dir = "down" --redundant
		end
	end
	return fallyspike:new(x, y, dir, strafer)
end

function fallyspike:update()
	if self.active then
		if self.hgrav ~= 0 then
			self.hmom = self.hmom + self.hgrav
			if math.abs(self.hmom) > self.maxspeed then self.hmom = self.maxspeed * sign(self.hmom) end
		end
		if self.vgrav ~= 0 then
			self.vmom = self.vmom + self.vgrav
			if math.abs(self.vmom) > self.maxspeed then self.vmom = self.maxspeed * sign(self.vmom) end
		end
		if self.hmom ~= 0 then self.collisionfunctions.horizontal(self) end
		if self.vmom ~= 0 then self.collisionfunctions.vertical(self) end
		if self.strafer then
			self.overlappers[1].y = self.y
			self.level.updateobjmap_queued[self.overlappers[1]] = true
			self.overlappers[2].x = self.x
			self.level.updateobjmap_queued[self.overlappers[2]] = true
		end
		if self.x + self.width < 0 or self.y + self.height < 0 or self.y > self.level.height or self.x > self.level.width then
			self.irrelevant = true
		end
	end
end

function fallyspike:draw()
	local myquad = self.dir
	if self.strafer then myquad = "strafer_" .. self.strafer end
	love.graphics.draw(graphics.load("fallingspikes"), self.quads[myquad], self.x_clamped, self.y_clamped)
end

return fallyspike