fireball = object:new()

local img = graphics.load("fireball")

fireball.quads = {
	quad(0,  0, 16, 16, img),
	quad(16, 0, 16, 16, img),
	quad(32, 0, 16, 16, img),
	quad(48, 0, 16, 16, img),
}

function fireball.editordraw(x, y, options)
	love.graphics.draw(graphics.load("fireball"), fireball.quads[1], x, y)
end

function fireball:init(x, y, hmom, vmom, slowanim)
	self.type = "fireball"
	self.x = x
	self.y = y
	self.y_clamped = y
	self.x_clamped = x
	self.hmom = hmom
	self.vmom = vmom
	self.width = tilesize
	self.height = tilesize
	self.solid = false
	self.deathly = true
	self.dontkillfollowers = true
	self.collidewith = {["reverser"] = true}
	--self.collidewith = self.collidewith or {["reverser"] = true} --try to inherit parent's self.collidewith instead of creating a new table... presuming "x or {}" doesn't create a new table regardless of whether x
	self.collisioncondition = function(self, collidee) --different than collidewith because this only gets called to REVERT colliding with a tile; collidewith is used to denote this should collide with a particular tile that isn't solid. jank
		if collidee.player then return false end
		return true
	end
	self.frame = 1
	self.animtimer = 6
	self.animtimer_start = 6
	if slowanim then
		self.animtimer = 8
		self.animtimer_start = 8
	end
end

function fireball:setup(x, y, options, level, edge, tags)
	local hmom = 0
	local vmom = 0
	if tags["facerandom"] then tags[pick{"faceup","faceleft","facedown","faceright"}] = true end
	if tags["faceup"] then vmom = -2 end
	if tags["faceleft"] then hmom = -2 end
	if tags["facedown"] then vmom = 2 end
	if tags["faceright"] then hmom = 2 end
	local slowanim = false
	if tags["fake"] then slowanim = true end --Per J2E.
	return fireball:new(x, y, hmom, vmom, slowanim)
end

fireball.collisionfunctions = {
	horizontal = function(self)
		local _, position = mobtools.doCollisionScan("horizontal", self)
		if position == nil then
			self.x = self.x + self.hmom
			self.x_clamped = math.floor(self.x + .5)
		else
			self.x = position
			self.hmom = -1 * self.hmom
			if self.x ~= self.x_clamped then self.x_clamped = math.floor(self.x + .5) end
		end
	end,
	vertical = function(self)
		local _, position = mobtools.doCollisionScan("vertical", self)
		if position == nil then
			self.y = self.y + self.vmom
			self.y_clamped = math.floor(self.y + .5)
		else
			self.y = position
			self.vmom = -1 * self.vmom
			if self.y ~= self.y_clamped then self.y_clamped = math.floor(self.y + .5) end
		end
	end,
}

function fireball:update()
	if self.animtimer > 0 then
		self.animtimer = self.animtimer - 1
	end
	if self.animtimer == 0 then
		self.frame = self.frame + 1
		if self.frame > 4 then self.frame = 1 end
		self.animtimer = self.animtimer_start
	end
	if self.hmom ~= 0 then self.collisionfunctions.horizontal(self) end
	if self.vmom ~= 0 then self.collisionfunctions.vertical(self) end
	if self.x + self.width < 0 or self.y + self.height < 0 or self.y > self.level.height or self.x > self.level.width then
		self.solid = false; self.irrelevant = true
	end
end

function fireball:draw()
	love.graphics.draw(graphics.load("fireball"), fireball.quads[self.frame], self.x_clamped, self.y_clamped)
end

return fireball