fireman = mob:new()

function fireman:init(x, y, alt, fast)
	self.type = "fireman"
	self.player = true
	self.playerno = 1
	self.x = x
	self.y = y
	self.x_clamped = x
	self.y_clamped = y
	self.solid = true
	self.hmom = 0
	self.vmom = 0
	self.speed = 2
	self.jumpstrength = 5
	self.gravity = 0.2
	self.height = tilesize * 2
	self.width = tilesize
	self.collisiondir = "none"
	self.alive = true
	self.grounded = true
end

function fireman:setup(x, y, options, level, edge, tags)
	return fireman:new(x, y)
end

function fireman:die() --rip
	local level = self.level
	self.alive = false
	self.solid = false
	self:playsfx("ogmo die")
	level.liveplayeramt = level.liveplayeramt - 1
	if level.liveplayeramt > 0 then print("fireman is dead! players remaining: " .. level.liveplayeramt) end
end

function fireman:update()
	if self.alive then
		self.collisiondir = "none"
		self.vmom = self.vmom + self.gravity
		if self.grounded then
			self.hmom = 0
			if love.keyboard.isDown("left") then
				self.hmom = -1 * self.speed
			elseif love.keyboard.isDown("right") then
				self.hmom = self.speed
			end
			
			if love.keyboard.isDown("s") then
				self.grounded = false
				self.vmom = self.jumpstrength * -1
			end
		end
		
		
		if self.hmom ~= 0 then
			self.collisiondir, newposition = mobtools.doCollisionScan("horizontal", self)
			if newposition ~= nil then
				self.x = newposition
				if (not self.grounded) and self.vmom >= 0 then
					if love.keyboard.isDown("s") then --walljump
						self.hmom = self.hmom * -1
						self.vmom = self.jumpstrength * -1
					end
				end
			else
				self.x = self.x + self.hmom
			end
		end
		if self.vmom ~= 0 then
			self.collisiondir, newposition = mobtools.doCollisionScan("vertical", self)
			if newposition ~= nil then
				self.y = newposition
				if self.collisiondir == "down" then
					self.grounded = true
					self.vmom = 0
				end
			else
				self.y = self.y + self.vmom
				self.grounded = false
			end
		end
		local overlaps = mobtools.doOverlapScan(self)
		for k,v in ipairs(overlaps) do
			if(v.deathly) then
				self:die()
				break
			end
		end
		self.x_clamped = math.floor(self.x + .5)
		self.y_clamped = math.floor(self.y + .5)
		
		--if math.abs(self.hmom) < 0.1 then
		if self.hmom == 0 then
			self.x = self.x_clamped
		end
		if self.vmom == 0 then
			self.y = self.y_clamped
		end
	end
end

function fireman:onCollide(collider, side)
	if collider.deathly then
		self:die()
	end
end

function fireman.editordraw(x, y, options)
	love.graphics.setColor(1,0,0)
	love.graphics.rectangle("fill", x, y, 16, 16)
	love.graphics.setColor(1,1,1)
end

function fireman:draw()
	if self.alive then
		love.graphics.setColor(1,0,0)
		love.graphics.rectangle("fill", self.x_clamped, self.y_clamped, self.width, self.height)
		love.graphics.setColor(1,1,1)
	end
end

return fireman