follower = mob:new()

local img = graphics.load("follower")

follower.quads = {
	["normal"] = quad(0,  0, 16, 16, img),
	["spiky"] = quad(16, 0, 16, 16, img),
}

function follower.editordraw(x, y, options)
	local myquad = "normal"
	if options["deathly"] then myquad = "spiky" end
	love.graphics.draw(graphics.load("follower"), follower.quads[myquad], x, y)
end

function follower:init(x, y, deathly)
	self.type = "follower"
	self.x = x
	self.y = y
	self.y_clamped = y
	self.x_clamped = x
	self.hmom = 0
	self.vmom = 0
	self.width = tilesize
	self.height = tilesize
	self.solid = true
	self.deathly = deathly
	self.kind = kind
	self.speed = 4 --ogmo max speed
	self.dontkillfollowers = true --spiked followers can't kill other followers in j2e v3.5
end

function follower:setup(x, y, options, level, edge, tags)
	local deathly = options["deathly"]
	return follower:new(x, y, not not deathly)
end

function follower:die()
	self.irrelevant = true
	self.solid = false
	self:playsfx("crackblock shatter")
end

function follower:onCollide(obj)
	if obj.deathly and not obj.dontkillfollowers then
		self:die()
	end
end

function follower:update()
	self.hmom = 0
	if self.level.camerafocus then
		local focus = self.level.camerafocus
		self.hmom = focus.x - self.x
		if math.abs(self.hmom) > self.speed then
			local sign = (self.hmom < 0 and -1) or 1
			self.hmom = self.speed * sign
		end
	end
	if math.abs(self.hmom) < 0.01 then self.hmom = 0 end
	if self.hmom ~= 0 then
		local walled, position, walledby = mobtools.doCollisionScan("horizontal", self)
		if position == nil then
			self.x = self.x + self.hmom
			self.x_clamped = math.floor(self.x + .5)
		else
			self.x = position
			if self.x ~= self.x_clamped then self.x_clamped = math.floor(self.x + .5) end
			for _,collider in ipairs(walledby) do
				if collider.onCollide then
					collider:onCollide(self, walled)
				end
			end
		end
	end
	local overlaps = mobtools.doOverlapScan(self)
	for _,obj in ipairs(overlaps) do
		if(obj.deathly and not obj.dontkillfollowers) then
			self:die(); break
		end
	end
end

function follower:draw()
	local myquad = "normal"
	if self.deathly then myquad = "spiky" end
	love.graphics.draw(graphics.load("follower"), follower.quads[myquad], self.x_clamped, self.y)
end

return follower