gate = object:new()

local img = graphics.load("gates")

gate.quads = {
	["or_v"] = 	quad(0, 0, 16, 16, img),
	["or_h"] = 	quad(16, 0, 16, 16, img),
	
	["on_u"] = 	quad(0, 16, 16, 16, img), --omg On_U Sound, the label by Adrians Herwood, collaborator with Keith LebLanc
	["on_d"] = 	quad(16, 16, 16, 16, img),
	["on_l"] = 	quad(32, 16, 16, 16, img),
	["on_r"] = 	quad(48, 16, 16, 16, img),
	
	["elec_u"] = quad(0, 32, 16, 16, img),
	["elec_d"] = quad(0, 48, 16, 16, img),
	["elec_l"] = quad(0, 64, 16, 16, img),
	["elec_r"] = quad(0, 80, 16, 16, img),
	["button_u"] = quad(16, 32, 16, 16, img),
	["button_d"] = quad(16, 48, 16, 16, img),
	["button_l"] = quad(16, 64, 16, 16, img),
	["button_r"] = quad(16, 80, 16, 16, img),
	["star_u"] = quad(32, 32, 16, 16, img),
	["star_d"] = quad(32, 48, 16, 16, img),
	["star_l"] = quad(32, 64, 16, 16, img),
	["star_r"] = quad(32, 80, 16, 16, img),
	["arrow_u"] = quad(48, 32, 16, 16, img),
	["arrow_d"] = quad(48, 48, 16, 16, img),
	["arrow_l"] = quad(48, 64, 16, 16, img),
	["arrow_r"] = quad(48, 80, 16, 16, img),
	["ghost_u"] = quad(64, 32, 16, 16, img),
	["ghost_d"] = quad(64, 48, 16, 16, img),
	["ghost_l"] = quad(64, 64, 16, 16, img),
	["ghost_r"] = quad(64, 80, 16, 16, img),
}

gate.conditions = {
	elec = function(self)
		return game.solid_colors.electricity
	end,
	button = function(self)
		if self.disjunct then
			return self.level:getAmt("padsheld") > 0
		else
			return self.level:amtIsMax("padsheld") and self.level:getMax("padsheld") > 0
		end
	end,
	star = function(self)
		if self.disjunct then
			return self.level:getAmt("starstaken") > 0
		else
			return self.level:amtIsMax("starstaken") and self.level:getMax("starstaken") > 0
		end
	end,
	arrow = function(self)
		if self.disjunct then
			if self.level:getAmt("arrowsgone") > 0 then
				self.satisfied = true
				return true
			end
		else
			if self.level:amtIsMax("arrowsgone") and self.level:getMax("arrowsgone") > 0 then
				self.satisfied = true
				return true
			end
		end
	end,
	ghost = function(self)
		if self.disjunct then
			return self.level:getAmt("ghostsgone") > 0
		else
			return self.level:amtIsMax("ghostsgone") and self.level:getMax("ghostsgone") > 0
		end
	end,
}

function gate.editordraw(x, y, options)
	local kind = options["kind"]
	local dir = options["dir"] or "up"
	local dirletter = string.sub(dir, 1, 1)
	love.graphics.draw(graphics.load("gates"), gate.quads[kind .. "_" .. dirletter], x, y)
end

function gate:init(x, y, dir, kind, disjunct)
	self.type = "gate"
	self.x = x
	self.y = y
	self.hmom = 0
	self.vmom = 0
	self.openspeed = 2
	self.x_from_origin = 0
	self.y_from_origin = 0
	self.width = tilesize
	self.height = tilesize
	self.solid = true
	self.dir = dir
	self.dirletter = string.sub(self.dir, 1, 1)
	self.state = "off"
	self.kind = kind --self.rude = false
	--self.condition = false
	self.disjunct = disjunct
	self.satisfied = false
	self.ridable = true
end

function gate:setup(x, y, options, level, edge, tags)
	local dir = options["dir"] or "up"
	local kind = options["kind"]
	local disjunct = tags["variant1"]
	if next(tags) then
		local directions = {"up", "down", "left", "right"}
		if tags["facerandom"] then dir = pick(directions)
		else
			for _,v in ipairs(directions) do
				if tags["face" .. v] then
					dir = v
					break
				end
			end
		end
	end
	return gate:new(x, y, dir, kind, disjunct)
end

function gate:update()
	self.state = "off"
	self.hmom = 0
	self.vmom = 0
	if self.satisfied or gate.conditions[self.kind](self) then self.state = "on" end
	if self.dir == "right" then
		if self.state == "on" and self.x_from_origin < 16 then
			self.hmom = self.openspeed
		elseif self.state == "off" and self.x_from_origin > 0 then
			self.hmom = -1 * self.openspeed
		end
	elseif self.dir == "left" then
		if self.state == "on" and self.x_from_origin > -16 then
			self.hmom = -1 * self.openspeed
		elseif self.state == "off" and self.x_from_origin < 0 then
			self.hmom = self.openspeed
		end
	elseif self.dir == "down" then
		if self.state == "on" and self.y_from_origin < 16 then
			self.vmom = self.openspeed
		elseif self.state == "off" and self.y_from_origin > 0 then
			self.vmom = -1 * self.openspeed
		end
	elseif self.dir == "up" then
		if self.state == "on" and self.y_from_origin > -16 then
			self.vmom = -1 * self.openspeed
		elseif self.state == "off" and self.y_from_origin < 0 then
			self.vmom = self.openspeed
		end
	end
	self.x = self.x + self.hmom
	self.x_from_origin = self.x_from_origin + self.hmom
	self.y = self.y + self.vmom
	self.y_from_origin = self.y_from_origin + self.vmom
end

function gate:draw()
	love.graphics.draw(graphics.load("gates"), gate.quads[self.kind .. "_" .. self.dirletter], self.x, self.y)
	if self.state == "on" then
		love.graphics.draw(graphics.load("gates"), gate.quads["on_" .. self.dirletter], self.x, self.y)
	end
	if self.disjunct then
		if self.dir == "left" or self.dir == "right" then
			love.graphics.draw(graphics.load("gates"), gate.quads["or_h"], self.x, self.y)
		else
			love.graphics.draw(graphics.load("gates"), gate.quads["or_v"], self.x, self.y)
		end
	end
	--printWithOutline(self.level:getAmt("padsheld"), 10, 10)
	--printWithOutline(self.level:getMax("padsheld"), 10, 20)
end

return gate