gem = object:new()

gem.automask = true
gem.maskimagename = "gem mask"

local img = graphics.load("gems")

gem.quads = {
	["outline_blue"] = love.graphics.newQuad(0, 0, 16, 16, img),
	["inside_blue"] = love.graphics.newQuad(16, 0, 16, 16, img),
	["inside_blueholo1"] = love.graphics.newQuad(32, 0, 16, 16, img),
	["inside_blueholo2"] = love.graphics.newQuad(48, 0, 16, 16, img),
	["outline_red"] = love.graphics.newQuad(0, 16, 16, 16, img),
	["inside_red"] = love.graphics.newQuad(16, 16, 16, 16, img),
	["inside_redholo1"] = love.graphics.newQuad(32, 16, 16, 16, img),
	["inside_redholo2"] = love.graphics.newQuad(48, 16, 16, 16, img),
}

function gem.editordraw(x, y, options)
	local kind = options["kind"] or "blue"
	love.graphics.draw(graphics.load("gems"), gem.quads["outline_" .. kind], x, y)
	love.graphics.draw(graphics.load("gems"), gem.quads["inside_" .. kind], x, y)
end

function gem:init(x, y, kind, room, savecollected)
	self.type = "gem"
	self.x = x
	self.y = y
	self.hmom = 0
	self.ymom = 0
	self.width = tilesize
	self.height = tilesize
	self.solid = false
	self.collected = false
	self.savecollected = savecollected
	self.kind = kind
	self.room = room
	self.parity = ((math.floor(self.x / 16) + math.floor(self.y / 16)) % 2 == 1)
end

function gem:setup(x, y, options, level, edge, tags)
	local kind = "blue"
	if options["kind"] then kind = options["kind"] end
	level:incMax(kind .. "gemscollected")
	
	local room = tostring(level.roomX) .. "," .. tostring(level.roomY)
	
	local savecollected = false
	if game.collectedgems[kind][room] then savecollected = true end
	--[[local collected = false
	if
		game.collectedallgems[kind][room] or
		game.unsavedallgems[kind][room]
	then
		collected = true
	end]]
	--instead of this, just make it so the level itself has a "bluegemcollected" variable and if so then the gem does nothing on overlap and is drawn as empty. likewise with red gems.
	
	return gem:new(x, y, kind, room, savecollected)
end

function gem:update()
end

function gem:onOverlap(obj)
	if self.collected or self.level.allgems[self.kind] then return end
	if(obj.player and not obj.ghost) then
		--todo: make the hologram sound shift in pitch depending on how many are left. that would be cute.
		local sfx = "gem hologram"
		self.collected = true
		local mecollected = self.kind .. "gemscollected"
		self.level:incAmt(mecollected)
		if self.level:amtIsMax(mecollected) then
			print("got all the gems wowwwww")
			sfx = "gem collect"
			
			game.unsavedgems[self.kind][self.room] = true
			self.level.allgems[self.kind] = true
			if self.kind == "red" then
				game.unsavedgems_redcount = game.unsavedgems_redcount + 1 --for statusbar
			else
				game.unsavedgems_bluecount = game.unsavedgems_bluecount + 1
			end
			statusbar.update("gems", self.level)
			
		elseif self.level:getAmt(mecollected) > self.level:getMax(mecollected) then
			print("too many gems :S")
		end
		self:playsfx(sfx)
	end
end

function gem:draw()
	local outline = "outline_" .. self.kind
	if self.savecollected then
		local r,g,b,a = love.graphics.getColor()
		love.graphics.setColor(r,g,b,a * 0.5)
		love.graphics.draw(graphics.load("gems"), self.quads[outline], self.x, self.y)
		love.graphics.setColor(r,g,b,a)
		return
	end
	love.graphics.draw(graphics.load("gems"), self.quads[outline], self.x, self.y)
	if self.level.allgems[self.kind] then return end
	local inside = "inside_" .. self.kind
	local r,g,b,a
	if self.collected then
		r,g,b,a = love.graphics.getColor()
		love.graphics.setColor(r,g,b,a * 0.5)
		local holobool = ((self.level.animtimer - 1) % 4 >= 2)
		if self.parity then holobool = not holobool end
		if holobool then
			inside = inside .. "holo1"
		else
			inside = inside .. "holo2"
		end
	end
	love.graphics.draw(graphics.load("gems"), self.quads[inside], self.x, self.y)
	if r then
		love.graphics.setColor(r,g,b,a)
	end
end

return gem