ghosttile = object:new()

local img = graphics.load("ghosttiles")

ghosttile.quads = {
	["up"] = quad(0, 0, 16, 5, img),
	["left"] = quad(16, 0, 5, 16, img),
	["right"] = quad(11, 16, 5, 16, img),
	["down"] = quad(16, 27, 16, 5, img),
}

function ghosttile.editordraw(x, y, options)
	local dir = options.dir
	x = x + ((dir ~= "right" and 0) or 11)
	y = y + ((dir ~= "down" and 0) or 11)
	love.graphics.draw(graphics.load("ghosttiles"), ghosttile.quads[options.dir or "up"], x, y)
end

function ghosttile:init(x, y, dir)
	self.type = "ghosttile"
	self.x = x
	self.y = y
	self.hmom = 0
	self.vmom = 0
	self.width = tilesize
	self.height = tilesize
	self.dir = dir --i keep forgetting this accidental kinda shitty design pattern. you're never actually supposed to refer to the arguments to init() because those might be nil when the object is cloned in the process of cloning the level (which happens for EVERY level so there's a "backup" level that can be loaded when you die without having to read the level from disk again). what happens is, we set self.dir to dir, which might be nil, but if it is nil then it will be treated as the self.dir of the object it was cloned from because we did metatable indexing stuff. messy shit!
	if not self.dir then self.dir = up end
	if self.dir == "left" or self.dir == "right" then
		self.width = 5
		if dir == "right" then self.x = self.x + 11 end --referring now to dir and not self.dir because this shouldn't apply again if the object is cloned (and thus dir is now nil). this can also be done in setup
	else
		self.height = 5
		if dir == "down" then self.y = self.y + 11 end
	end
	self.solid = true
end

function ghosttile:setup(x, y, options, level, edge, tags)
	return ghosttile:new(x, y, options.dir or "up")
end

local dircheckfuncs = {
	up = function(self, focus)
		if focus.y > self.y then return false end
		return (focus.y + focus.height <= self.y)
	end,
	right = function(self, focus)
		if focus.x < self.x then return false end
		return (focus.x >= self.x + self.width)
	end,
	down = function(self, focus)
		if focus.y < self.y then return false end
		return (focus.y >= self.y + self.height)
	end,
	left = function(self, focus)
		if focus.x > self.x then return false end
		return (focus.x + focus.width <= self.x)
	end,
}

function ghosttile:update()
	self.solid = false
	if not self.fake then
		if self.level.camerafocus and self.level.camerafocus.alive then
			local focus = self.level.camerafocus
			self.solid = dircheckfuncs[self.dir](self, focus)
		end
	end
end

function ghosttile:draw()
	local r,g,b,a = love.graphics.getColor()
	if not self.solid then love.graphics.setColor(r,g,b,0.25) end
	love.graphics.draw(graphics.load("ghosttiles"), self.quads[self.dir], self.x, self.y)
	love.graphics.setColor(r,g,b,a) --reset after level draws this
end

return ghosttile