ice = object:new()

function ice.editordraw(x, y, options)
	local dir = options.dir
	love.graphics.draw(graphics.load("ice"), ice.quads[options.dir or "up"], x, y)
end

local img = graphics.load("ice")

ice.quads = {
	["up"] = quad(0, 0, 16, 16, img),
	["left"] = quad(16, 0, 16, 16, img),
	["right"] = quad(0, 16, 16, 16, img),
	["down"] = quad(16, 16, 16, 16, img),
}

local dirsetups = {
	up = function(self)
		self.height = tilesize / 2
		self.y = self.y - 1
		self.y_draw_offset = 1
	end,
	left = function(self)
		self.width = tilesize / 2
		self.x = self.x - 1
		self.x_draw_offset = 1
	end,
	down = function(self)
		self.height = tilesize / 2
		self.y = self.y + (tilesize / 2) + 1
		self.y_draw_offset = -1 - (tilesize / 2)
	end,
	right = function(self)
		self.width = tilesize / 2
		self.x = self.x + (tilesize / 2) + 1
		self.x_draw_offset = -1 - (tilesize / 2)
	end,
}

local gravisdown = function(obj)
	return obj.ghost or (obj.gravity * obj.level.gravity >= 0)
end

local overlapfuncs = {
	up = function(self, obj)
		if obj.type == "ogmo" and (not obj.iced) and obj.grounded and gravisdown(obj) then
			obj.iced = true
		end
	end,
	left = function(self, obj)
		if obj.type == "ogmo" then
			obj.icewalled.right = true --this ice covers the left side of a surface, so ogmo can't walljump off the wall to his right
		end
	end,
	right = function(self, obj)
		if obj.type == "ogmo" then
			obj.icewalled.left = true
		end
	end,
	down = function(self, obj)
		if obj.type == "ogmo" and (not obj.iced) and obj.verticaled == "up" and (not gravisdown(obj)) then --downward ice makes ogmo slip if his gravity is upward and he's on a ceiling. i thought it would be cute
			obj.iced = true
		end
	end
}

function ice:init(x, y, dir)
	self.type = "ice"
	self.x = x
	self.y = y
	self.y_draw_offset = 0
	self.x_draw_offset = 0
	self.hmom = 0
	self.vmom = 0
	self.width = tilesize
	self.height = tilesize
	self.solid = false
	self.dir = dir
	self.onOverlap = overlapfuncs[self.dir] --this is a first, i think; an object setting one of its own functions in its init routine. there's probably some safety reason or other why custom objects shouldn't be allowed to do this
	dirsetups[self.dir](self)
end

function ice:setup(x, y, options, level, edge, tags)
	local dir = options.dir or "up"
	if tags["faceleft"] then dir = "left"
	elseif tags["faceright"] then dir = "right"
	elseif tags["facedown"] then dir = "down"
	elseif tags["facerandom"] then dir = pick(consts.directions) end
	return ice:new(x, y, dir)
end

function ice:update()
end

--function ice:onOverlap(obj)
--	self:overlapfunc(obj)
--end


function ice:draw()
	love.graphics.draw(graphics.load("ice"), self.quads[self.dir], self.x + self.x_draw_offset, self.y + self.y_draw_offset)
end

return ice