jumparrow = object:new()

jumparrow.automask = true

jumparrow.quads = {
	["active"] = love.graphics.newQuad(0, 0, 16, 16, 32, 16),
	["inactive"] = love.graphics.newQuad(16, 0, 16, 16, 32, 16)
}

function jumparrow.editordraw(x, y, options)
	love.graphics.draw(graphics.load("jumparrow"), jumparrow.quads["active"], x, y)
end

function jumparrow:init(x, y, dirs)
	self.type = "jumparrow"
	self.x = x
	self.y = y
	self.hmom = 0
	self.vmom = 0
	self.dirs = dirs --using a table is safe (see: the whole cloning shenanigans) because it never gets modified, so it doesn't matter that self.dirs will be nil if this is cloned and will point to the original table
	self.width = tilesize
	self.height = tilesize
	self.solid = false
	self.active = true
	self.inactivetimer = 0
	self.inactiveframes = 300 --estimate based on interval of cannon shots in j2e and definition of cannons as shooting every 100 frames in j4r
end

local colliderwidth = 12
local colliderheight = 14

local function spawncollider(self) --specifically uses a fireball, since fireballs already have the same collision logic that i'd want to put in moving arrows. this is one of those kludges that is going to feel awesome and resourceful to me and horrific to everyone else
	local dirs = self.dirs
	local tags = {}
	for k,v in pairs(dirs) do tags["face" .. v] = true end
	
	local collider = objects["fireball"]:setup(self.x + math.floor((self.width - colliderwidth) / 2), self.y + math.floor((self.height - colliderheight) / 2), {}, self.level, "none", tags)
	collider.width = colliderwidth
	collider.height = colliderheight
	--collider.type = "fireball_for_jumparrow" --not necessary because no other objects will ever see this object. it's not *in* level.objects or level.objmap
	
	self.collider = collider
	collider.level = self.level
	--table.insert(self.level.objects, collider)
	--self.level:updateobjmap(collider)
end

function jumparrow:late_init()
	if next(self.dirs) then
		spawncollider(self)
	end
end

--jumparrow.clone_init = jumparrow.late_init

function jumparrow:setup(x, y, options, level, edge, tags)
	level:incMax("arrowsgone")
	local dirs = {}
	if tags["facerandom"] then tags[pick{"faceup","faceleft","facedown","faceright"}] = true end
	if tags["faceup"] then dirs[#dirs + 1] = "up" end
	if tags["faceleft"] then dirs[#dirs + 1] = "left" end
	if tags["facedown"] then dirs[#dirs + 1] = "down" end
	if tags["faceright"] then dirs[#dirs + 1] = "right" end
	return jumparrow:new(x, y, dirs)
end

function jumparrow:update()
	if self.collider then
		self.collider.level = self.level
		self.collider:update()
		if self.collider.irrelevant then
			self.irrelevant = true
			if not self.active then self.level:decAmt("arrowsgone") end
			self.level:decMax("arrowsgone")
			return
		end
		self.x = self.collider.x - math.floor((self.width - colliderwidth) / 2)
		self.y = self.collider.y - math.floor((self.height - colliderheight) / 2)
	end
	if self.inactivetimer == 0 and not self.active then
		self.active = true
		self.level:decAmt("arrowsgone")
	else self.inactivetimer = self.inactivetimer - 1 end
end

function jumparrow:onOverlap(obj)
	if self.active then
		if(obj.type == "ogmo") and not (obj.starstate.gray) then
			if obj.jumps < (obj.defaultjumps - 1) then
				obj.jumps = obj.jumps + 1
				if self.level.camerafocus == obj then statusbar.update("jump") end
			end
			self.active = false
			self:playsfx("jump arrow collect")
			self.level:incAmt("arrowsgone")
			self.inactivetimer = self.inactiveframes
		end
	end
end

function jumparrow:draw()
	local mystate = "active"
	if not self.active then mystate = "inactive" end
	love.graphics.draw(graphics.load("jumparrow"), jumparrow.quads[mystate], self.x, self.y)
end

return jumparrow