jumppad = mob:new() --crate jumppad needs to access nudge

function jumppad.editordraw(x, y, options)
	local dir = options["responsedir"] or "up"
	local kind = options["kind"] or "normal"
	love.graphics.draw(graphics.load("jumppads"), jumppad.quads[kind .. "_" .. dir], x, y)
	love.graphics.draw(graphics.load("jumppads"), jumppad.quads[kind .. "_arrow_" .. dir], x, y)
end

local img = graphics.load("jumppads")

jumppad.oppositedirs = {
	["right"] = "left",
	["down"]  = "up",
	["left"]  = "right",
	["up"]    = "down"
}

jumppad.directions = {
	"left",
	"up",
	"right",
	"down"
}

jumppad.quads = {
	["normal_up"] = 			love.graphics.newQuad(0, 0, 16, 16, img),
	["normal_left"] = 			love.graphics.newQuad(16, 0, 16, 16, img),
	["normal_down"] = 			love.graphics.newQuad(32, 0, 16, 16, img),
	["normal_right"] = 			love.graphics.newQuad(48, 0, 16, 16, img),
	["normal_arrow_up"] = 		love.graphics.newQuad(64 + 0, 0, 16, 16, img),
	["normal_arrow_left"] = 	love.graphics.newQuad(64 + 16, 0, 16, 16, img),
	["normal_arrow_down"] = 	love.graphics.newQuad(64 + 32, 0, 16, 16, img),
	["normal_arrow_right"] = 	love.graphics.newQuad(64 + 48, 0, 16, 16, img),
	["fragile_up"] = 			love.graphics.newQuad(0, 16, 16, 16, img),
	["fragile_left"] = 			love.graphics.newQuad(16, 16, 16, 16, img),
	["fragile_down"] = 			love.graphics.newQuad(32, 16, 16, 16, img),
	["fragile_right"] = 		love.graphics.newQuad(48, 16, 16, 16, img),
	["fragile_arrow_up"] = 		love.graphics.newQuad(64 + 0, 16, 16, 16, img),
	["fragile_arrow_left"] = 	love.graphics.newQuad(64 + 16, 16, 16, 16, img),
	["fragile_arrow_down"] = 	love.graphics.newQuad(64 + 32, 16, 16, 16, img),
	["fragile_arrow_right"] = 	love.graphics.newQuad(64 + 48, 16, 16, 16, img),
	["crate_up"] = 				love.graphics.newQuad(0, 32, 16, 16, img),
	["crate_left"] = 			love.graphics.newQuad(16, 32, 16, 16, img),
	["crate_down"] = 			love.graphics.newQuad(32, 32, 16, 16, img),
	["crate_right"] = 			love.graphics.newQuad(48, 32, 16, 16, img),
	["crate_arrow_up"] = 		love.graphics.newQuad(64 + 0, 32, 16, 16, img),
	["crate_arrow_left"] = 		love.graphics.newQuad(64 + 16, 32, 16, 16, img),
	["crate_arrow_down"] = 		love.graphics.newQuad(64 + 32, 32, 16, 16, img),
	["crate_arrow_right"] = 	love.graphics.newQuad(64 + 48, 32, 16, 16, img),
	["arrow_up_active"] = 		love.graphics.newQuad(64 + 0, 48, 16, 16, img),
	["arrow_left_active"] = 	love.graphics.newQuad(64 + 16, 48, 16, 16, img),
	["arrow_down_active"] = 	love.graphics.newQuad(64 + 32, 48, 16, 16, img),
	["arrow_right_active"] = 	love.graphics.newQuad(64 + 48, 48, 16, 16, img),
}

function jumppad:init(x, y, kind, dir, dir2)
	self.type = "jumppad"
	self.x = x
	self.y = y
	self.y_clamped = y
	self.x_clamped = x
	self.hmom = 0
	self.vmom = 0
	self.width = tilesize
	self.height = tilesize
	self.solid = true
	self.responsedir = dir
	self.launchdir = dir2
	self.timer = 0
	self.kind = kind
	self.launchspeed = 8
	self.gravity = 4 --crate
	self.speed = 1 --crate
	self.dropheight = 0 --crate
	self.crushduckingogmo = true --crate
	self.canride = true --crate
end

function jumppad:setup(x, y, options, level, edge, tags)
	local dir = options["responsedir"] or "up"
	local dir2 = dir
	local kind = options["kind"] or "normal"
	if tags["facerandom"] then dir2 = pick(jumppad.directions)
	elseif next(tags) then
		for i,v in ipairs(jumppad.directions) do
			if tags["face" .. v] then
				dir2 = v
				break
			end
		end
	end
	return jumppad:new(x, y, kind, dir, dir2)
end

function jumppad:update()
	if self.kind == "crate" then
		objects.crate.update(self)
	end
	if self.timer > 0 then
		self.timer = self.timer - 1
	end
end

function jumppad:onCollide(collidee, side)
	if collidee.type == "ogmo" then
		if side == jumppad.oppositedirs[self.responsedir] then
			if self.timer == 0 and (self.launchdir == self.responsedir or self.kind == "fragile") then self:playsfx("superjump") end
			self.timer = 8
			local dir = self.launchdir
			if self.launchdir == jumppad.oppositedirs[self.responsedir] then
				collidee["stuck" .. self.launchdir] = true
			end
			if dir == "up" then collidee.vmom = -1 * self.launchspeed
			elseif dir == "down" then collidee.vmom = 1 * self.launchspeed
			elseif dir == "left" or dir == "right" then
				if dir == "left" then collidee.hmom = -1 * self.launchspeed
				elseif dir == "right" then collidee.hmom = 1 * self.launchspeed
				end
				collidee.tempmaxspeed = self.launchspeed
				--below is copypasted from rubber
				local frictiondivisor = math.abs(collidee.hmom) * 2
				if frictiondivisor < 1 then frictiondivisor = 1
				elseif frictiondivisor > 6 then frictiondivisor = 6 end
				collidee.tempfriction = collidee.friction / (frictiondivisor)
				collidee.tempfrictiontimer = collidee.walljumptempfrictionframes
			end
			if self.kind == "fragile" then self.irrelevant = true; self.solid = false; end
		end
		if self.kind == "crate" and (side == "left" or side == "right") then
			if side == jumppad.oppositedirs[self.responsedir] then
				if self.launchdir ~= self.responsedir then
					if self.launchdir == "up" then --if you're trying to push a jumppad that's going to launch you straight up, only allow it if the player cannot go up any further
						local oldvmom = collidee.vmom
						collidee.vmom = -(1/16) --16th of a pixel
						local collisionresult
						collisionresult = mobtools.doCollisionScan("vertical", collidee)
						collidee.vmom = oldvmom
						if collisionresult ~= "none" then objects.crate.onCollide(self, collidee, side) end
					else
						objects.crate.onCollide(self, collidee, side)
					end
				end
			else
				objects.crate.onCollide(self, collidee, side)
			end
		end
	end
end

function jumppad:draw()
	love.graphics.draw(graphics.load("jumppads"), jumppad.quads[self.kind .. "_" .. self.responsedir], self.x_clamped, self.y_clamped)
	love.graphics.draw(graphics.load("jumppads"), jumppad.quads[self.kind .. "_arrow_" .. self.launchdir], self.x_clamped, self.y_clamped)
	if self.timer > 0 then
		love.graphics.draw(graphics.load("jumppads"), jumppad.quads["arrow_" .. self.launchdir .. "_active"], self.x_clamped, self.y_clamped)
	end
end

return jumppad