kitty = object:new()

--kitty.manual_invisible_draw = true

local img = graphics.load("kitten")

kitty.quads = {
	["left1"] = quad(0, 0, 16, 16, img),
	["left2"] = quad(16, 0, 16, 16, img),
	["right1"] = quad(0, 16, 16, 16, img),
	["right2"] = quad(16, 16, 16, 16, img),
}

function kitty.editordraw(x, y, options)
	love.graphics.draw(graphics.load("kitten"), kitty.quads["left1"], x, y)
end

function kitty:init(x, y, text, name, r, g, b, anim_max, namecolor, secret)
	self.type = "kitty"
	self.x = x
	self.y = y
	self.hmom = 0
	self.vmom = 0
	self.width = (tilesize * 3) --large hitbox for overlap purposes, so you can press A while you're next to the kitty but not visually overlapping it
	self.height = tilesize
	self.solid = false
	self.keytoprint = false
	self.justoverlapped = false
	self.name = name
	self.text = text
	self.dir = "left"
	self.r = r
	self.g = g
	self.b = b
	self.namecolor = namecolor
	self.animtimer = 0
	self.animtimer_max = anim_max
	self.phase = "1"
	self.secret = secret
end

function kitty:setup(x, y, options, level, edge, tags)
	local number = 1
	for i=0, 9 do
		if tags["variant" .. i] then
			number = i; break;
		end
	end
	if not number then number = 1 end
	local name = "ms. placeholder"
	local text = {"i am a placeholder.","i hold places.","it's tough work but someone's gotta do it."}
	if level.npcs and level.npcs[number] then
		name = level.npcs[number].name
		text = table.copy(level.npcs[number].text)
	end
	for i,v in ipairs(text) do
		text[i] = string.gsub(v, "\\n", "\n")
	end
	name = string.gsub(name, "\\n", "\n")
	local seed1 = (x * 100000) + y
	local seed2 = ((level.roomX or 0) * 100000) + (level.roomY or 0)
	--[[
	xpos  ypos
	000001
		  000001
	]]
	--[[
	xroom yroom
	000001
		  000001
	]]
	local r = 1
	local g = 1
	local b = 1
	local anim_max = 30
	local namecolor = {}
	local secret = false
	if not tags["invisible"] or tags["facerandom"] then
		local kittyrng = love.math.newRandomGenerator(seed1, seed2)
		r = kittyrng:random(127, 255) / 255
		g = kittyrng:random(127, 255) / 255
		b = kittyrng:random(127, 255) / 255
		local lightness = kittyrng:random() --when called without arguments, generates a real between 0 and 1
		--lightness = lightness * lightness --square it so it's closer to 1 or 0
		--r = r + ((1 - r) * lightness)
		--g = g + ((1 - g) * lightness)
		--b = b + ((1 - b) * lightness)
		anim_max = kittyrng:random(10, 60)
	end
	if tags["facerandom"] then
		r = 255/255
		g = 106/255
		b = 0/255
		anim_max = 10
		secret = true
	end
	namecolor[1] = r
	namecolor[2] = g
	namecolor[3] = b
	local newkitty = kitty:new(x - tilesize, y, text, name, r, g, b, anim_max, namecolor, secret)
	newkitty.tags = tags
	if newkitty.tags["invisible"] then
		newkitty.tags["invisible"] = false
		newkitty.invisible = true
	end
	return newkitty
end

function kitty:update()
	self.animtimer = self.animtimer + 1
	if self.animtimer > self.animtimer_max then
		if self.phase == "1" then self.phase = "2"
		else self.phase = "1" end
		self.animtimer = 1
	end
	if self.level.camerafocus then
		if self.level.camerafocus.x < self.x + tilesize then self.dir = "left"
		else self.dir = "right" end
	end
	if self.justoverlapped then
		self.justoverlapped = false
	else
		self.showtext = false
	end
end

function kitty:onOverlap(obj)
	if obj.player and (not obj.ghost) then
		self.showtext = true
		self.justoverlapped = true
	end
end

function kitty:onInteract(obj)
	if obj.player and (not obj.ghost) and not game.pausedfortextbox then
		self:playsfx("npc talk")
		for i,v in ipairs(self.text) do
			game.showtextbox(v, self.name, self.namecolor)
		end
	end
end

function kitty:draw()
	if not self.invisible then
		if self.secret then
			love.graphics.draw(graphics.load("999"), kitty.quads[self.dir .. self.phase], self.x + tilesize, self.y)
		else
			local r,g,b,a = love.graphics.getColor()
			love.graphics.setColor(self.r * r, self.g * g, self.b * b, a)
			love.graphics.draw(graphics.load("kitten"), kitty.quads[self.dir .. self.phase], self.x + tilesize, self.y)
			love.graphics.setColor(r, g, b, a)
		end
	end
	if self.showtext then
		local text = game.interactprompt
		local x = self.x + tilesize + (tilesize / 2) - math.floor(text:getWidth() / 2) --extra tilesize is because kitty's visual x position is offset from its actual x position, so its overlap hitbox can be wider
		local y = self.y - tilesize
		local r,g,b,a = love.graphics.getColor()
		love.graphics.setColor(1,1,1,1) --you can't have color tags apply to interact prompts but you *can* have additive tags apply because i'm too lazy to disable additive blending lol.
		drawWithOutline(text, x, y)
		love.graphics.setColor(r,g,b,a)
		--printWithOutline(self.keytoprint, self.x + 4, self.y - tilesize) --+4 is completely arbitrary because i don't feel like centering it properly right now
	end
end

return kitty