lightbridge = object:new()

local img = graphics.load("lightbridge")

lightbridge.quads = {
	right = quad(0, 0, 16, 16, img),
	left = quad(0, 40, 16, 16, img),
	meter_right = {
		[1] = quad(0, 16, 8, 8, img),
		[2] = quad(8, 16, 8, 8, img),
		[3] = quad(16, 16, 8, 8, img),
		[4] = quad(0, 24, 8, 8, img),
		[5] = quad(8, 24, 8, 8, img),
		[6] = quad(16, 24, 8, 8, img),
		[7] = quad(0, 32, 8, 8, img),
		[8] = quad(8, 32, 8, 8, img),
		[9] = quad(16, 32, 8, 8, img),
	},
	meter_right_lights = {
		[1] = quad(24 + 0, 16, 8, 8, img),
		[2] = quad(24 + 8, 16, 8, 8, img),
		[3] = quad(24 + 16, 16, 8, 8, img),
		[4] = quad(24 + 0, 24, 8, 8, img),
		[5] = quad(24 + 8, 24, 8, 8, img),
		[6] = quad(24 + 16, 24, 8, 8, img),
		[7] = quad(24 + 0, 32, 8, 8, img),
		[8] = quad(24 + 8, 32, 8, 8, img),
		[9] = quad(24 + 16, 32, 8, 8, img),
	},
	meter_left = {
		[1] = quad(0, 16 + 40, 8, 8, img),
		[2] = quad(8, 16 + 40, 8, 8, img),
		[3] = quad(16, 16 + 40, 8, 8, img),
		[4] = quad(0, 24 + 40, 8, 8, img),
		[5] = quad(8, 24 + 40, 8, 8, img),
		[6] = quad(16, 24 + 40, 8, 8, img),
		[7] = quad(0, 32 + 40, 8, 8, img),
		[8] = quad(8, 32 + 40, 8, 8, img),
		[9] = quad(16, 32 + 40, 8, 8, img),
	},
	meter_left_lights = {
		[1] = quad(24 + 0, 16 + 40, 8, 8, img),
		[2] = quad(24 + 8, 16 + 40, 8, 8, img),
		[3] = quad(24 + 16, 16 + 40, 8, 8, img),
		[4] = quad(24 + 0, 24 + 40, 8, 8, img),
		[5] = quad(24 + 8, 24 + 40, 8, 8, img),
		[6] = quad(24 + 16, 24 + 40, 8, 8, img),
		[7] = quad(24 + 0, 32 + 40, 8, 8, img),
		[8] = quad(24 + 8, 32 + 40, 8, 8, img),
		[9] = quad(24 + 16, 32 + 40, 8, 8, img),
	},
	platform = quad(16, 3, 16, 10, img)
}

function lightbridge.editordraw(x, y, options)
	local dir = "right"
	if options.dir == "left" then dir = "left" end
	love.graphics.draw(graphics.load("lightbridge"), lightbridge.quads[dir], x, y)
end

--[[
__________________________________

the bridges themselves
__________________________________
]]

local bridge = object:new()

local withreverser = {["reverser"] = true}
local collideup = {["up"] = true}
local function bridge_collisioncondition(self, collidee, axis)
	if collidee.player then return false
	elseif collidee.type == "crate" and collidee.dropheight ~= 0 then
		return false
	end
	return true
end

function bridge:init(x, y, hmom, owner)
	self.type = "lightbridge_bridge"
	self.drawearly = true
	self.x = x
	self.y = y
	self.width = 16
	self.height = 10
	self.x_clamped = self.x
	self.hmom = hmom
	self.speed = math.abs(hmom)
	self.solid = true
	self.owner = owner
	self.collidewith = withreverser
	self.collsides = collideup
	self.collisioncondition = bridge_collisioncondition
end

function bridge:onCollide(collidee, side)
	self.direction = "right"
	if self.hmom < 0 then self.direction = "left" end
	objects["platform"].onCollide(self, collidee, side)
end

function bridge:onBoom()
	self:destroy()
end

function bridge:destroy()
	self.irrelevant = true
	for i,v in ipairs(self.owner.bridges) do
		if v == self then
			table.remove(self.owner.bridges, i)
			break
		end
	end
end

function bridge:update()
	local walled, position, walledby = mobtools.doCollisionScan("horizontal", self)
	if position == nil then
		self.x = self.x + self.hmom
		self.x_clamped = math.floor(self.x + .5)
	else
		local justreverse = true
		for i,v in ipairs(walledby) do
			if v.onCollide then
				v:onCollide(self, walled)
			end
			if not v.reverser then
				justreverse = false
			end
		end
		if justreverse then
			self.x = position
			if self.x ~= self.x_clamped then self.x_clamped = math.floor(self.x + .5) end
			self.hmom = self.hmom * -1
		else
			self:destroy()
		end
	end
end

function bridge:draw()
	love.graphics.draw(graphics.load("lightbridge"), lightbridge.quads["platform"], self.x, self.y)
end

--[[
__________________________________

bridge generator
__________________________________
]]

function lightbridge:init(x, y, dir, amt)
	self.type = "lightbridge"
	self.drawlate = true
	self.x = x
	self.y = y
	self.hmom = 0
	self.vmom = 0
	self.width = tilesize
	self.height = tilesize
	self.solid = true
	self.dir = dir
	self.amt = amt
	self.bridges = {}
	self.speed = 1
end

function lightbridge:setup(x, y, options, level, edge, tags)
	local dir = options.dir or "right"
	if tags["faceleft"] then dir = "left" end
	if tags["facerandom"] then dir = pick({"left", "right"}) end
	local amt = 1
	if next(tags) then
		for i=1, 9 do
			if tags["variant" .. i] then
				amt = i; break;
			end
		end
	end
	return lightbridge:new(x, y, dir, amt)
end

function lightbridge:update()
end

function lightbridge:onDuck(ducker)
	if #self.bridges == self.amt then
		self.bridges[1]:destroy()
		--the bridge's :destroy() code takes care of shifting self.bridges
	end
	local b = bridge:new(self.x, self.y + 3, self.speed * (self.dir == "left" and -1 or 1), self)
	b.level = self.level
	b.tags = self.tags
	table.insert(self.bridges, b)
	table.insert(self.level.objects, b)
	self.level:updateobjmap(b)
end

function lightbridge:draw()
	local img = graphics.load("lightbridge")
	love.graphics.draw(img, lightbridge.quads[self.dir], self.x, self.y)
	local meter = math.clamp(self.amt, 1, 9)
	local meter_lights = math.clamp(#self.bridges, 0, 9)
	if self.dir == "right" then
		local x = self.x
		local y = self.y + 8
		love.graphics.draw(img, lightbridge.quads.meter_right[meter], x, y)
		if meter_lights > 0 then
			love.graphics.draw(img, lightbridge.quads.meter_right_lights[meter_lights], x, y)
		end
	else
		local x = self.x + 8
		local y = self.y + 8
		love.graphics.draw(img, lightbridge.quads.meter_left[meter], x, y)
		if meter_lights > 0 then
			love.graphics.draw(img, lightbridge.quads.meter_left_lights[meter_lights], x, y)
		end
	end
end

return lightbridge