platform = object:new()

local img = graphics.load("platform")

platform.quads = {
	["big"] = quad(0, 0, 32, 16, img),
	["small"] = quad(32, 0, 16, 16, img),
	["big_on"] = quad(0, 16, 32, 16, img),
	["small_on"] = quad(32, 16, 16, 16, img),
	["big_off"] = quad(0, 32, 32, 16, img),
	["small_off"] = quad(32, 32, 16, 16, img),
	["right_off"] = quad(96, 0, 16, 16, img),
	["right_on"] = quad(112, 0, 16, 16, img), --right on bro
	["left_off"] = quad(96, 16, 16, 16, img),
	["left_on"] = quad(112, 16, 16, 16, img),
}

function platform.editordraw(x, y, options)
	if options["battery"] then
		love.graphics.draw(graphics.load("platform"), platform.quads["small_off"], x, y)
	elseif options["onedir"] then
		local dir = "right"
		if options["dir"] then dir = options["dir"] end
		love.graphics.draw(graphics.load("platform"), platform.quads[dir .. "_off"], x, y)
	else
		love.graphics.draw(graphics.load("platform"), platform.quads["small"], x, y)
	end
end

function platform:init(x, y, dir, kind, on, size, width)
	self.type = "platform"
	self.x = x
	self.y = y
	self.x_clamped = x
	self.y_clamped = y
	self.hmom = 0
	self.vmom = 0
	self.width = width
	self.height = tilesize
	self.solid = true
	self.speed = 2
	self.direction = dir
	self.collidewith = {["reverser"] = true}
	self.indestructible = true
	self.size = size
	self.kind = kind
	if self.kind == "onedir" then self.speed = 1 end
	self.on = on
	self.itstopped = false
end

function platform:setup(x, y, options, level, edge, tags)
	local dir = "right"
	local kind = "normal"
	local on = false
	if options["dir"] then dir = options["dir"] end
	if tags["faceleft"] then dir = "left" end
	if options["battery"] then kind = "battery" end
	if options["onedir"] then kind = "onedir" end
	if tags["variant1"] then on = true end
	local spawn = true
	local lastobj = level.objects[#level.objects]
	if (kind ~= "onedir") and lastobj and lastobj["type"] == "platform" and lastobj.y == y and lastobj.x == x - tilesize and lastobj.size == "small" and lastobj.direction == dir and lastobj.kind == kind and lastobj.on == on then
		lastobj.size = "big"
		lastobj.width = tilesize * 2
		spawn = false
	end
	if spawn then
		return platform:new(x, y, dir, kind, on, "small", tilesize)
	end
end

function platform:invertvoltage()
	if self.kind ~= "onedir" and self.kind ~= "battery" then --onedirs are meant to be treated more like follower blocks than normal platforms. Follower blocks don't care about voltage or electricity. Battery platforms are also excluded because they "runs on [their] own battery" and aren't affected by whether electricity is on, so logically they shouldn't care about voltage either.
		self.direction = consts.oppositedirs[self.direction]
	end
end

function platform:update()
	self.hmom = 0
	self.itstopped = false
	if (self.kind == "normal" and game.solid_colors.electricity) or (self.on) then
		if self.direction == "right" then
			self.hmom = self.speed
		elseif self.direction == "left" then
			self.hmom = -1 * self.speed
		end
	else
		self.x = self.x_clamped
	end
	local walled, position, walledby = mobtools.doCollisionScan("horizontal", self)
	if position == nil then
		self.x = self.x + self.hmom
		self.x_clamped = math.floor(self.x + .5)
	else
		self.itstopped = true
		local hitnothing = true
		local reverses = false
		for i,v in ipairs(walledby) do
			if hitnothing then
				hitnothing = false
				if v.type == "crate" and v.dropheight ~= 0 then --don't reverse for falling crates
					hitnothing = true
				elseif v.type == "ogmo" and ( --niceness for when ogmo dismounts a platform in the direction it's moving to prevent him from changing the platform's direction for 1 frame
					(not v.grounded) and
					v.hmom * self.hmom >= 0 and
					( --if falling ogmo won't be in the way of the platform next frame then don't reverse
						(self.hmom > 0 and (self.x + self.width + self.hmom) < (v.x + v.hmom)) or
						(self.hmom < 0 and (self.x + self.hmom) > (v.x + v.width + v.hmom))
					)
				) then
					hitnothing = true
				end
			end
			if v.reverser then
				reverses = true
			end
		end
		self.x = position
		if (not hitnothing) and ((self.kind ~= "onedir") or reverses) then self.direction = consts.oppositedirs[self.direction] end
		if self.x ~= self.x_clamped then self.x_clamped = math.floor(self.x + .5) end
	end
	if self.kind == "onedir" then --one-direction platforms resemble follower blocks and die from the same hazards as follower blocks in jumper redux
		local overlaps = mobtools.doOverlapScan(self)
		for _,obj in ipairs(overlaps) do
			if(obj.deathly and not obj.dontkillfollowers) then
				objects["follower"].die(self); break
			end
		end
	end
end

function platform:onDuck(ducker)
	if self.kind == "battery" then
		self.on = not self.on
		self:playsfx("electricity switch")
	end
end

function platform:onCollide(collidee, side)
	if self.type == "platform" then --in case things that aren't platforms try to call this function, skip the "is this on", "is the electricity on", etc. stuff. so far nothing but platform actually calls this i think? crates could if they're on a platform so that stacked crates work
		if self.itstopped then return end
		if (self.kind == "onedir" and not self.on) and (side == "down" and collidee.nudge and collidee.solid) then
			self.on = true
		end
		if self.kind == "battery" or self.kind == "onedir" and not self.on then return end
		if self.kind == "normal" and not game.solid_colors.electricity then return end
	end
	if (side == "down" and collidee.nudge) or (side == "up" and collidee.type == "ogmo" and (not collidee.ghost) and self.level.gravity < 0) or (collidee.type == "walkyspike") then
		local sign = 1
		if self.direction == "left" then sign = -1 end
		local temps = self.level.temps
		if not temps.platformed then
			temps.platformed = {}
		end
		local obj = collidee
		if not temps.platformed[obj] then --each table in temps is cleared after every update
			--local unblocked_h, unblocked_v, blockinfo = obj:nudge(self.speed, 0, true)
			obj:nudge(self.speed * sign, 0)
			temps.platformed[obj] = true
		end
	end
end

function platform:draw()
	if self.kind == "battery" then
		local myquad = "_on"
		if not self.on then myquad = "_off" end
		love.graphics.draw(graphics.load("platform"), self.quads[self.size .. myquad], self.x_clamped, self.y_clamped)
	elseif self.kind == "onedir" then
		local myquad = "_on"
		if not self.on then myquad = "_off" end
		love.graphics.draw(graphics.load("platform"), self.quads[self.direction .. myquad], self.x_clamped, self.y_clamped)
	else
		love.graphics.draw(graphics.load("platform"), self.quads[self.size], self.x_clamped, self.y_clamped)
	end
end

return platform