pressurepad = object:new()

--name is a misnomer since it doesn't actually require pressure - it just requires something to be THERE. oncollide would make it so that something actually has to collide with it for it to be pressed, but this behavior is consistent with j2e and allows for cracked blocks to press buttons which is pretty cool

local img = graphics.load("pressurepad")

pressurepad.quads = {
	["off_up"] = 	quad(0, 0, 16, 16, img),
	["on_up"] = 	quad(16, 0, 16, 16, img),
	["off_down"] = 	quad(0, 16, 16, 16, img),
	["on_down"] = 	quad(16, 16, 16, 16, img),
	["off_left"] = 	quad(0, 32, 16, 16, img),
	["on_left"] = 	quad(16, 32, 16, 16, img),
	["off_right"] = quad(0, 48, 16, 16, img),
	["on_right"] = 	quad(16, 48, 16, 16, img),
}
function pressurepad.editordraw(x, y, options)
	love.graphics.draw(graphics.load("pressurepad"), pressurepad.quads["off_up"], x, y)
end

function pressurepad:init(x, y, dir)
	self.type = "pressurepad"
	self.x = x
	self.y = y
	self.hmom = 0
	self.vmom = 0
	self.width = tilesize
	self.height = tilesize
	self.solid = true
	self.dir = dir
	self.state = "off"
	self.laststate = "off"
	
	self.collidewith = {["fallyspike"] = true}
	self.collisioncondition = function(self, collidee)
		if collidee.type == "fallyspike" and not collidee.mine then return false end
		return true
	end
end

function pressurepad:setup(x, y, options, level, edge, tags)
	level:incMax("padsheld")
	local dir = "up"
	if next(tags) then
		local directions = {"up", "down", "left", "right"}
		if tags["facerandom"] then dir = pick(directions)
		else
			for _,v in ipairs(directions) do
				if tags["face" .. v] then
					dir = v
					break
				end
			end
		end
	end
	return pressurepad:new(x, y, dir)
end

function pressurepad:update()
	self.laststate = self.state
	self.state = "off"
	
	if self.dir == "left" or self.dir == "right" then
		local oldhmom = self.hmom
		self.hmom = -1
		if self.dir == "right" then self.hmom = 1 end
		local collisionresult
		collisionresult = mobtools.doCollisionScan("horizontal", self)
		self.hmom = oldhmom
		if collisionresult ~= "none" then
			self.state = "on"
		end
	else
		local oldvmom = self.vmom
		self.vmom = -1
		if self.dir == "down" then self.vmom = 1 end
		local collisionresult
		collisionresult = mobtools.doCollisionScan("vertical", self)
		self.vmom = oldvmom
		if collisionresult ~= "none" then
			self.state = "on"
		end
	end
	
	if self.laststate == "off" and self.state == "on" then
		self.level:incAmt("padsheld")
	elseif self.laststate == "on" and self.state == "off" then
		self.level:decAmt("padsheld")
	end
end

function pressurepad:draw()
	love.graphics.draw(graphics.load("pressurepad"), pressurepad.quads[self.state .. "_" .. self.dir], self.x, self.y)
end

return pressurepad