local img = graphics.load("sentry")

--sentry bullet
local bullet = object:new()
--cross-references from cannonball a lot

bullet.quads = {
	[1] = quad(32, 0, 6, 6, img),
	[2] = quad(32, 0, 6, 6, img),
	[3] = quad(32, 6, 6, 6, img),
	[4] = quad(32, 6, 6, 6, img),
	[5] = quad(32, 12, 6, 6, img),
	[6] = quad(32, 12, 6, 6, img),
	[7] = quad(32, 18, 6, 6, img),
	[8] = quad(32, 18, 6, 6, img),
}

function bullet:init(x, y, hmom, vmom, green)
	self.x = x
	self.y = y
	self.width = 6
	self.height = 6
	self.hmom = hmom
	self.vmom = vmom
	self.deathly = true
	self.solid = false
	--self.weak = true
	self.dontkillfollowers = true
	self.frame = 1
	self.lastframe = 8
	self.animtimer = 1
	self.animtimer_start = 1
	self.green = green
end

bullet.collisionfunctions = {
	horizontal = function(...)
		objects["cannonball"].collisionfunctions.horizontal(...)
	end,
	vertical = function(...)
		objects["cannonball"].collisionfunctions.vertical(...)
	end,
}
function bullet:onOverlap(obj)
	if obj.player then
		self.irrelevant = true --self.deathly remains true, so the player will still die
		self.level:decAmt("sentrybullets") --maybe i should add support to run an "on remove" function that runs whenever an object is removed due to obj.irrelevant being true. so that if another object decides to make my object irrelevant the sentrybullets amt can properly decrease (i think there aren't any objects that just generally decide "lol your object is irrelevant now" but that might happen in more specific situations like ogmo walljumping off of something fragile)
	end
end

function bullet:update()
	objects["cannonball"].update(self)
	if self.irrelevant then
		self.level:decAmt("sentrybullets")
	end
end

function bullet:draw()
	if self.silhouette then
		love.graphics.draw(graphics.load("sentry"), bullet.quads[self.frame], self.x, self.y)
	else
		local r,g,b,a = love.graphics.getColor()
		--if r ~= 1 or g ~= self.green or b ~= 
		love.graphics.setColor(1 * r, self.green * g, 0, 1)
		love.graphics.draw(graphics.load("sentry"), bullet.quads[self.frame], self.x, self.y)
		love.graphics.setColor(r,g,b,a)
	end
end

--sentry itself
sentry = object:new()

--quad setup
sentry.quads = {}
sentry.quads["base"] = {}
sentry.quads["shine"] = {}

local frameorder = {
	1,
	1,
	1,
	1,
	1,
	2,
	2,
	3,
	3,
	3,
	4,
	4,
	4,
	4,
	4,
	3,
	3,
	2,
	2,
	2,
}

local frames

local function setquads(t)
	for i,v in ipairs(frameorder) do
		t[i] = frames[v]
	end
end

frames = {
	quad(0, 0, 8, 8, img),
	quad(8, 0, 8, 8, img),
	quad(16, 0, 8, 8, img),
	quad(24, 0, 8, 8, img),
}

setquads(sentry.quads["base"])

frames = {
	quad(0, 8, 8, 8, img),
	quad(8, 8, 8, 8, img),
	quad(16, 8, 8, 8, img),
	quad(24, 8, 8, 8, img),
}

setquads(sentry.quads["shine"])

sentry.quads["editor"] = quad(0, 16, 8, 8, img)
--end quad setup


function sentry.editordraw(x, y, options)
	love.graphics.draw(graphics.load("sentry"), sentry.quads["editor"], x + 4, y + 4)
end

function sentry:init(x, y, green, timerstart)
	self.type = "sentry"
	self.x = x
	self.y = y
	self.hmom = 0
	self.ymom = 0
	self.width = 8
	self.height = 8
	self.solid = false
	self.green = green
	self.timerstart = timerstart
	self.timer = self.timerstart
	self.tooclose = false
	self.minimumdistance = 32
	self.bulletspeed = 3
end

function sentry:setup(x, y, options, level, edge, tags)
	level:setMax("sentrybullets",64)
	local inverse = options["inverse"]
	local green = 0
	local timerstart = 120 --sentries take a little longer to fire than cannons; this number is an estimate
	--actually that estimate turned out spot-on; sentries have 20 frames (in j2e, which runs at 30fps) and fire after 3 pulses. woo!
	if tags["variant1"] then green = 0.333; timerstart = 80; end
	if tags["variant2"] then green = 0.666; timerstart = 40; end
	--[[if next(tags) then
		for i=0, 9 do
			if tags["variant" .. tostring(i)] then
				timer = timer - math.modf(timer * (3/4) * (i/10))
				green = i/10
				break
			end
		end
	end]]
	local newsentry = sentry:new(x + 4, y + 4, green, timerstart)
	newsentry.tags = tags
	if newsentry.tags["silhouette"] then
		newsentry.tags["silhouette"] = false --Disable automatic draw behavior.
		newsentry.silhouette = true
	end
	return newsentry
end

function sentry:firebullet()
	if self.level.camerafocus and self.level.camerafocus.alive then
		local focus = self.level.camerafocus
		local x1 = self.x + math.floor(self.width / 2)
		local y1 = self.y + math.floor(self.height / 2)
		local x2 = focus.x + math.floor(focus.width / 2)
		local y2 = focus.y + math.floor(focus.height / 2)
		
		local hdiff = x2 - x1 --may be negative but this doesn't matter since it's getting squared
		local vdiff = y2 - y1
		
		local distance = math.sqrt(hdiff^2 + vdiff^2)
		
		if distance <= self.minimumdistance then return end --don't fire if too close
		local hmom = (hdiff / distance) * self.bulletspeed
		local vmom = (vdiff / distance) * self.bulletspeed
		
		local b = bullet:new(self.x + 1, self.y + 1, hmom, vmom, self.green)
		b.level = self.level
		b.tags = self.tags
		if self.silhouette then b.silhouette = true end
		table.insert(self.level.objects, b)
		self.level:updateobjmap(b)
		
		self.level:incAmt("sentrybullets")
	end
end

function sentry:update()
	--[[
	self.tooclose = false
	local focus = self.level.camerafocus
	if focus and focus.alive then
		local mybox_top = self.y - 16
		local mybox_left = self.x - 16
		local mybox_bottom = self.y + self.height + 16
		local mybox_right = self.x + self.width + 16
	end]]
	
	self.timer = self.timer - 1
	if self.timer <= 0 then
		self:firebullet()
		self.timer = self.timerstart + self.timer
	end
end

function sentry:draw()
	if self.silhouette then
		love.graphics.draw(graphics.load("sentry"), sentry.quads.base[1], self.x, self.y)
	else
		local r,g,b,a = love.graphics.getColor()
		--if r ~= 1 or g ~= self.green or b ~= 
		love.graphics.setColor(1 * r, self.green * g, 0, 1)
		
		local frameamt = #sentry.quads.base
		local frame = ((frameamt - math.floor(self.timer / 2)) % frameamt) + 1
		if frame < 1 then frame = 1 end
		if frame > frameamt then frame = frameamt end
		
		love.graphics.draw(graphics.load("sentry"), sentry.quads.base[frame], self.x, self.y)
		love.graphics.setColor(r,g,b,a)
		love.graphics.draw(graphics.load("sentry"), sentry.quads.shine[frame], self.x, self.y)
	end
end

return sentry