switchpad = object:new()

local img = graphics.load("switchstuff")

switchpad.quads = {
	["red_big_left"] = quad(0, 0, 16, 16, img),
	["red_big_right"] = quad(16, 0, 16, 16, img),
	["red_small"] = quad(32, 0, 16, 16, img),
	["blue_big_left"] = quad(0, 16, 16, 16, img),
	["blue_big_right"] = quad(16, 16, 16, 16, img),
	["blue_small"] = quad(32, 16, 16, 16, img),
	
	["s1_red_big_left"] = quad(0, 64, 16, 16, img),
	["s1_red_big_right"] = quad(16, 64, 16, 16, img),
	["s1_red_small"] = quad(32, 64, 16, 16, img),
	["s2_red_big_left"] = quad(0, 80, 16, 16, img),
	["s2_red_big_right"] = quad(16, 80, 16, 16, img),
	["s2_red_small"] = quad(32, 80, 16, 16, img),
	
	["s1_blue_big_left"] = quad(0, 96, 16, 16, img),
	["s1_blue_big_right"] = quad(16, 96, 16, 16, img),
	["s1_blue_small"] = quad(32, 96, 16, 16, img),
	["s2_blue_big_left"] = quad(0, 112, 16, 16, img),
	["s2_blue_big_right"] = quad(16, 112, 16, 16, img),
	["s2_blue_small"] = quad(32, 112, 16, 16, img),
	
	["editor"] = quad(0, 128, 16, 16, img),
	
	["cyan_big_left"] = quad(64 + 0, 0, 16, 16, img),
	["cyan_big_right"] = quad(64 + 16, 0, 16, 16, img),
	["cyan_small"] = quad(64 + 32, 0, 16, 16, img),
	["yellow_big_left"] = quad(64 + 0, 16, 16, 16, img),
	["yellow_big_right"] = quad(64 + 16, 16, 16, 16, img),
	["yellow_small"] = quad(64 + 32, 16, 16, 16, img),
	
	["s1_cyan_big_left"] = quad(64 + 0, 64, 16, 16, img),
	["s1_cyan_big_right"] = quad(64 + 16, 64, 16, 16, img),
	["s1_cyan_small"] = quad(64 + 32, 64, 16, 16, img),
	["s2_cyan_big_left"] = quad(64 + 0, 80, 16, 16, img),
	["s2_cyan_big_right"] = quad(64 + 16, 80, 16, 16, img),
	["s2_cyan_small"] = quad(64 + 32, 80, 16, 16, img),
	
	["s1_yellow_big_left"] = quad(64 + 0, 96, 16, 16, img),
	["s1_yellow_big_right"] = quad(64 + 16, 96, 16, 16, img),
	["s1_yellow_small"] = quad(64 + 32, 96, 16, 16, img),
	["s2_yellow_big_left"] = quad(64 + 0, 112, 16, 16, img),
	["s2_yellow_big_right"] = quad(64 + 16, 112, 16, 16, img),
	["s2_yellow_small"] = quad(64 + 32, 112, 16, 16, img),
	
	["editor_secondary"] = quad(64 + 0, 128, 16, 16, img)
}

function switchpad.editordraw(x, y, options)
	local myquad = "editor"
	if options["secondary"] then myquad = "editor_secondary" end
	if options["singlet"] and options["singlet"] ~= "none" then
		if not options["secondary"] then
			myquad = "s1_" .. options["singlet"] .. "_small"
		else
			myquad = "s2_" .. options["singlet"] .. "_small"
		end
	end
	if not switchpad.quads[myquad] then
		print(myquad)
		for k,v in pairs(switchpad.quads) do print(k, myquad) end
	end
	love.graphics.draw(graphics.load("switchstuff"), switchpad.quads[myquad], x, y)
end

function switchpad:init(x, y, ext, singlet, otherpart, secondary)
	self.type = "switchpad"
	self.x = x
	self.y = y
	self.hmom = 0
	self.ymom = 0
	self.width = tilesize
	self.height = tilesize
	self.solid = true
	self.ext = ext
	self.singlet = singlet
	self.otherpart = otherpart
	self.secondary = secondary
end

function switchpad:setup(x, y, options, level)
	local singlet = options["singlet"] or "none"
	local secondary = not not options["secondary"]
	if singlet == "cyan" or singlet == "yellow" then secondary = true end
	local ext = false
	local otherpart = false
	local lastobj = level.objects[#level.objects]
	if lastobj and lastobj["type"] == "switchpad" and lastobj.y == y and lastobj.x == x - tilesize and lastobj.ext == false and lastobj.singlet == singlet then --i type lastobj["type"] because if i type lastobj.type npp highlights type as though it were the function and. man. i just don't want to deal with that right now even though it would take 2 seconds to find out whether lastobj.type would cause some error. also it looks ugly
		ext = "right"
		otherpart = lastobj
		lastobj.ext = "left"
	end
	local newswitchpad = switchpad:new(x, y, ext, singlet, otherpart, secondary)
	if otherpart then otherpart.otherpart = newswitchpad end --link the last object to this one
	return newswitchpad
end

function switchpad:onBoom()
	if self.otherpart then
		self.otherpart.ext = false
	end
	self.solid = false
	self.irrelevant = true
end

function switchpad:update()
end

function switchpad:onDuck(ducker)
	if self.singlet == "none" then
		if not self.secondary then
			if not game.alreadydone_temp["redblueswitch"] then
				game.alreadydone_temp["redblueswitch"] = true
				game.switchpadstate = not game.switchpadstate
				game.solid_colors["red"] = not game.solid_colors["red"]
				game.solid_colors["blue"] = not game.solid_colors["blue"]
				self:playsfx("switchblocks")
			end
		else
			game.alreadydone_temp["cyanyellowswitch"] = true
			game.switchpad2state = not game.switchpad2state
			game.solid_colors["cyan"] = not game.solid_colors["cyan"]
			game.solid_colors["yellow"] = not game.solid_colors["yellow"]
			self:playsfx("switchblocks")
		end
	else
		if not game.alreadydone_temp[self.singlet .. "switch"] then
			game.alreadydone_temp[self.singlet .. "switch"] = true
			game.solid_colors[self.singlet] = not game.solid_colors[self.singlet]
			self:playsfx("switchblocks")
		end
	end
end

function switchpad:draw()
	local myquad_singlet = ""
	if self.singlet and self.singlet ~= "none" then
		local s1 = false
		if game.solid_colors[self.singlet] then
			s1 = true
		end
		if not game.solid_colors_default[self.singlet] then s1 = not s1 end
		if s1 then
			myquad_singlet = "s1_"
		else
			myquad_singlet = "s2_"
		end
	end
	local myquad_pref
	if not self.secondary then
		myquad_pref = "blue_"
		if ((self.singlet == "none" or not self.singlet) and game.switchpadstate) or self.singlet == "red" then myquad_pref = "red_" end
	else
		local myquad_pref = "yellow_"
		if ((self.singlet == "none" or not self.singlet) and game.switchpad2state) or self.singlet == "cyan" then myquad_pref = "cyan_" end
	end
	local myquad_suff = "small"
	if self.ext then myquad_suff = "big_" .. self.ext end
	love.graphics.draw(graphics.load("switchstuff"), switchpad.quads[myquad_singlet .. myquad_pref .. myquad_suff], self.x, self.y)
end

return switchpad