tele = object:new()

tele.automask = true
tele.maskimagename = "teles"

local img = graphics.load("teles")

tele.quads = {
	["red"] = love.graphics.newQuad(0, 16, 16, 16, img),
	["green"] = love.graphics.newQuad(16, 16, 16, 16, img),
	["blue"] = love.graphics.newQuad(32, 16, 16, 16, img)
}

function tele.editordraw(x, y, options)
	love.graphics.draw(graphics.load("teles"), tele.quads[options.kind], x, y)
end

function tele:init(x, y, kind, deceit)
	self.type = "tele"
	self.x = x
	self.y = y
	self.hmom = 0
	self.ymom = 0
	self.width = tilesize
	self.height = tilesize
	self.solid = false
	self.kind = kind
	self.deceit = deceit
	self.revealed = false
end

function tele:late_init()
	if self.deceit then return end
	local targets = self.level.temps["tele_" .. self.kind]
	local closest
	local closest_dist
	if targets then
		for i,v in ipairs(targets) do
			if (not closest) then
				closest = v
			else
				local xdiff = self.x - v.x
				local ydiff = self.y - v.y
				local dist = (xdiff^2) + (ydiff^2)
				--Little shortcut: since we don't care about the actual distance, just its value relative to other distances, we don't need to bother taking the square root.
				--If a < b, then sqrt(a) < sqrt(b).
				--For positive a and b, anyway. Real a and b are assumed because this  is Jumper 4 Real.
				if not closest_dist then
					local xdiff = self.x - closest.x
					local ydiff = self.y - closest.y
					closest_dist = (xdiff^2) + (ydiff^2)
				end
				if dist < closest_dist then
					closest = v
					closest_dist = dist
				end
			end
		end
	end
	self.target = closest
end

function tele:setup(x, y, options, level, edge, tags)
	local deceit = false
	if tags["fake"] then
		tags["fake"] = false
		deceit = true
	end
	return tele:new(x, y, options.kind, deceit)
end

function tele:update()
end

function tele:onOverlap(obj)
	if self.target then
		if self.target.irrelevant then 
			self.target = nil
			self.revealed = true
			return
		end
		local target = self.target
		obj.x = target.x + math.floor((target.width - obj.width) / 2)
		obj.y = target.y + math.floor((target.height - obj.height) / 2)
		obj.x_clamped = math.floor(obj.x + 0.5)
		obj.y_clamped = math.floor(obj.y + 0.5)
		self.level.updateobjmap_queued[obj] = true
		self:playsfx("teleport")
	else
		if not self.revealed then
			self:playsfx("teleporter reveal")
		end
		self.revealed = true
	end
end

function tele:draw()
	if not self.revealed then
		love.graphics.draw(graphics.load("teles"), self.quads[self.kind], self.x, self.y)
	else
		local r,g,b,a = love.graphics.getColor()
		love.graphics.setColor(r * 0.5, g * 0.5, b * 0.5, a)
		love.graphics.draw(graphics.load("teles"), self.quads[self.kind], self.x, self.y)
		love.graphics.setColor(r,g,b,a)
	end
end

return tele