walkyspike = mob:new()

local img = graphics.load("walkyspike")

walkyspike.quads = {
	[1] = quad(0,  0, 16, 16, img),
	[2] = quad(16, 0, 16, 16, img),
	[3] = quad(32, 0, 16, 16, img),
	[4] = quad(48, 0, 16, 16, img),
	["cw"] = quad(32, 16, 16, 16, img),
	["ccw"] = quad(48, 16, 16, 16, img),
}

function walkyspike.editordraw(x, y, options)
	love.graphics.draw(graphics.load("walkyspike"), walkyspike.quads[1], x, y)
	if options["turndir"] == "ccw" then love.graphics.draw(graphics.load("walkyspike"), walkyspike.quads["ccw"], x, y)
	else love.graphics.draw(graphics.load("walkyspike"), walkyspike.quads["cw"], x, y) end
end

function walkyspike:init(x, y, hmom, vmom, ccw)
	self.type = "walkyspike"
	self.drawlate = true
	self.x = x
	self.y = y
	self.y_clamped = y
	self.x_clamped = x
	self.hmom = hmom
	self.vmom = vmom
	self.width = tilesize
	self.height = tilesize
	self.solid = false
	self.deathly = true
	self.ccw = ccw
	self.hit_wall = false
	self.no_latch = false
	self.min_run_time = 8
	self.time_since_latch = self.min_run_time
	self.dontkillfollowers = true
	self.collidewith = {["reverser"] = true}
	self.collisioncondition = function(self, collidee) --different than collidewith because this only gets called to REVERT colliding with a tile; collidewith is used to denote this should collide with a particular tile that isn't solid. jank
	--this refers to what the WALKYSPIKE will collide with, not whether other things will collide with the walkyspike
		if collidee.player then return false end
		return true
	end
	self.frame = 1
	self.animtimer = 6
	self.animtimer_start = 6
end

function walkyspike:setup(x, y, options, level, edge, tags)
	local hmom = 0
	local vmom = 0
	local ccw = false
	if options["turndir"] == "ccw" then ccw = true end
	if tags["facerandom"] then tags[pick{"faceup","faceleft","facedown","faceright"}] = true end
	if tags["faceup"] then vmom = -2 end
	if tags["faceleft"] then hmom = -2 end
	if tags["facedown"] then vmom = 2 end
	if tags["faceright"] then hmom = 2 end
	if hmom ~= 0 and vmom ~= 0 then vmom = 0 end
	if hmom == 0 and vmom == 0 then hmom = 2 end
	return walkyspike:new(x, y, hmom, vmom, ccw)
end

walkyspike.collisionfunctions = {
	horizontal = function(self, latching)
		local oldhmom = self.hmom
		--if not latching (e.g. if we can hit a wall and turn) then look a little bit ahead in case we end up basically flush with a wall
		--even if we ARE latching 1/16 still gets added to the momentum, just outside this function, but i consider this slightly different because with latching we just want to see if we're flush with something, but with non-latching we want to move but also accept being flush with something post-movement as collision... which means adding for flushness in either case. this is stupid.
		if not latching then
			local sign = 1
			if self.hmom < 0 then sign = -1 end
			self.hmom = self.hmom + ((1/16) * sign)
			--there is probably a much less stupid way to do this. i considered dividing the momentum by math.abs(the momentum) and assuming the answer is either 1 or -1 but that's, you know, division. and also breaks for 0
		end
		local walled, position, walledby = mobtools.doCollisionScan("horizontal", self)
		self.hmom = oldhmom
		if position == nil then
			if latching then
				self.no_latch = true
			else
				self.x = self.x + self.hmom
				self.x_clamped = math.floor(self.x + .5)
			end
		else
			local bounce = false
			for _,collider in ipairs(walledby) do
				if collider.onCollide then
					collider:onCollide(self, walled)
				end
				if (not latching) and (collider.bounce or collider.reverser) then
					bounce = true
				end
			end
			if (not latching) then
				if bounce then
					self.hmom = -1 * self.hmom
					self.ccw = not self.ccw
				else
					self.hit_wall = true
				end
			else
				self.time_since_latch = 0
			end
			self.x = position
			if self.x ~= self.x_clamped then self.x_clamped = math.floor(self.x + .5) end
		end
	end,
	vertical = function(self, latching)
		local oldvmom = self.vmom
		if not latching then
			local sign = 1
			if self.vmom < 0 then sign = -1 end
			self.vmom = self.vmom + ((1/16) * sign)
		end
		local verticaled, position, verticaledby = mobtools.doCollisionScan("vertical", self)
		self.vmom = oldvmom
		if position == nil then
			if latching then
				self.no_latch = true
			else
				self.y = self.y + self.vmom
				self.y_clamped = math.floor(self.y + .5)
			end
		else
			local bounce = false
			for _,collider in ipairs(verticaledby) do
				if collider.onCollide then
					collider:onCollide(self, verticaled)
				end
				if (not latching) and (collider.bounce or collider.reverser) then
					bounce = true
				end
			end
			self.y = position
			if (not latching) then
				if bounce then
					self.vmom = -1 * self.vmom
					self.ccw = not self.ccw
				else
					self.hit_wall = true
				end
			else
				self.time_since_latch = 0
			end
			if self.y ~= self.y_clamped then self.y_clamped = math.floor(self.y + .5) end
		end
	end,
}

function walkyspike:getlatchdir()
	local latchdir = 1
	if self.hmom < 0 or self.vmom > 0 then latchdir = -1 end
	if self.ccw then latchdir = latchdir * -1 end
	return latchdir
end

function walkyspike:turn(hitwall)
	if hitwall then self.time_since_latch = 0
	else self.time_since_latch = self.min_run_time end
	local vmom = self.vmom
	local hmom = self.hmom
	local dir = self:getlatchdir()
	if hitwall then dir = dir * -1 end
	self.vmom = math.abs(hmom) * dir
	self.hmom = math.abs(vmom) * dir
end

function walkyspike:update()
	if self.animtimer > 0 then
		self.animtimer = self.animtimer - 1
	end
	if self.animtimer == 0 then
		if self.ccw then
			self.frame = self.frame - 1
			if self.frame <= 0 then self.frame = 4 end
		else
			self.frame = self.frame + 1
			if self.frame > 4 then self.frame = 1 end
		end
		self.animtimer = self.animtimer_start
	end
	self.time_since_latch = self.time_since_latch - 1
	if self.time_since_latch < 0 then self.time_since_latch = 0 end
	
	self.hit_wall = false
	self.no_latch = false
	local primaryaxis = "horizontal"
	local latchaxis = "vertical"
	if self.vmom ~= 0 then latchaxis, primaryaxis = primaryaxis, latchaxis end
	self.collisionfunctions[primaryaxis](self, false)
	--setting both hmom and vmom temporarily because i don't have time to figure out which one and i think it would be too stupid to do something like self[string.sub(latchaxis, 1) .. "mom"] = whatever
	if self.time_since_latch == 0 then
		local oldhmom = self.hmom
		local oldvmom = self.vmom
		local latchdir = self:getlatchdir()
		self.hmom = (1/16) * latchdir
		self.vmom = (1/16) * latchdir
		self.collisionfunctions[latchaxis](self, true)
		self.hmom = oldhmom
		self.vmom = oldvmom
		if self.no_latch then
			self:turn(false)
		end
	end
	if self.hit_wall and not self.no_latch then
		self:turn(true)
	end
	
	self.overlaps = mobtools.doOverlapScan(self)
	for _,obj in ipairs(self.overlaps) do
		if obj.onOverlap then
			obj:onOverlap(self)
		end
	end
	
	if self.x + self.width < 0 or self.y + self.height < 0 or self.y > self.level.height or self.x > self.level.width then
		self.solid = false; self.irrelevant = true
	end
end

function walkyspike:draw()
	love.graphics.draw(graphics.load("walkyspike"), walkyspike.quads[self.frame], self.x_clamped, self.y_clamped)
end

return walkyspike