win = object:new()

function win.editordraw(x, y, options)
	local myquad = "normal"
	if options.alt then myquad = "alt"
	elseif options["end"] then myquad = "end" end
	love.graphics.draw(graphics.load("exits"), win.quads[myquad], x, y)
end

local img = graphics.load("exits")

win.quads = {
	["normal"] = love.graphics.newQuad(0, 0, 16, 16, img),
	["alt"] = love.graphics.newQuad(16, 0, 16, 16, img),
	["end"] = love.graphics.newQuad(32, 0, 16, 16, img),
}

function win:init(x, y, edge, id, saves, graphic)
	self.type = "win"
	self.x = x
	self.y = y
	self.width = tilesize
	self.height = tilesize
	self.edge = edge
	self.exitid = id
	self.passive = true
	self.consec_x = false --consec_x is the position of the leftmost win object in a row of consecutive win objects containing this object
	self.consec_y = false --consec_y is the position of the BOTTOMMOST win object in a column of consecutive win objects containing this object. why bottommost instead of topmost? because consec_x and consec_y are used for the "preserve win offset" tag for ogmos that makes their x/y position the same when they enter a level relative to where they exited the level, and it's more natural to place ogmo on the floor than on the ceiling, but it's also more natural to place ogmo on the left wall [of a top/bottom-edge screen exit] than on the right wall. explained that pretty badly but well there's my comment hope you enjoyed
	self.saves = saves
	self.graphic = graphic
end

function win:setup(x, y, options, level, edge, tags)
	local id
	if next(tags) then
		for i=0, 9 do
			if tags["variant" .. i] then
				id = i break;
			end
		end
	end
	if not id then
		for i,v in ipairs(consts.directions) do
			if tags["face" .. v] then
				id = v
				break
			end
		end
		if not id then
			if edge ~= "none" then
				id = edge
			else id = 1 end --may be nil, in which case void occurs
		end
	end
	local saves = true
	if tags["nosave"] or options["alt"] then saves = false end
	local graphic = "normal"
	if options["alt"] then graphic = "alt"
	elseif options["end"] then graphic = "end" end
	if options["end"] then id = 1 end
	return win:new(x, y, edge, id, saves, graphic)
end

function win:set_consec_xs()
	if self.consec_x then return end
	local row_wins = {}
	for _,obj in ipairs(self.level.objects) do
		if obj.type == "win" and obj.y == self.y and (not obj.irrelevant) and (not obj.consec_x) then
			row_wins[#row_wins + 1] = obj
		end
	end
	table.sort(row_wins, function(a,b) return a.x < b.x end)
	local row_wins_consec = {self}
	local leftmost_x = self.x
	local rightmost_x = self.x
	local my_index = nil
	
	for i=#row_wins, 1, -1 do --look for wins left of this
		if row_wins[i] == self then my_index = i
		elseif row_wins[i].x == leftmost_x - tilesize then
			leftmost_x = row_wins[i].x
			row_wins_consec[#row_wins_consec + 1] = row_wins[i]
		elseif row_wins[i].x < leftmost_x then break end
	end
	
	for i=my_index, #row_wins do --look for wins right of this
		if row_wins[i] == self then --do nothing
		elseif row_wins[i].x == rightmost_x + tilesize then
			rightmost_x = row_wins[i].x
			row_wins_consec[#row_wins_consec + 1] = row_wins[i]
		elseif row_wins[i].x > rightmost_x then break end
	end
	
	for _,obj in ipairs(row_wins_consec) do
		obj.consec_x = leftmost_x
	end
	--the above for loop sets the consec_x of this win object and all horizontally consecutive win objects
	--this can be optimized a bit. if we set each iterated object's consec_x to leftmost_x in the "look for wins right of this" loop instead of adding them to row_wins_consec but leave the iteration through row_wins_consec the same, then we don't visit the right items twice. of course, in practice, nobody cares.
end

function win:set_consec_ys()
	if self.consec_y then return end
	local row_wins = {}
	for _,obj in ipairs(self.level.objects) do
		if obj.type == "win" and obj.x == self.x and (not obj.irrelevant) and (not obj.consec_y) then
			row_wins[#row_wins + 1] = obj
		end
	end
	table.sort(row_wins, function(a,b) return a.y < b.y end)
	local row_wins_consec = {self}
	local topmost_y = self.y
	local bottommost_y = self.y
	local my_index = nil
	
	for i=#row_wins, 1, -1 do --look for wins above this
		if row_wins[i] == self then my_index = i
		elseif row_wins[i].y == topmost_y - tilesize then
			topmost_y = row_wins[i].y
			row_wins_consec[#row_wins_consec + 1] = row_wins[i]
		elseif row_wins[i].y < topmost_y then break end
	end
	
	for i=my_index, #row_wins do --look for wins below this
		if row_wins[i] == self then --do nothing
		elseif row_wins[i].y == bottommost_y + tilesize then
			bottommost_y = row_wins[i].y
			row_wins_consec[#row_wins_consec + 1] = row_wins[i]
		elseif row_wins[i].y > bottommost_y then break end
	end
	
	for _,obj in ipairs(row_wins_consec) do
		obj.consec_y = bottommost_y
	end
end

function win:update()
	if not self.consec_x then self:set_consec_xs() end
	if not self.consec_y then self:set_consec_ys() end
	local winners = {}
	--maybe players should handle this to reduce the amount of objects calling dooverlapscan. would be boilerplatey if more types of players ever come up though. maybe something in mob?
	if not self.edge then winners = mobtools.doOverlapScan(self, true)
	else winners = mobtools.doPastEdgeScan(self, self.edge) end
	--note that it's a bit weird to use pastedgescan for edge wins since most edge wins are placed on the edge of the level (hm!) so they only check one objmap cell, meaning they're equivalent to dooverlapscan... unless the win has an edge tag but is placed not at the edge of the level. frankly pretty silly and i don't see much use for it. i wrote this code to envision what it would be like if wins didn't use pastedgescan but i don't feel like testing it rn:
	--[[winners = mobtools.doOverlapScan(self, true)
	if self.edge then
		local edgewinners = {}
		for i,v in ipairs(winners) do
			if mobtools.doPastEdgeCheck(self, v, self.edge) then
				table.insert(edgewinners, v)
			end
		end
		winners = edgewinners
	end]]
	for _,obj in ipairs(winners) do
		if obj.player and (obj.alive) and (not obj.ghost) and (not self.docutsceneinit) and (not obj.cutscenemovement) then --checking for not self.docutsceneinit is actually unnecessary since player no longer starts outside of the level and thus potentially past a win's edge
			local xoff = nil
			if self.edge == "up" or self.edge == "down" then xoff = obj.x - self.consec_x end
			local yoff = nil --yoff!
			if self.edge == "left" or self.edge == "right" then yoff = obj.y - self.consec_y end
			if obj.ducking then yoff = yoff - 3 end
			local exit = {
				--number = self.exitno or 1,
				id = self.exitid,
				careening = obj.careening,
				jumpsleft = obj.jumps,
				hmom = obj.hmom,
				vmom = obj.vmom,
				xoffset = xoff,
				yoffset = yoff,
				gray = (obj.starstate and obj.starstate.gray),
				purple = (obj.starstate and obj.starstate.purple),
				icestar = (obj.starstate and obj.starstate.ice),
			}
			--if game.editormode then
			if false then
				exit.id = 0
				editor.entrance = exit
			end
			game.win(exit, self.level, self.saves, self.edge)
			break
		end
	end
end

function win:draw()
	if settings.seetheunseeable then
		love.graphics.draw(graphics.load("exits"), self.quads[self.graphic], self.x, self.y)
	end
	--[[
	local r, g, b, a = love.graphics.getColor()
	love.graphics.setColor(0, 0, 0, 1)
	if     self.edge == "down"  then love.graphics.print("v", self.x, self.y)
	elseif self.edge == "right" then love.graphics.print(">", self.x, self.y)
	elseif self.edge == "left"  then love.graphics.print("<", self.x, self.y)
	elseif self.edge == "up"    then love.graphics.print("^", self.x, self.y) end
	love.graphics.setColor(r, g, b, a)
	]]
end

return win