roommap = class:new()

roommap.tooltip = "this map shows nearby rooms. click on a room to go to it. press M to toggle between small and large view."

roommap.cellsize = 12
local c = roommap.cellsize
local img = graphics.load("ui/mapcells")
roommap.quads = {
	["saved"] = love.graphics.newQuad(0, 0, c, c, img),
	["unsaved"] = love.graphics.newQuad(c, 0, c, c, img),
	["empty"] = love.graphics.newQuad(c*2, 0, c, c, img),
	["selected"] = love.graphics.newQuad(0, c, c, c, img),
	["delete"] = love.graphics.newQuad(c*3, 0, c, c, img),
	["clipboard"] = love.graphics.newQuad(c, c, c, c, img),
}

function roommap:init(rel_x, rel_y, colamt, rowamt, relativeto)
	self.rel_x = rel_x
	self.rel_y = rel_y
	self.colamt = colamt
	self.rowamt = rowamt
	self.coloffset = math.floor(colamt / 2)
	self.rowoffset = math.floor(rowamt / 2)
	self.relativeto = relativeto
	
	self.width = self.colamt * roommap.cellsize
	self.height = self.rowamt * roommap.cellsize
end

function roommap:setup(rel_x, rel_y, colamt, rowamt, relativeto, container_w, container_h)
	if (not relativeto) or relativeto == "none" then relativeto = "topleft" end
	local newroommap = roommap:new(rel_x, rel_y, colamt, rowamt, relativeto)
	newroommap:updateposfordimensions(container_w or love.graphics.getWidth(), container_h or love.graphics.getHeight())
	return newroommap
end

function roommap:update()
end

function roommap:updateposfordimensions(w, h)
	local x = self.rel_x
	local y = self.rel_y
	if self.relativeto ~= "topleft" then
		if self.relativeto == "bottomleft" or self.relativeto == "bottomright" then
			y = h - self.rel_y
		end
		if self.relativeto == "topright" or self.relativeto == "bottomright" then
			x = w - self.rel_x
		end
	end
	self.x = x
	self.y = y
end

function roommap:mousepressed(x, y)
	local result_x = math.floor(x / roommap.cellsize)
	local result_y = math.floor(y / roommap.cellsize)
	editor.gotoRoom(result_x + (editor.roomX - self.coloffset), result_y + (editor.roomY - self.rowoffset))
end

function roommap:draw()
	local r,g,b,a = love.graphics.getColor()
	love.graphics.setColor(consts.gray)
	love.graphics.rectangle("fill", self.x, self.y, self.width, self.height)
	love.graphics.setColor(r,g,b,a)
	local img = graphics.load("ui/mapcells")
	local y_rel = 0
	--		-32768
	--		32767
	for y1 = (editor.roomY - self.rowoffset), ((editor.roomY - self.rowoffset) + self.rowamt - 1) do
		local row = editor.extantrooms[y1]
		local x_rel = 0
		if y1 >= -32768 and y1 <= 32767 then
			for x1 = (editor.roomX - self.coloffset), ((editor.roomX - self.coloffset) + self.colamt - 1) do
				local quad = "empty"
				if x1 >= -32768 and x1 <= 32767 then
					if row then
						if (editor.unsaved and (x1 == editor.roomX and y1 == editor.roomY)) or (editor.unsavedrooms[y1] and editor.unsavedrooms[y1][x1]) then quad = "unsaved"
						elseif editor.extantrooms[y1][x1] then quad = "saved" end
					end
					love.graphics.draw(img, roommap.quads[quad], self.x + (x_rel * roommap.cellsize), self.y + (y_rel * roommap.cellsize))
					if editor.deleterooms[y1] and editor.deleterooms[y1][x1] then
						love.graphics.draw(img, roommap.quads["delete"], self.x + (x_rel * roommap.cellsize), self.y + (y_rel * roommap.cellsize))
					end
					if x1 == editor.roomX and y1 == editor.roomY then
						love.graphics.draw(img, roommap.quads["selected"], self.x + (x_rel * roommap.cellsize), self.y + (y_rel * roommap.cellsize))
					end
					local clipboard = editor.clipboard
					if editor.currentlevelset == clipboard.levelset and x1 == clipboard.x and y1 == clipboard.y and not (clipboard.object or clipboard.empty) then
						love.graphics.draw(img, roommap.quads["clipboard"], self.x + (x_rel * roommap.cellsize), self.y + (y_rel * roommap.cellsize))
					end
				end
				x_rel = x_rel + 1
			end
		end
		y_rel = y_rel + 1
	end
end

--return roommap