statusbar = {}

statusbar.img = graphics.load("ui/hud")
local img = statusbar.img
statusbar.quads = {
	["arrow"] = quad(0, 0, 8, 8, img),
	["noarrow"] = quad(8, 0, 8, 8, img),
	["bluegem"] = quad(16, 0, 8, 8, img),
	["redgem"] = quad(24, 0, 8, 8, img),
	["graystar"] = quad(32, 0, 8, 8, img),
	["icestar"] = quad(40, 0, 8, 8, img),
	["purplestar"] = quad(48, 0, 8, 8, img),
	["out"] = quad(56, 0, 8, 8, img),
	["nothing"] = quad(64, 0, 8, 8, img),
	["blue0"] = quad(72, 8, 8, 8, img),
	["blue+"] = quad(80, 8, 8, 8, img),
	["red0"] = quad(72, 16, 8, 8, img),
	["red+"] = quad(80, 16, 8, 8, img),
}
for i=1,9 do
	statusbar.quads["blue" .. tostring(i)] = quad((i - 1) * 8, 8, 8, 8, img)
	statusbar.quads["red" .. tostring(i)] = quad((i - 1) * 8, 16, 8, 8, img)
end
statusbar.iconsize = 8

statusbar.w = 1
statusbar.h = 1

statusbar.maxwidth = 8

statusbar.active = true
statusbar.out = false
statusbar.nothing = false

statusbar.entries = {
	stars = {nil, nil, nil, false},
	gems1 = {nil, nil, false},
	gems2 = {nil, nil, false}
}
statusbar.entries_order = {"stars", "gems1", "gems2"}

statusbar.update_cases = {
	["jump"] = function(focus)
		local stars = statusbar.entries.stars
		if focus.jumps then
			local noinsert = (stars[1] ~= nil)
			if focus.starstate and focus.starstate.gray then
				stars[1] = "graystar"
			elseif focus.jumps > 0 then
				stars[1] = "arrow"
			else
				stars[1] = "noarrow"
			end
			return noinsert --skip re-measuring if this replaced something non-nil
		else
			stars[1] = nil
		end
	end,
	["stars"] = function(focus)
		local stars = statusbar.entries.stars
		stars[2] = nil
		stars[3] = nil
		if focus.starstate then
			stars[1] = (focus.starstate.gray and "graystar") or stars[1]
			stars[2] = (focus.starstate.purple and "purplestar") or nil
			stars[3] = (focus.starstate.ice and "icestar") or nil
		end
	end,
	["gems"] = function(focus)
		local red = game.unsavedgems_redcount
		local blue = game.unsavedgems_bluecount
		local gems1 = statusbar.entries.gems1
		local gems2 = statusbar.entries.gems2
		gems1[1] = nil
		gems1[2] = nil
		gems2[1] = nil
		gems2[2] = nil
		if red == 0 and blue == 0 then
			return
		elseif red < 2 and blue < 2 then
			if blue > 0 then
				gems1[1] = "bluegem"
			end
			if red > 0 then
				gems1[2] = "redgem"
			end
			return
		else
			if blue > 0 then
				gems1[1] = "bluegem"
				if blue > 1 then
					local symbol = tostring(blue)
					if #symbol > 1 then symbol = "+" end
					local icon = "blue" .. symbol
					if not statusbar.quads[icon] then icon = "out" end
					gems1[2] = icon
				end
			end
			if red > 0 then
				gems2[1] = "redgem"
				if red > 1 then
					local symbol = tostring(red)
					if #symbol > 1 then symbol = "+" end
					local icon = "red" .. symbol
					if not statusbar.quads[icon] then icon = "out" end
					gems2[2] = icon
				end
			end
		end
	end
}
statusbar.update_cases_order = sortedstringfields(statusbar.update_cases, "statusbar update cases")

statusbar.skipmeasure = false
function statusbar.measure()
	if statusbar.out then
		statusbar.w = 1
		statusbar.h = 1
		return
	end
	
	local w = 0
	local h = 0
	for _,v in pairs(statusbar.entries) do
		local len = 0
		for _,_ in pairs(v) do
			len = len + 1
		end
		if len > w then w = len end
		if len > 0 then h = h + 1 end
	end
	if w == 0 and h == 0 then
		statusbar.w = 8
		statusbar.h = 8
	end
	statusbar.w = w
	statusbar.h = h
end

function statusbar.nofocus()
	statusbar.out = true
	statusbar.measure()
end

function statusbar.update(case, lvl)
	if not statusbar.active then return end
	--if statemachine.currentstate ~= game then error("asked to update statusbar, but not ingame", 2) end
	local focus = (lvl or game.activelevels[1]).camerafocus
	if not focus then
		statusbar.out = true
		statusbar.measure()
		print("statusbar: no level focus")
		return
	end
	statusbar.out = false
	if case then
		local skipremeasure = statusbar.update_cases[case](focus)
		if not skipremeasure then
			statusbar.measure()
		end
	else
		for i,v in ipairs(statusbar.update_cases_order) do
			statusbar.update_cases[v](focus)
		end
		statusbar.measure()
	end
end

function statusbar.draw()
	if not statusbar.active then return end
	local img = graphics.load("ui/hud")
	local r,g,b,a = love.graphics.getColor()
	love.graphics.setColor(0,0,0,0.5)
	local rect_w = statusbar.w * statusbar.iconsize
	local rect_h = statusbar.h * statusbar.iconsize
	love.graphics.rectangle("fill", 0, 0, rect_w, rect_h)
	love.graphics.setColor(1,1,1,0.5)
	love.graphics.rectangle("line", -0.5, -0.5, rect_w + 1, rect_h + 1)
	love.graphics.setColor(r,g,b,a)
	if statusbar.out then
		love.graphics.draw(img, statusbar.quads["out"], 0, 0)
		return
	end
	local y = 0
	for i=1, statusbar.h do
		local y_advance = false
		local t = statusbar.entries[statusbar.entries_order[i]]
		local x = 0
		for ii=1, statusbar.w do
			local icon = t[ii]
			if icon then
				love.graphics.draw(img, statusbar.quads[icon], x, y)
				x = x + statusbar.iconsize
				y_advance = true
			end
		end
		if y_advance then
			y = y + 8
		end
	end
end