symboldisplay = {
	scroll = 0,
	scrollmax = 0,
	categories = {"mobs", "objects", "tiles", "j1", "j1e", "j2", "j2e", "jr", "j4r", "ascii"},
	sections = {},
	titlefont = love.graphics.newFont(16),
	docalcwhenbegun = false,
	clickspots = {},
	backtext = love.graphics.newText(love.graphics.getFont()),
	backhovered = false,
	acknowledgeclick = false,
}

symboldisplay.backtext:setf("back", 64, "center")

symboldisplay.categories_set = {}
for i,v in ipairs(symboldisplay.categories) do
	symboldisplay.categories_set[v] = true
end


local function add_to_section(where, what)
	if not symboldisplay.sections[where] then
		symboldisplay.sections[where] = {}
	end
	table.insert(symboldisplay.sections[where], what)
end

function symboldisplay.initsections()
	for i,v in ipairs(levelsymbols_sortedkeys_byorder) do
		if levelsymbols[v].categories and not levelsymbols[v].nevershow then
			for i2,v2 in ipairs(levelsymbols[v].categories) do
				if symboldisplay.categories_set[v2] then
					add_to_section(v2, v)
				end
			end
		end
	end
end

function symboldisplay.calc_clickspots()
	symboldisplay.clickspots = {}
	local y = 16
	local y_tiled = 1
	for _,category in ipairs(symboldisplay.categories) do
		y = y + (16 * 2)
		y_tiled = y_tiled + 2
		local section = symboldisplay.sections[category]
		if section then
			local x = 16
			local x_tiled = 1
			for i,v in ipairs(section) do
				if not symboldisplay.clickspots[y_tiled] then symboldisplay.clickspots[y_tiled] = {} end
				symboldisplay.clickspots[y_tiled][x_tiled] = v
				x = x + 16
				x_tiled = x_tiled + 1
				if section[i + 1] and x + (16 * 2) > love.graphics.getWidth() then
					x = 16
					x_tiled = 1
					y = y + 16
					y_tiled = y_tiled + 1
				end
			end
		end
		y = y + (16 * 2)
		y_tiled = y_tiled + 2
	end
	symboldisplay.scrollmax = y - love.graphics.getHeight()
	if symboldisplay.scrollmax < 0 then symboldisplay.scrollmax = 0 end
	if symboldisplay.scroll > symboldisplay.scrollmax then symboldisplay.scroll = symboldisplay.scrollmax end
end

symboldisplay.initsections()
symboldisplay.calc_clickspots()

function symboldisplay.begin()
	if symboldisplay.docalcwhenbegun then
		symboldisplay.calc_clickspots()
		symboldisplay.docalcwhenbegun = false
	end
end

function symboldisplay.stop()
	symboldisplay.acknowledgeclick = false
end

function symboldisplay.update()
	editor.tooltip = nil
	symboldisplay.backhovered = false
	if love.mouse.getY() > 16 and love.mouse.getY() <= editor.height - 16 then
		local x_tiled = math.floor(love.mouse.getX() / 16)
		local y_tiled = math.floor((love.mouse.getY() + symboldisplay.scroll) / 16)
		if symboldisplay.clickspots[y_tiled] and symboldisplay.clickspots[y_tiled][x_tiled] then
			local symbol = symboldisplay.clickspots[y_tiled][x_tiled]
			editor.tooltip = editor.getSymbolTooltip(levelsymbols[symbol])
			if symboldisplay.acknowledgeclick then
				if love.mouse.isDown(1) then editor.lmbtile = symbol
				elseif love.mouse.isDown(2) then editor.rmbtile = symbol
				elseif love.mouse.isDown(3) then editor.mmbtile = symbol end
			end
		end
	elseif love.mouse.getY() > editor.height - 16 then
		if love.mouse.getX() > (editor.width - 64) then
			symboldisplay.backhovered = true
		end
	end
end

function symboldisplay.mousepressed(x, y, button)
	symboldisplay.acknowledgeclick = true
	if y > (editor.height - 16) and x > (editor.width - 64) then
		statemachine.setstate("editor")
	end
end

function symboldisplay.keypressed(key)
	if key == "`" then
		statemachine.setstate("editor")
	end
end

function symboldisplay.wheelmoved(x, y)
	y = y * 16
	symboldisplay.scroll = symboldisplay.scroll - math.floor(y)
	if symboldisplay.scroll < 0 then symboldisplay.scroll = 0
	elseif symboldisplay.scroll > symboldisplay.scrollmax then symboldisplay.scroll = symboldisplay.scrollmax end
end

function symboldisplay.draw()
	local y = 16
	local r, g, b, a = love.graphics.getColor()
	love.graphics.push()
	love.graphics.translate(0, (-1 * symboldisplay.scroll))
	love.graphics.setColor(0.2, 0.2, 0.2, 1)
	love.graphics.rectangle("fill", 0, y, love.graphics.getWidth(), love.graphics.getHeight() + symboldisplay.scrollmax)
	
	for _,category in ipairs(symboldisplay.categories) do
		love.graphics.setColor(0.333, 0.333, 0.333, 1)
		love.graphics.rectangle("fill", 0, y, love.graphics.getWidth(), 16)
		love.graphics.setColor(r, g, b, a)
		love.graphics.print(category, symboldisplay.titlefont, (16 * 2), y)
		y = y + (16 * 2)
		local section = symboldisplay.sections[category]
		if section then
			local x = 16
			for i,v in ipairs(section) do
				editor.drawSymbol(v, x, y, true, true)
				x = x + 16
				if section[i + 1] and x + (16 * 2) > love.graphics.getWidth() then x = 16; y = y + 16; end
			end
		end
		y = y + 32
	end
	love.graphics.pop()
	
	--love.graphics.print(symboldisplay.scroll)
	love.graphics.setColor(0, 0, 0, 1)
	love.graphics.rectangle("fill", 0, 0, love.graphics.getWidth(), 16)
	love.graphics.rectangle("fill", 0, (editor.height - 16), love.graphics.getWidth(), 16)
	love.graphics.setColor(1, 1, 1, 1)
	love.graphics.print("element select submenu (shortcut: `)")
	
	if symboldisplay.backhovered then
		love.graphics.setColor(0.666, 0.666, 0.666, 1)
	else
		love.graphics.setColor(0.5, 0.5, 0.5, 1)
	end
	love.graphics.rectangle("fill", (editor.width - 64), (editor.height - 16), 64, 16)
	love.graphics.setColor(r, g, b, a)
	love.graphics.draw(symboldisplay.backtext, (editor.width - 64), (editor.height - 16))
	
	if editor.tooltip ~= nil then
		printAsTooltip(editor.tooltip, editor.tooltipScale)
	end
end